/***************************************************************************
                          basicgraph.cpp  - description
                             -------------------
    begin                : Thu May 30 2002
    copyright            : (C) 2002-03 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "basicgraph.h"

#include <cmath>

#include <qcursor.h>
#include <qpixmap.h>
#include <qpainter.h>

#ifdef KDE_APP
#include <kprinter.h>
#else
#include <qprinter.h>
#endif //KDE_APP

#include "polarcoord.h"
#include "rectcoord.h"
#include "graphevent.h"
#include "distancefunction.h"

BasicGraph::BasicGraph( QWidget *parent, const char *name ) : 
	QWidget ( parent, name ),
	xMin(-15), xMax(15), xScale2(2),
	yMin(-15), yMax(15), yScale2(2),
	xScale(1), yScale(1),
	xZoom(1), yZoom(1),
	doingZoomBox(false), snapToGrid(false),
	showAxis(true), showGrid(true), showGridNum(true), showScale(true)
{
	_axisColor = QColor(255,0,0);
	_gridColor = QColor(0,5,150);
	_backgroundColor = QColor(0,0,0);
	_scaleColor = QColor(60,120,0);
	
	axisWidth = 1;
	graphFont = QFont("helvetica",10,QFont::Black);
	scaleFont = QFont("helvetica",8);
	mouseDown = false;

	//rectangular or polar grid mode -- polar not yet implemented
	mode = RECTANGULAR;

	setPaletteBackgroundColor(_backgroundColor);
	setMouseTracking(true);

	buffer = new QPixmap(size());
	buffer->fill(_backgroundColor);
	p = new QPainter();

	updateGraph();
}

BasicGraph::~BasicGraph()
{
	delete buffer;
	delete p;
}

void BasicGraph::updateGraph()
{
	origin.setX( int(-xMin * width() / (xMax - xMin)) );
	origin.setY( int(-yMax * height() / (yMin - yMax)) );
	xScale = width() / (xMax - xMin);
	yScale = height() / (yMin - yMax);

	cancelMathFunctions();

	repaint(false);
}

/*
static double max(double a, double b)
{
	return (a > b) ? a : b;
}
*/

void BasicGraph::drawGrid(QPainter *painter)
{
	painter->save();
	painter->setPen(QPen(_gridColor,1,Qt::DotLine));
	//rectangular, stat, parametric mode
//	if (mode == RECTANGULAR)
//	{
		//x grid left of axis
		for (int i = 0; i < -xMin / xScale2; i++)
		{
			//painter->drawText((int) (origin.x() - i * xScale * xScale2),8,QString("%1").arg(toGraphXCoord((origin.x() - i * xScale * xScale2))));
			painter->drawLine((int) (origin.x() - i * xScale * xScale2), 0, (int) (origin.x() - i * xScale * xScale2), height());
		}
		//x grid right of axis
		for (int i = 1; i < xMax / xScale2; i++)
		{
			painter->drawLine((int) (origin.x() + i * xScale * xScale2), 0, (int) (origin.x() + i * xScale * xScale2), height());
		}
		//y grid below axis
		for (int i = 0; i < -yMin / yScale2; i++)
		{
			painter->drawLine(0, (int) (origin.y() + i * -yScale * yScale2), width(), (int) (origin.y() + i * -yScale * yScale2));
		}
		//y grid above axis
		for (int i = 1; i < yMax / yScale2; i++)
		{
			painter->drawLine(0, (int) (origin.y() - i * -yScale * yScale2), width(), (int) (origin.y() - i * -yScale * yScale2));
		}
//	}
//FIXME
/*	else if (mode == POLAR)
	{
		//radius grid
		for (double r = xScale2; r < max(max(xMin,xMax),max(yMin,yMax)); r += xScale2) //<---work on this
		{
			for (int theta = 0; theta < 360; theta++)
			{
				PolarCoord p(r, theta, PolarCoord::DEGREES);
				PolarCoord nextP(r, theta + 1, PolarCoord::DEGREES);
				painter->drawLine(	toPixelXCoord(p.toRectangular().getX()),
							toPixelYCoord(p.toRectangular().getY()),
							toPixelXCoord(nextP.toRectangular().getX()),
							toPixelYCoord(nextP.toRectangular().getY())
				);
			}
		}

		//angle grid
		for (double angle = 0; angle < 360; angle += yScale2) //and this
		{
			PolarCoord p(max(max(xMin,xMax),max(yMin,yMax)), angle, PolarCoord::DEGREES);
			painter->drawLine(	origin.x(),
						origin.y(),
						toPixelXCoord(p.toRectangular().getX()),
						toPixelYCoord(p.toRectangular().getY())
			);
		}
	}*/
	painter->restore();
}

void BasicGraph::drawAxis(QPainter *painter)
{
	painter->save();
	painter->setPen(QPen(_axisColor,axisWidth,Qt::SolidLine));
	painter->drawLine(origin.x(), 0, origin.x(), height()); //x-axis
	painter->drawLine(0, origin.y(), width(), origin.y());  //y-axis
	painter->restore();
}

void BasicGraph::drawScale(QPainter *painter)
{
	painter->save();
	painter->setFont(scaleFont);
	painter->setPen(QColor(_scaleColor));
	QFontMetrics metrics(painter->font());
		//x scale left of axis
		for (int i = 0; i < -xMin / xScale2; i++)
		{
				QString scale;
				scale = QString::number(toGraphXCoord(origin.x() - i * xScale * xScale2));
				painter->drawText((int) (origin.x() - i * xScale * xScale2) - metrics.width(scale)/2, metrics.height(), scale);
		}
		//x scale right of axis
		for (int i = 1; i < xMax / xScale2; i++)
		{
				QString scale;
				scale = QString::number(toGraphXCoord(origin.x() + i * xScale * xScale2));
				painter->drawText((int) (origin.x() + i * xScale * xScale2) - metrics.width(scale)/2, metrics.height(), scale);
		}
		//y scale below axis
		for (int i = 0; i < -yMin / yScale2; i++)
		{
				QString scale;
				scale = QString::number(toGraphYCoord(origin.y() + i * -yScale * yScale2));
				painter->drawText(0, (int) (origin.y() + i * -yScale * yScale2) + metrics.height()/2,  scale);
		}
		//y scale above axis
		for (int i = 1; i < yMax / yScale2; i++)
		{
				QString scale;
				scale = QString::number(toGraphYCoord(origin.y() - i * -yScale * yScale2));
				painter->drawText(0, (int) (origin.y() - i * -yScale * yScale2) + metrics.height()/2, scale);
		}
	painter->restore();
}

void BasicGraph::paintEvent(QPaintEvent *)
{
	bitBlt(this,0,0,buffer);
}

void BasicGraph::paint(QPainter* painter)
{
	painter->setPen( Qt::blue );
	painter->setFont(graphFont);
        
	if (showGrid)
		drawGrid(painter);
	drawAfterAxis(painter);
	if (showAxis)
		drawAxis(painter);
	if (showScale)
		drawScale(painter);
	drawMathFunctions(p);
	if (doingZoomBox)
	{
		painter->setPen( QColor(255,130,20) );
		painter->drawText(30,height()-30,QString(tr("Click and drag to create box")));
		if (mouseDown)
			painter->drawRect( QRect( QPoint(lastClick.x(),lastClick.y()), QPoint(toPixelXCoord(getMouseX()),toPixelYCoord(getMouseY())) ) );
	}
}

void BasicGraph::resizeEvent(QResizeEvent *)
{
	buffer->resize(size());
	updateGraph();
}

double BasicGraph::toGraphXCoord(double x) const
{
	return ((x - origin.x()) / xScale);
}

double BasicGraph::toGraphYCoord(double y) const
{
	return ((y - origin.y()) / yScale);
}

int BasicGraph::toPixelXCoord(double x) const
{
	return static_cast<int>(origin.x() + x * (xScale));
}

int BasicGraph::toPixelYCoord(double y) const
{
	return static_cast<int>(origin.y() + y * (yScale));
}

void BasicGraph::mouseDoubleClickEvent(QMouseEvent *e)
{
	int x = e->x();
	int y = e->y();

	//bring graph to the mouse position
	yMin += (y - height() / 2) / yScale;
	yMax += (y - height() / 2) / yScale;
	xMin += (x - width() / 2) / xScale;
	xMax += (x - width() / 2) / xScale;

	emitDimensionsChanged();
	updateGraph();
}

void BasicGraph::mouseMoveEvent(QMouseEvent *e)
{
	setMouseX(toGraphXCoord(e->x()));
	setMouseY(toGraphYCoord(e->y()));

	updateCoords();
	if (mouseDown)
	{
		if (doingZoomBox)
			repaint(false); //repaint to show selected box
		else
		{
			double deltax = toGraphXCoord(e->x())-toGraphXCoord(lastClick.x());
			double deltay = toGraphYCoord(e->y())-toGraphYCoord(lastClick.y());

			//bring graph to the mouse position
			yMin -= deltay;
			yMax -= deltay;
			xMin -= deltax;
			xMax -= deltax;
			emitDimensionsChanged(); // should I do this here or wait until the mouse has been released?
			updateGraph();

			lastClick.setX(e->x());
			lastClick.setY(e->y());
		}
	}
}

void BasicGraph::mousePressEvent(QMouseEvent *e)
{
	mouseDown = true;

	mathFunctionClick(e,getMouseX(),getMouseY());

	lastClick.setX(toPixelXCoord(getMouseX()));
	lastClick.setY(toPixelYCoord(getMouseY()));
	lastClickGraphCoordX = getMouseX();
	lastClickGraphCoordY = getMouseY();

	setCursor(QCursor(Qt::PointingHandCursor));
}

void BasicGraph::mouseReleaseEvent(QMouseEvent *)
{
	if (doingZoomBox && mouseDown)
	{
		double _xMin = (lastClickGraphCoordX < getMouseX()) ? lastClickGraphCoordX : getMouseX();
		double _xMax = (lastClickGraphCoordX > getMouseX()) ? lastClickGraphCoordX : getMouseX();
		double _yMin = (lastClickGraphCoordY < getMouseY()) ? lastClickGraphCoordY : getMouseY();
		double _yMax = (lastClickGraphCoordY > getMouseY()) ? lastClickGraphCoordY : getMouseY();

		setRange(_xMin, _xMax, _yMin, _yMax);
		emitDimensionsChanged();
		doingZoomBox = false;
		repaint(false);
	}

	mouseDown = false;
	setCursor(QCursor(Qt::ArrowCursor));
}

void BasicGraph::updateCoords()
{
	emit activeCoordinateChanged( QString("x = %1").arg(mouseX), QString("y = %1").arg(mouseY) );
}

void BasicGraph::emitDimensionsChanged()
{
	GraphEvent event(xMin,xMax,xScale2,yMin,yMax,yScale2);
	emit dimensionsChanged(event);
}

int BasicGraph::setRange(double _xMin, double _xMax, double _yMin, double _yMax)
{
	if (_xMin < _xMax && _yMin < _yMax)
	{
		xMin = _xMin;
		xMax = _xMax;
		yMin = _yMin;
		yMax = _yMax;
		updateGraph();
		return 0;
	}
	else
		return 1;
}

int BasicGraph::setXMin(double _xMin)
{
	if (_xMin < xMax)
	{
		xMin = _xMin;
		updateGraph();
		return 0;
	}
	else
		return 1;
}

int BasicGraph::setXMax(double _xMax)
{
	if (_xMax > xMin)
	{
		xMax = _xMax;
		updateGraph();
		return 0;
	}
	else
		return 1;
}

int BasicGraph::setXScale(double _xScale)
{
	if (_xScale > 0)
	{
		xScale2 = _xScale;
		repaint(false);
		return 0;
	}
	else
		return 1;
}

int BasicGraph::setYMin(double _yMin)
{
	if (_yMin < yMax)
	{
		yMin = _yMin;
		updateGraph();
		return 0;
	}
	else
		return 1;
}

int BasicGraph::setYMax(double _yMax)
{
	if (_yMax > yMin)
	{
		yMax = _yMax;
		updateGraph();
		return 0;
	}
	else
		return 1;
}

int BasicGraph::setYScale(double _yScale)
{
	if (_yScale > 0)
	{
		yScale2 = _yScale;
		repaint(false);
		return 0;
	}
	else
		return 1;
}

void BasicGraph::setAxis(bool state)
{
	showAxis = state;
	repaint(false);
}

void BasicGraph::setGrid(bool state)
{
	showGrid = state;
	repaint(false);
}

void BasicGraph::setGridNum(bool state)
{
	showGridNum = state;
	repaint(false);
}

void BasicGraph::setScale(bool state)
{
	showScale = state;
	repaint(false);
}

void BasicGraph::setAxisColor(const QColor &c)
{
	_axisColor = c;
	repaint(false);
}

void BasicGraph::setGridColor(const QColor &c)
{
	_gridColor = c;
	repaint(false);
}

void BasicGraph::setBackgroundColor(const QColor &c)
{
	_backgroundColor = c;
	setPaletteBackgroundColor(_backgroundColor);
	repaint(false);
}

void BasicGraph::setScaleColor(const QColor &c)
{
	_scaleColor = c;
	repaint(false);
}

void BasicGraph::setXZoomFactor(int zoom)
{
	xZoom = zoom;
}

void BasicGraph::setYZoomFactor(int zoom)
{
	yZoom = zoom;
}

QColor BasicGraph::axisColor() const
{
	return _axisColor;
}

QColor BasicGraph::gridColor() const
{
	return _gridColor;
}

QColor BasicGraph::backgroundColor() const
{
	return _backgroundColor;
}

QColor BasicGraph::scaleColor() const
{
	return _scaleColor;
}

int BasicGraph::getXZoomFactor() const
{
	return xZoom;
}

int BasicGraph::getYZoomFactor() const
{
	return yZoom;
}

void BasicGraph::zoomIn()
{
	double half_x = (xMax-xMin)/2;
	double half_y = (yMax-yMin)/2;
	(void)setRange( xMin + half_x - (xMax-xMin) / ( pow(2,xZoom+1) ),
				xMin + half_x + (xMax-xMin)/(pow(2,xZoom+1)),
				yMin + half_y - (yMax-yMin)/(pow(2,yZoom+1)),
				yMin + half_y + (yMax-yMin)/(pow(2,yZoom+1))
				);
	emitDimensionsChanged();
}

void BasicGraph::zoomOut()
{
	double half_x = (xMax-xMin)/2;
	double half_y = (yMax-yMin)/2;
	(void)setRange( xMin + half_x - (xMax-xMin)*( pow(2,xZoom-1) ),
				xMin + half_x + (xMax-xMin)*(pow(2,xZoom-1)),
				yMin + half_y - (yMax-yMin)*(pow(2,yZoom-1)),
				yMin + half_y + (yMax-yMin)*(pow(2,yZoom-1))
				);
	emitDimensionsChanged();
}

void BasicGraph::zoomStd()
{
	yScale2 = 2;
	xScale2 = 2;
	setRange(-15,15,-15,15);
	emitDimensionsChanged();
}

void BasicGraph::zoomBox()
{
	doingZoomBox = true;
	repaint(false);
}

void BasicGraph::setAxisWidth(int i)
{
	axisWidth = i;
	repaint(false);   
}

void BasicGraph::print(
	#ifdef KDE_APP
	KPrinter *printer
	#else
	QPrinter *printer
	#endif //KDE_APP
	)
{
	if ( printer->setup(this) )
	{              
		qDebug( "Printing...");
		QPainter p;
		if ( !p.begin( printer ) )
			return;
		paint(&p); //paint grid and axis
		p.end();
		qDebug( "Printing completed");
	}
	else
		qDebug( "Printing aborted" );
}

void BasicGraph::setMouseX(double x, bool snap)
{
	if (snapToGrid && snap)
	{
		double interval = snapIntervalX();
		double remainder = fmod(x - startSnap(), interval);
		if (remainder < 0)
		{
			if ( fabs(remainder) < (interval/2) )
				mouseX = x - remainder; //round up
			else
				mouseX = x - (interval + remainder); //round down
		}
		else 
		{
			if ( fabs(remainder) < (interval/2) )
				mouseX = x - remainder; //round down
			else
				mouseX = x + (interval - remainder); //round up
		}
	}
	else
		mouseX = x;
}

void BasicGraph::setMouseY(double y, bool snap)
{
	if (snapToGrid && snap)
	{
		double interval = snapIntervalY();
		double remainder = fmod(y, interval);
		if (remainder < 0)
		{
			if ( fabs(remainder) < (interval/2) )
				mouseY = y - remainder;
			else
				mouseY = y - (interval + remainder);
		}
		else
		{
			if ( fabs(remainder) < (interval/2) )
				mouseY = y - remainder;
			else
				mouseY = y + (interval - remainder);
		}
	}
	else
		mouseY = y;
}

void BasicGraph::installMathFunctions()
{
	addMathFunction( new DistanceFunction(this,0), "Distance" );
}

int BasicGraph::execMathFunction( const char * id )
{
	getMathFunction(id)->exec();
	return 0;
}
