/***************************************************************************
                          densityplot.cpp  -  description
                             -------------------
    begin                : Sat Feb 15 2003
    copyright            : (C) 2002-03 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "densityplot.h"

#include <cmath>

#include <qpainter.h>

#include "fungparser.h"
#include "expression.h"
#include "fungmath.h"

DensityPlot::DensityPlot ( QWidget *parent, const char *name ) : ExpressionGraph( parent, name )
{
	onlyShowActive(true);
}

DensityPlot::~DensityPlot()
{
}

void DensityPlot::drawExpression(QPainter *painter, Expression & expression)
{
	FungParser *fp = expression.getParsedExpression(dependent_var());

	QColor color = painter->pen().color();

	double *values = new double[width()*height()];
	int i = 0;              
	double extreme = 0;

	//we need to find the value of the expression with the greatest absolute value for calculating the color
        //... while looping through and getting values
        //they are saved to speed things up in the values[] array                                
	for (int x = 0; x < width(); x++)
	{
		for (int y = 0; y < height(); y++)
		{
			double d[] = {toGraphXCoord(x),toGraphYCoord(y),animatorValue()};
                        values[i] = fp->Eval(d);

			if ( fabs(values[i]) >= fabs(extreme) )
				extreme = values[i];

			i++;                              
		}
	}

	double alpha = (extreme == 0) ? 0 : 255 / fabs(extreme*2);      
	int h, s, v; color.getHsv( h, s, v );                
	i = 0;        
	for (int x = 0; x < width(); x++)
	{
		for (int y = 0; y < height(); y++)
		{
			QColor c;
			c.setHsv( h, s, int(ROUND(255/2 + values[i] * alpha )) );
                        
			painter->setPen( c );                                                                 
			painter->drawPoint(x,y);
                        
			i++;
		}
	}
	
	delete[] values;
}

int DensityPlot::setXYForTrace(const double mousex, const double mousey, double *x, double *y, Expression &)
{
	*x = mousex;
	*y = mousey;

	return 0;
}

void DensityPlot::updateCoords()
{
	if (isTracing())
	{
		FungParser *fp = getParsedCurrentExpression();
		double d[] = {getMouseX(),getMouseY(),animatorValue()};
		double z = fp->Eval(d);

		emit activeCoordinateChanged( QString("x = %1").arg(getMouseX()),
	                                      QString("y = %1").arg(getMouseY()),
                                              QString("z = %1").arg(z) );
	}
	else
		BasicGraph::updateCoords();
}

