/***************************************************************************
                          expressiongraph.cpp  -  description
                             -------------------
    begin                : Fri Dec 20 2002
    copyright            : (C) 2002 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "expressiongraph.h"

#include <qpixmap.h>

#ifdef KDE_APP
#include <kprinter.h>
#else
#include <qprinter.h>
#endif //KDE_APP

#include <qpainter.h>

#include "mathfunction.h"
#include "expression.h"
#include "fungparser.h"

ExpressionGraph::ExpressionGraph(QWidget *parent, const char *name ) : 
	BasicGraph(parent,name),
	Animator(),
	doTrace(false),
	drawConnected(true),
	expressionIndex(0),
	functionWidth(1),
	angle(0),
	only_show_active(false)
{
	initializeAnimator( this );
}

ExpressionGraph::~ExpressionGraph()
{
	for( ExpressionPtrVector::const_iterator it = functions.begin(); it != functions.end(); it++ )
		delete (*it);
}

void ExpressionGraph::paintEvent(QPaintEvent *)
{
	buffer->fill(backgroundColor());
	p->begin(buffer);
	paint(p);
	p->end();

	bitBlt(this,0,0,buffer);
}

void ExpressionGraph::drawAfterAxis(QPainter *p)
{
	drawExpressions(p);
}

void ExpressionGraph::exportAsImage(const QString& file, const QString& fileType)
{
	QPixmap pm(width(),height());
	pm.fill(backgroundColor());
	QPainter painter;
	painter.begin(&pm);
	paint(&painter); //draw BasicGraph stuff (axis and grid)
	painter.end();
	pm.save(file,fileType);
}


void ExpressionGraph::print(
	#ifdef KDE_APP
	KPrinter *printer
	#else
	QPrinter *printer
	#endif //KDE_APP
	)
{
	if ( printer->setup(this) )
	{               // printer dialog
		qDebug( "Printing...");
		QPainter p;
		if ( !p.begin( printer ) )
			return;
		p.setClipRect(0,0,width(),height());
		p.fillRect(0,0,width(),height(),backgroundColor());
		int tempWidth = functionWidth; //temporarily make the functions bigger for printing
		functionWidth = 2;
                paint(&p); //paint stuff from BasicGraph
		functionWidth = tempWidth;
                p.end();
		qDebug( "Printing completed");
	}
	else
		qDebug( "Printing aborted" );
}

Expression ExpressionGraph::getExpression(unsigned int i) const
{
	return (functions.size() >= i + 1 && functions[i]) ? *functions[i] : Expression("",true,QColor());
}


void ExpressionGraph::drawExpressions(QPainter *painter)
{
	QFontMetrics metrics(painter->font());
        
	if (only_show_active)
	{
		if (functions.size() <= expressionIndex){return;}        
		if (functions[expressionIndex]->show == false){return;}
                
		painter->setPen(QPen(functions[expressionIndex]->color,functionWidth));

		//if (!isValidExpression(functions[expressionIndex]))  // this doesn't need to called since it has already been checked...?
		//	return;

		drawExpression(painter,*functions[expressionIndex]);

		painter->setFont(graphFont);
		painter->setPen(QPen(functions[expressionIndex]->color,functionWidth));
		for (unsigned int n=0; n<var_count(); n++)
			painter->drawText(30,35+(metrics.height()+1)*(var_count()+n),QString("%1%2=").arg(dependent_var(n)).arg(expressionIndex+1)+QString(functions[expressionIndex]->getExpression(dependent_var(n)).c_str()));

		if ( displayAnimatorValue() )
		{
			painter->setPen(QColor(255,255,255));
			painter->drawText(30,height()-metrics.height(),QString("n=%1").arg(animatorValue())); /* Probably temporary until implement slider */
		}
	}
	else
	{
		int shown_exp = 0;        
		for(unsigned int i=0; i<functions.size(); i++)
		{
			if (functions[i]->show == false){continue;}
                
			painter->setPen(QPen(functions[i]->color,functionWidth));

			//if (!isValidExpression(functions[i])) //do a few checks before deleting this...
			//	return;

			drawExpression(painter,*functions[i]);

			painter->setFont(graphFont);
			painter->setPen(QPen(functions[i]->color,functionWidth));
			for (unsigned int n=0; n<var_count(); n++)
				painter->drawText(30,35+(metrics.height()+1)*(shown_exp*var_count()+n),QString("%1%2=").arg(dependent_var(n)).arg(i+1)+QString(functions[i]->getExpression(dependent_var(n)).c_str()));
			
			shown_exp++; //control reaches here only if the expression at this index of i is showing so increase number of shown expressions       
        	}

		if ( displayAnimatorValue() )
		{
			painter->setPen(QColor(255,255,255));
			painter->drawText(30,height()-metrics.height(),QString("n=%1").arg(animatorValue())); /* Probably temporary until implement slider */
		}
	}
}

void ExpressionGraph::drawExpressionAt(QPainter *, int)
{
	
}
/*
void ExpressionGraph::callDrawExpression(Expression &expression, QPainter *painter)
{
//	FungParser fp(angle);

//	std::string s = expression.expression();
//	parseExpression(s,&fp);

	drawExpression(painter,expression);
}
*/
/*void ExpressionGraph::setColor(const QColor& c, unsigned int index)
{
	functions[index]->color = c;
	repaint(false);
}
*/
void ExpressionGraph::setShowCurrentExpression(bool state)
{
	if (functions.size() >= expressionIndex + 1)
		functions[expressionIndex]->show = state;  repaint(false);
}

void ExpressionGraph::setTrace(bool b)
{
	doTrace = b;

	if ( doTrace == false ) 
		emit activeCoordinateChanged(QString(""),QString(""),QString("")); //clear coordinates

	repaint(false);
}

void ExpressionGraph::addExpression(const Expression & expression)
{
	std::string error;
	if (isValidExpression(expression,&error))
	{
		emit infoMessageReady("");
		
		Expression *node = new Expression(expression);
		//node->setDeleteable(true);

		for( unsigned int i = 0; i < var_count(); i++ )
		{
			FungParser *fp = new FungParser;

			std::string s = node->getExpression(dependent_var(i));
			qDebug("Adding: %s",s.c_str());
			parseExpression(s,fp,i);
			node->setParsedExpression(fp,dependent_var(i));
		}

		if (functions.size() < expressionIndex+1)
			functions.resize(functions.size()+1);
		else
			delete functions[expressionIndex];

		functions[expressionIndex]=node;

		reparseAll();

		cancelMathFunctions();

		repaint(false);
	}
	else
		emit infoMessageReady(error.c_str());
}

void ExpressionGraph::removeExpression(unsigned int index)
{
	if (functions.size() >= index + 1)
	{
		for( unsigned int i=0; i<var_count(); i++)
			functions[index]->getParsedExpression(dependent_var(i));

		functions.erase(functions.begin()+index);
	}

	reparseAll();

	cancelMathFunctions();

	repaint(false);
}

void ExpressionGraph::prevExpression()
{
	if (expressionIndex == 0)
		expressionIndex = functions.size();
	else
		expressionIndex--;

	cancelMathFunctions();
	repaint(false);
}

void ExpressionGraph::nextExpression()
{
	if (expressionIndex == functions.size())
		expressionIndex = 0;
	else
		expressionIndex++;

	cancelMathFunctions();
	repaint(false);
}

void ExpressionGraph::mouseMoveEvent(QMouseEvent *m)
{
	if ( isTracing() && (functions.size() >= expressionIndex + 1) )
	{
		if (functions.size() < expressionIndex + 1)
			return;

		setMouseX(traceDepX(toGraphXCoord(m->x())));
		setMouseY(traceDepY(toGraphYCoord(m->y())));

		double mousex,mousey;
		int error = setXYForTrace(getMouseX(),getMouseY(),&mousex,&mousey,*functions[expressionIndex]);

		if (error) // oops, there is no current function
		{
			putUndefinedMessage(getMouseX(),getMouseY());
			repaint(false); //repaint to clear trace cursor
			return;
		}

		setMouseX(mousex,false);
		setMouseY(mousey,false);

		repaint(false);

		int pixel_mouse_x = toPixelXCoord(getMouseX());
		int pixel_mouse_y = toPixelYCoord(getMouseY());
                
		QPainter painter(this);
		painter.setPen(QPen(QColor(10,200,50),0,Qt::DotLine));
		painter.drawRect(pixel_mouse_x-3,pixel_mouse_y-3,6,6);
		painter.drawLine(pixel_mouse_x,0,pixel_mouse_x,height());
		painter.drawLine(0,pixel_mouse_y,width(),pixel_mouse_y);
		updateCoords();
	}
	else
		BasicGraph::mouseMoveEvent(m); //there's nothing this class should do with the mouse, pass it to BasicGraph for default behavior
}

bool ExpressionGraph::isValidExpression(const Expression &expression, std::string *errorMsg)
{
	int ret;
	for (unsigned int i=0; i<var_count(); i++)
	{
		FungParser fp;
		std::string s = expression.getExpression(dependent_var(i));
		qDebug("ExpressionGraph::isValidExpression()");
		ret = parseExpression(s,&fp,i);

		if (ret != -1)
		{
			if (errorMsg)
				*errorMsg = ((var_count() > 1) ? std::string(dependent_var(i))+": " : "") + std::string(fp.ErrorMsg());
			return false;
		}
	}
	return true;
}

int ExpressionGraph::parseExpression( std::string & expression, FungParser *fp, int i)
{
	return fp->Parse(expression,std::string(std::string(independent_vars())+",n").c_str(),dependent_var(i),functions,!angle);
}

void ExpressionGraph::reparseAll()
{
	qDebug("ExpressionGraph::reparseAll()");
	for ( unsigned int i = 0; i < functions.size(); i++ )
	{
		for ( unsigned int n = 0; n < var_count(); n++ )
		{
			std::string s = functions[i]->getExpression(dependent_var(n));
			
			FungParser *fp = new FungParser;
			parseExpression( s, fp, n );
			
			functions[i]->setParsedExpression(fp,dependent_var(n));
		}
	}
}

void ExpressionGraph::putUndefinedMessage(double x, double)
{
	emit activeCoordinateChanged( QString("%1 = %2").arg(independent_vars()).arg(x),
	                              QString("%1 = Undefined").arg(dependent_var()) );
}
/*
FungParser * ExpressionGraph::getParsedExpression( int index, int i )
{
	qDebug("ExpressionGraph::getParsedExpression()");
	FungParser *fp = new FungParser(getAngle());
	if (functions.size() < index + 1)
		return 0;
	std::string s = functions[index]->getExpression(dependent_var(i)); 
	int ret = fp->Parse(s,independent_vars(),dependent_var(i),functions,'n',animatorValue());
	if (ret != -1)
		return 0;
	return fp;   
}
*/                       
FungParser * ExpressionGraph::getParsedCurrentExpression(int i)
{
	if (functions.size() < expressionIndex + 1)
		return 0;
	return functions[expressionIndex]->getParsedExpression(dependent_var(i)); 
}

void ExpressionGraph::nextFrameReady()
{
	emit animationValueChanged(animatorValue());
	repaint(false);
}
