/***************************************************************************
                          frequencyplot.cpp  -  description
                             -------------------
    begin                : Sun May 11 2003
    copyright            : (C) 2003 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "frequencyplot.h"

#include <cmath>
#include <algorithm> //sort

#include <qpainter.h>

#include "fungmath.h"
#include "statinfo.h"

FrequencyPlot::FrequencyPlot(std::vector<double> & xlist, int classes, bool show, const QColor & color) :
	PlotModule(show,color),
	_xlist(xlist),
	_classes(classes)
{
	unsigned int xElements = _xlist.size();

	std::sort( _xlist.begin(), _xlist.end() );

	_interval = (_xlist[xElements-1]-_xlist[0]+1)/static_cast<double>(_classes);

	_frequency.resize(_classes);
	for (int i=0; i<_classes ; i++)
		_frequency[i] = 0;
	
	int x=0;
	for (double i=_xlist[0]; i<=_xlist[xElements-1]; i+=_interval)
	{
		for (unsigned int j=0; j<xElements; j++)
		{
			if (_xlist[j]>=i && _xlist[j]<i+_interval)
				_frequency[x]++;
		}
		x++;
	}
	_min = _xlist[0];
}

void FrequencyPlot::drawPlot( QPainter *painter, int *, StatPlotGraph *graph, double xScale, double yScale )
{
	int x=0;
	for (double i=_xlist[0]; i<=_xlist[_xlist.size()-1]; i+=_interval)
	{
		painter->setRasterOp(Qt::XorROP);
		painter->fillRect(
			toPixelXCoord(graph,i)+1,
			toPixelYCoord(graph,0)+(int)(ROUND(yScale*(_frequency[x]))),
			(int)(ROUND(xScale*_interval)),
			(int)(ROUND(-yScale*_frequency[x])),
			QBrush(color(),Qt::Dense4Pattern)
		);
		painter->setRasterOp(Qt::CopyROP);
		painter->setPen(QPen(QColor(0,0,0),2));
		painter->drawRect(
			toPixelXCoord(graph,i)+1,
			toPixelYCoord(graph,0)+(int)(ROUND(yScale*_frequency[x])),
			(int)(ROUND(xScale*_interval)),
			(int)(ROUND(-yScale*_frequency[x]))
		);
		x++;
	}
}

void FrequencyPlot::trace( double x, double, double *new_x, double *new_y )
{
	for (double i=_min; i<_min+_interval*(_classes-1); i+=_interval)
	{
		if ( x > i && x <= i + _interval )
		{
			*new_x = i+_interval/2;
			*new_y = StatInfo::frequency(_xlist,i,i+_interval);
			break;
		}
	}
	if (x <= _min + _interval)
	{
		*new_x = _min+_interval/2;
		*new_y = StatInfo::frequency(_xlist,_min,_min+_interval);
	}
	else if (x > _min + _interval * ( _classes - 1 ) )
	{
		*new_x = _min+_interval*(_classes-1)+_interval/2;
		*new_y = StatInfo::frequency(_xlist,_min+_interval*(_classes-1),_min+_interval*(_classes-1)+_interval);
	}
}

std::string FrequencyPlot::updateCoords( double x, double y )
{
	std::string message;

	char *s1 = new char(num_digits(static_cast<int>(y)));
	itoa(static_cast<int>(y),s1);
	message.append(s1);
	delete[] s1;

	
	message.append(" elements(s) between ");

	char *s2 = new char( num_digits( static_cast<int>(x+_interval/2) ) + 3 );
	dtoa(x-_interval/2,s2);
	message.append( s2 );
	delete[] s2;

	message.append(" and ");

	char *s3 = new char( num_digits( static_cast<int>(x+_interval/2) ) + 3 );
	dtoa(x+_interval/2,s3);
	message.append( s3 );
	delete[] s3;

	return message;
}

void FrequencyPlot::zoomStat( double *xmin, double *xmax, double *ymin, double *ymax )
{
	*xmax = _min+(_interval*((double)_classes+1));
	*xmin = _min-(_interval);
	*ymin = 0;
	double maxFreq = 0;
	for (int i=0; i<_classes; i++)
		maxFreq = StatInfo::bigger(_frequency[i],maxFreq);  //go through all intervals to find the one with the most elements where yMax should be
	*ymax = maxFreq+1;
}

