/***************************************************************************
                          functiongraph.cpp  -  description
                             -------------------
    begin                : Sat May 25 2002
    copyright            : (C) 2002-03 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "functiongraph.h"    

#include <qpainter.h>

#include "expression.h"
#include "fungparser.h"      
#include "definiteintegralfunction.h"
#include "arclengthfunction.h"
#include "maximumfunction.h"
#include "minimumfunction.h"
#include "meanvaluefunction.h"

FunctionGraph::FunctionGraph ( QWidget *parent, const char *name ) : ExpressionGraph( parent, name )
{
}

FunctionGraph::~FunctionGraph()
{
}

void FunctionGraph::installMathFunctions()
{
	BasicGraph::installMathFunctions();

	addMathFunction(new DefiniteIntegralFunction(this,1),"Definite Integral");
	addMathFunction(new ArcLengthFunction(this,1),"Arc Length");
	addMathFunction(new MaximumFunction(this,1),"Maximum");
	addMathFunction(new MinimumFunction(this,1),"Minimum");
	addMathFunction(new MeanValueFunction(this,1),"Mean Value");
}

void FunctionGraph::drawExpression(QPainter *painter, Expression & expression)
{
	for (double x = 0; x < width(); x++)
	{
		FungParser *fp = expression.getParsedExpression(dependent_var());

		double xval[] = {toGraphXCoord(x),animatorValue()};
                
		double evaluated = fp->Eval(xval);
		
                if (fp->EvalError()){continue;}

		if (isDrawConnected())
		{
			double nextxval[] = {toGraphXCoord(x+1),animatorValue()};
			double nextevaluated = fp->Eval(nextxval);

			if (!fp->EvalError())
				painter->drawLine((int)x,toPixelYCoord(evaluated),(int)x+1,toPixelYCoord(nextevaluated));
		}
		else
			painter->drawPoint((int)x,toPixelYCoord(evaluated));
	}
}

int FunctionGraph::setXYForTrace(const double mousex, const double, double *x, double *y, Expression &expression)
{
	FungParser *fp = expression.getParsedExpression(dependent_var());

	double d[] = {mousex,animatorValue()};

	*x = mousex;
	*y = fp->Eval(d);

	return fp->EvalError();
}

int FunctionGraph::execMathFunction( const char * id )
{
	MathFunction *mf = getMathFunction(id);
	if ( mf )
	{
		switch ( mf->expressionsRequired() )
		{
			case 0: return BasicGraph::execMathFunction(id); break;
			case 1:
			{
				FungParser *fp = getParsedCurrentExpression();
				if (fp)
				{
					mf->exec(fp);
					return 1;
				}
				return -1;
				break;
			}
			default: break;
		}
	}

	return -1;
}


