/***************************************************************************
                          parametricgraph.cpp  -  description
                             -------------------
    begin                : Sun May 26 2002
    copyright            : (C) 2002 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "parametricgraph.h"

#include <qpainter.h>

#include "expression.h"
#include "fungparser.h"
#include "arclengthfunction.h"

ParametricGraph::ParametricGraph(QWidget *parent, const char *name) : ExpressionGraph (parent, name)
{
	tMin = -15;
	tMax = 15;
	tStep = 0.1;
}

ParametricGraph::~ParametricGraph()
{
}

void ParametricGraph::installMathFunctions()
{
	BasicGraph::installMathFunctions();

	addMathFunction(new ArcLengthFunction(this,2),"Arc Length");
}

int ParametricGraph::execMathFunction( const char * id )
{
	MathFunction *mf = getMathFunction(id);
	if ( mf )
	{
		switch ( mf->expressionsRequired() )
		{
			case 0: return BasicGraph::execMathFunction(id); break;
			case 1: //qWarning("Invalid MathFunction: %s",id); break;
			case 2: 
			{
				FungParser *fpx = getParsedCurrentExpression(X_VAR);
				FungParser *fpy = getParsedCurrentExpression(Y_VAR);
				if (fpx && fpy)
				{
					mf->exec(fpx,fpy);
					return 0;
				}
				return -1;
			}
			default: break;
		}
	}
	return -1;
}

void ParametricGraph::sendCustomClick(QMouseEvent *e, MathFunction *m, double, double)
{
	qDebug("custom click: %f",traceDepX(toGraphXCoord(e->x())));
	m->sendClick(e,traceDepX(toGraphXCoord(e->x())));
}

void ParametricGraph::putUndefinedMessage(double x, double)
{
	emit activeCoordinateChanged( QString("%1 = %2").arg(independent_vars()).arg(traceDepX(x)),
	                              QString("%1 = Undefined").arg(dependent_var(X_VAR)),
	                              QString("%1 = Undefined").arg(dependent_var(Y_VAR)) );
}

void ParametricGraph::updateCoords()
{
	if (isTracing())
	{
		emit activeCoordinateChanged( QString("%1 = %2").arg(independent_vars()).arg(t_trace),
	                                      QString("%1 = %2").arg(dependent_var(X_VAR)).arg(getMouseX()),
		                              QString("%1 = %2").arg(dependent_var(Y_VAR)).arg(getMouseY()) );
       
	}
	else
		BasicGraph::updateCoords();
}

void ParametricGraph::drawExpression(QPainter* painter, Expression & expression)
{
	for (double t = tMin; t < tMax; t+=tStep)
	{
		FungParser *xfp = expression.getParsedExpression(dependent_var(X_VAR));
		FungParser *yfp = expression.getParsedExpression(dependent_var(Y_VAR));

		double d[] = {t,animatorValue()};

		double xevaluated = xfp->Eval(d);
		double yevaluated = yfp->Eval(d);
		if (xfp->EvalError() || yfp->EvalError()){continue;}

		if (isDrawConnected())
		{
			double nexttval = t+tStep;

			double dnext[] = {nexttval,animatorValue()};

			double nextxevaluated = xfp->Eval(dnext);
			double nextyevaluated = yfp->Eval(dnext);
			if (xfp->EvalError() || yfp->EvalError()){continue;}

			painter->drawLine(toPixelXCoord(xevaluated),toPixelYCoord(yevaluated),toPixelXCoord(nextxevaluated),toPixelYCoord(nextyevaluated));
		}
		else
			painter->drawPoint(toPixelXCoord(xevaluated),toPixelYCoord(yevaluated));	
	}
//		painter->drawText(25,35+(2*i)*(metrics.height()+1),QString("x%1=").arg(i+1)+functions->at(i)->xexpression);
//		painter->drawText(25,35+(2*i+1)*(metrics.height()+1),QString("y%1=").arg(i+1)+functions->at(i)->yexpression);
}
/*
void ParametricGraph::callDrawExpression(Expression &expression, QPainter *painter)
{
	FungParser fpx(getAngle());
	FungParser fpy(getAngle());

	std::string sx = expression.xexpression();
	std::string sy = expression.yexpression();

	parseExpression(sx,&fpx,X_VAR);
	parseExpression(sy,&fpy,Y_VAR);

	drawExpression(painter,fpx,fpy);
}
*/
int ParametricGraph::setTMin(double _tMin)
{
	if (_tMin < tMax)
	{
		tMin = _tMin;
		repaint(false);
		return 0;
	}
	else
		return 1;
}

int ParametricGraph::setTMax(double _tMax)
{
	if (_tMax > tMin)
	{
		tMax = _tMax;
		repaint(false);
		return 0;
	}
	else
		return 1;
}

int ParametricGraph::setTStep(double _tStep)
{
	if (_tStep > 0)
	{
		tStep = _tStep;
		repaint(false);
		return 0;
	}
	else
		return 1;
}

int ParametricGraph::setXYForTrace(const double mousex, const double, double *x, double *y, Expression &expression)
{
	FungParser *fpx = expression.getParsedExpression(dependent_var(X_VAR));
	FungParser *fpy = expression.getParsedExpression(dependent_var(Y_VAR));

	double d[] = {mousex,animatorValue()};

	*x = fpx->Eval(d);
	*y = fpy->Eval(d);

	t_trace = mousex;

	return (fpx->EvalError() || fpy->EvalError());
}







