/***************************************************************************
                          polargraph.cpp  -  description
                             -------------------
    begin                : Thu May 30 2002
    copyright            : (C) 2002 by Fungmeista
    email                : confederacy2@excite.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "polargraph.h"

#include <qpainter.h>

#include "polarcoord.h"
#include "rectcoord.h"
#include "fungparser.h"
#include "expression.h"

PolarGraph::PolarGraph(QWidget *parent, const char *name ) : ExpressionGraph(parent,name)
{
	setCoordinateStyle(POLAR);

	tMin = -15;
	tMax = 15;
	tStep = 0.1;
}

int PolarGraph::setXYForTrace(const double mousex, const double, double *x, double *y, Expression &expression)
{
	FungParser *fp = expression.getParsedExpression(dependent_var());

	double d[] = {mousex,animatorValue()};
	double r = fp->Eval(d);

	PolarCoord p(r, mousex, getAngle());
	*x = p.toRectangular().getX();
	*y = p.toRectangular().getY();

	t_trace = mousex;

	return fp->EvalError();
}

void PolarGraph::updateCoords()
{
	if (isTracing())
		emit activeCoordinateChanged( QString("x = %1").arg(getMouseX()), QString("y = %1").arg(getMouseY()), QString("t = %1").arg(t_trace) );
	else
		BasicGraph::updateCoords();
}

void PolarGraph::drawExpression(QPainter *painter, Expression & expression)
{
	for (double theta = tMin; theta < tMax; theta+=tStep)
	{
		FungParser *fp = expression.getParsedExpression(dependent_var());

		double d[] = {theta,animatorValue()};

		double r = fp->Eval(d);
		if (fp->EvalError()){continue;}

		PolarCoord p(r, theta, getAngle());

		if (isDrawConnected())
		{
			double temp = theta + tStep;

			double dnext[] = {temp,animatorValue()};

			r = fp->Eval(dnext);
			if (fp->EvalError()){continue;}

		PolarCoord nextP(r, theta + tStep, getAngle());
		painter->drawLine(
			toPixelXCoord(p.toRectangular().getX()),
			toPixelYCoord(p.toRectangular().getY()),
			toPixelXCoord(nextP.toRectangular().getX()),
			toPixelYCoord(nextP.toRectangular().getY()));
		}
		else
		{
			painter->drawPoint(
				toPixelXCoord(p.toRectangular().getX()),
				toPixelYCoord(p.toRectangular().getY()));
		}
	}
}

int PolarGraph::setTMin(double _tMin)
{
	if (_tMin < tMax)
	{
		tMin = _tMin;
		repaint(false);
		return 0;
	}
	else
		return 1;
}

int PolarGraph::setTMax(double _tMax)
{
	if (_tMax > tMin)
	{
		tMax = _tMax;
		repaint(false);
		return 0;
	}
	else
		return 1;
}

int PolarGraph::setTStep(double _tStep)
{
	if (_tStep > 0)
	{
		tStep = _tStep;
		repaint(false);
		return 0;
	}
	else
		return 1;
}

