/***************************************************************************
                          statplotgraph.cpp  -  description
                             -------------------
    begin                : Sun Jun 2 2002 -- redone : Fri Oct 11 2002
    copyright            : (C) 2002 by Fungmeista
    email                : confederacy2@excite.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "statplotgraph.h"

#include <qpixmap.h>
#include <qpainter.h>

#ifdef KDE_APP
#include <kprinter.h>
#else
#include <qprinter.h>
#endif //KDE_APP

#include "plotmodule.h"
#include "statinfo.h"
#include "fungmath.h"

StatPlotGraph::StatPlotGraph(QWidget *parent, const char *name ) : BasicGraph(parent,name)
{
	plotIndex = 0;
	doTrace = false;
}

StatPlotGraph::~StatPlotGraph()
{}

QColor StatPlotGraph::getColor( unsigned int i ) const
{
	return (plots.size() >= i+1) ? plots[i]->color() : QColor();
}

void StatPlotGraph::setShowCurrentPlot(bool show)
{
	if (plots.size() != 0)
	{
		(plots[plotIndex])->setVisible(show);
		repaint(false);
	}
}

void StatPlotGraph::nextPlot()
{
	if (plotIndex + 1 == plots.size())
		plotIndex = 0;
	else
		plotIndex++;
	repaint(false);
}

void StatPlotGraph::prevPlot()
{
	if (plotIndex == 0)
		plotIndex = plots.size() - 1;
	else
		plotIndex--;
	repaint(false);
}

void StatPlotGraph::paintEvent(QPaintEvent *)
{
	buffer->fill(backgroundColor());
	p->begin(buffer);
	paint(p);
	drawPlots(p);
	p->end();
	bitBlt(this,0,0,buffer);
}

void StatPlotGraph::drawPlots(QPainter * painter)
{
	QFontMetrics metrics(painter->font());
	painter->setFont(QFont("times",12,QFont::Black));

	int baw_plots = 0;
	unsigned int visible_plots = 0;
	for ( unsigned int i=0; i<plots.size(); i++)
	{
		PlotModule *plot = plots[i];
		if (plot->show() == false){continue;}

		painter->setPen(plot->color());
		plot->drawPlot(painter,&baw_plots,this,xScale,yScale);
		
		painter->setPen(plot->color());
		painter->drawText(30,35+(metrics.height()+1)*visible_plots,QString(tr("Plot %1")).arg(i+1));

		visible_plots++;

		if ( i == plotIndex && isTracing() )
			painter->fillRect(toPixelXCoord(getMouseX()) - 6 / 2, toPixelYCoord(getMouseY()) - 6 / 2, 6, 6, QBrush(QColor(255,255,255),Qt::SolidPattern));
	}
}

void StatPlotGraph::addPlot(unsigned int index, PlotModule * plot)
{
	if ( plots.size() < index + 1 )
		plots.resize(plots.size()+1);
	plots[index] = plot;
	repaint(false);
}

void StatPlotGraph::removePlot(unsigned int index)
{
	qDebug(QString::number(index));
	if (plots.size() >= index + 1)
		plots.erase(plots.begin()+index);

	repaint(false);
}

void StatPlotGraph::mouseMoveEvent(QMouseEvent *m)
{
	if ( isTracing() && ( plots.size() >= plotIndex + 1))
	{
		double mousex, mousey;
		plots[plotIndex]->trace( toGraphXCoord(m->x()), toGraphYCoord(m->y()), &mousex, &mousey );

		setMouseX(mousex,false);
		setMouseY(mousey,false);

		updateCoords();
		repaint(false);
	}
	else
		BasicGraph::mouseMoveEvent(m);

}

void StatPlotGraph::updateCoords()
{
	if ( isTracing() )
	{
		std::string message = plots[plotIndex]->updateCoords( getMouseX(), getMouseY() );
		emit infoMessageReady(QString(message.c_str()));
		
	}
	else
		BasicGraph::updateCoords();
}

void StatPlotGraph::zoomStat()
{
	if ( plotIndex >= plots.size() ){qDebug("No plot entered"); return;}

	double xmin = xMin;
	double xmax = xMax;
	double ymin = yMin;
	double ymax = yMax;

	plots[plotIndex]->zoomStat( &xmin, &xmax, &ymin, &ymax );

	setRange(xmin,xmax,ymin,ymax);

	updateGraph();
	emitDimensionsChanged();
	repaint(false);
}


void StatPlotGraph::print(
	#ifdef KDE_APP
	KPrinter *printer
	#else
	QPrinter *printer
	#endif //KDE_APP
	)
{
	if ( printer->setup(this) )
	{               // printer dialog
		qDebug( "Printing...");
		QPainter p;
		if ( !p.begin( printer ) )
			return;
		p.setClipRect(0,0,width(),height());
		p.fillRect(0,0,width(),height(),backgroundColor());
		paint(&p); //paint stuff from BasicGraph
		drawPlots(&p);
		p.end();
		qDebug( "Printing completed");
	}
	else
		qDebug( "Printing aborted" );
}

void StatPlotGraph::exportAsImage(const QString &file, const QString &fileType)
{
	QPixmap pm(width(),height());
	pm.fill(backgroundColor());
	QPainter painter;
	painter.begin(&pm);
	paint(&painter); //draw BasicGraph stuff (axis and grid)
	drawPlots(&painter);
	painter.end();
	pm.save(file,fileType);
}

