/***************************************************************************
                          whiskerplot.cpp  -  description
                             -------------------
    begin                : Sun May 11 2003
    copyright            : (C) 2003 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "whiskerplot.h"

#include <algorithm> //sort

#include <qpainter.h>

#include "fungmath.h"
#include "statinfo.h"

WhiskerPlot::WhiskerPlot(std::vector<double> & xlist, bool show, const QColor & color) :
	PlotModule(show,color),
	_xlist(xlist)
{
	std::sort(_xlist.begin(), _xlist.end() );

	_median = StatInfo::median(_xlist);
	_first_quartile = StatInfo::first_quartile(_xlist);
	_third_quartile = StatInfo::third_quartile(_xlist);
	_min = StatInfo::min(_xlist);
	_max = StatInfo::max(_xlist);
}

void WhiskerPlot::drawPlot( QPainter *painter, int *index, StatPlotGraph *graph, double, double )
{
	painter->setPen(QPen(color(),2));
	const int whiskerAxis = -1 - boxAndWhiskerHeight/2;
	int totalWhiskerPlots = *index;
	const int y_coord_middle = whiskerAxis-totalWhiskerPlots*(boxAndWhiskerHeight+2);
	const int y_coord_top = y_coord_middle+boxAndWhiskerHeight/2;
	const int y_coord_bottom = y_coord_middle-boxAndWhiskerHeight/2;

	painter->fillRect(toPixelXCoord(graph,_min)-3,toPixelYCoord(graph,y_coord_middle)-3,6,6,QBrush(color(),Qt::Dense1Pattern)); //minimum
	painter->fillRect(toPixelXCoord(graph,_max)-3,toPixelYCoord(graph,y_coord_middle)-3,6,6,QBrush(color(),Qt::Dense1Pattern)); //maximum
	painter->drawLine(
			toPixelXCoord(graph,_median),
			toPixelYCoord(graph,y_coord_top),
			toPixelXCoord(graph,_median),
			toPixelYCoord(graph,y_coord_bottom));
	painter->drawLine(
			toPixelXCoord(graph,_first_quartile),
			toPixelYCoord(graph,y_coord_top),
			toPixelXCoord(graph,_first_quartile),
			toPixelYCoord(graph,y_coord_bottom));
	painter->drawLine(
			toPixelXCoord(graph,_third_quartile),
			toPixelYCoord(graph,y_coord_top),
			toPixelXCoord(graph,_third_quartile),
			toPixelYCoord(graph,y_coord_bottom));
	painter->drawLine(
			toPixelXCoord(graph,_first_quartile),
			toPixelYCoord(graph,y_coord_bottom),
			toPixelXCoord(graph,_third_quartile),
			toPixelYCoord(graph,y_coord_bottom));
	painter->drawLine(
			toPixelXCoord(graph,_third_quartile),
			toPixelYCoord(graph,y_coord_top),
			toPixelXCoord(graph,_first_quartile),
			toPixelYCoord(graph,y_coord_top));
	painter->drawLine(
			toPixelXCoord(graph,_first_quartile),
			toPixelYCoord(graph,y_coord_middle),
			toPixelXCoord(graph,_min),
			toPixelYCoord(graph,y_coord_middle));
	painter->drawLine(
			toPixelXCoord(graph,_third_quartile),
			toPixelYCoord(graph,y_coord_middle),
			toPixelXCoord(graph,_max),
			toPixelYCoord(graph,y_coord_middle));

	_vertical = *index;

	*index += 1;
}

void WhiskerPlot::trace( double x, double, double *new_x, double *new_y )
{
	if ( x <= (_min + _first_quartile)/2 )
		*new_x = _min;
	else if ( x > (_min+_first_quartile)/2 && x<=(_first_quartile+_median)/2 )
		*new_x = _first_quartile;
	else if ( x > (_first_quartile+_median)/2 && x<=(_median+_third_quartile)/2 )
		*new_x = _median;
	else if ( x > (_median+_third_quartile)/2 && x<=(_third_quartile+_max)/2 )
		*new_x = _third_quartile;
	else if ( x > (_third_quartile+_max)/2 )
		*new_x = _max;
		
	*new_y = (-1 - boxAndWhiskerHeight/2) - ( _vertical * (boxAndWhiskerHeight+2) );
}

std::string WhiskerPlot::updateCoords( double x, double )
{
	std::string message;

	if ( x == _min )
		message = "Minimum: ";
	if ( x == _max )
		message = "Maximum: ";
	if ( x == _first_quartile )
		message = "First Quartile: ";
	if ( x == _third_quartile )
		message = "Third Quartile: ";
	if ( x == _median )
		message = "Median: ";
		
	char *s = new char( num_digits( static_cast<int>(x) ) + 3 );
	dtoa(x,s);
	message.append( s );
	delete[] s;
	
	return message;
}

void WhiskerPlot::zoomStat( double *xmin, double *xmax, double *, double * )
{
	*xmin = _min-1;
	*xmax = _max+1;
}


