/***************************************************************************
                          glbasicgraph.cpp  -  description
                             -------------------
    begin                : Mon Sep 30 2002
    copyright            : (C) 2002-03 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "glbasicgraph.h"
 
#include <cmath>

#include "config.h"

#if HAVE_GLUT
#include <GL/glut.h>
#endif //HAVE_GLUT

#include <qcursor.h>
#include <qimage.h>
#include <qpainter.h>
#include <qtimer.h> //auto-rotate

#ifdef KDE_APP
#include <kprinter.h>
#else
#include <qprinter.h>
#endif //KDE_APP

#include "glgraphevent.h"

GLBasicGraph::GLBasicGraph( QWidget* parent, const char* name ) : QGLWidget( parent, name )
{
	lastClick = new QPoint(0,0); 

	xMin = -15.0f;
	xMax = 15.0f;
	yMin = -15.0f;
	yMax = 15.0f;
	zMin = -10.0f;
	zMax = 10.0f;

	xMinView = -480.0f;
	xMaxView = 480.0f;
	yMinView = -480.0f;
	yMaxView = 480.0f;
	zMinView = -480.0f;
	zMaxView = 480.0f;

	xRot = 300.0f;
	yRot = 0.0f;
	zRot = 330.0f;

	mouseDown = false;
	snapToGrid = false;

	showAxis = true;
	showScale = true;

	xZoom = yZoom = zZoom = 2;

	//colors
	_axisColor = QColor(255,0,0);
	_backgroundColor = QColor(0,0,0);
	_scaleColor = QColor(85,170,0);

	setMouseTracking(true);

	xScale = 1;
	yScale = 1;

	origin = new QPoint();
	
	rotating_x = rotating_y = rotating_z = false;
	rotate_speed = 50;
	rotate_timer = new QTimer( this ); //auto-deleted
	connect( rotate_timer, SIGNAL(timeout()), this, SLOT(autoRotateStep()) );
}

GLBasicGraph::~GLBasicGraph()
{
	delete lastClick;
	delete origin;
	delete rotate_timer;
}

void GLBasicGraph::initializeGL()
{
}

void GLBasicGraph::resizeGL( int w, int h )
{
	/*
	if ( w < h )
	{
		glViewport( 0, (h-w)/2, w, w );
		_width = _height = w;
	}
	else
	{
		glViewport( (w-h)/2, 0, h, h );
		_width = _height = h;
	}
	*/
	
	_width = w;
	_height = h;

	glViewport( 0, 0, w, h );

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();

	updateOrtho();

	//gluPerspective( yRot, h/w, abs(zMinView), zMaxView );
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();

	glDrawBuffer( GL_BACK );
	glEnable( GL_DEPTH_TEST );
	
	glShadeModel( GL_SMOOTH );
	glClearDepth( 1.0f );
	
	glEnable( GL_LINE_SMOOTH );
	glEnable( GL_BLEND );
	
	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
	glHint( GL_LINE_SMOOTH_HINT, GL_NICEST );
	
	updateView();
}

void GLBasicGraph::paintGL()
{
	qglClearColor( _backgroundColor );
	
	glClear( GL_COLOR_BUFFER_BIT  | GL_DEPTH_BUFFER_BIT);
	
	glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
	glPushMatrix();

	//glTranslated( -(xMax+xMin)/2, -(yMax+yMin)/2, -(zMax+zMin)/2 );
	//glTranslated( 0.0, 0.0, 0.0 );
	glRotatef( xRot, 1.0f, 0.0f, 0.0f );
	glRotatef( yRot, 0.0f, 1.0f, 0.0f );
	glRotatef( zRot, 0.0f, 0.0f, 1.0f );
	glScalef( xScale, yScale, xScale );

	if (showAxis)
		drawAxis();

	//drawCage();
}

void GLBasicGraph::drawAxis()
{
	double letter_width, letter_height;
	letter_width = letter_height = fabs( xMaxView / xScale ) / 30.0;

//	qglColor( _axisColor );
//	glLineWidth(6.0f);
	glBegin(GL_LINES);

		//x-axis and labels
		glColor3f(1.0f, 0.0f, 0.0f); //red
		glVertex3f(xMinView/xScale, 0.0, 0.0);
		glVertex3f(xMaxView/xScale, 0.0, 0.0);
		drawLetterMinus(xMinView/xScale,0.0,0.0,letter_width);
		drawLetterX(xMinView/xScale,0.0,0.0,letter_height,letter_width);
		drawLetterPlus(xMaxView/xScale,0.0,0.0,letter_height,letter_width);
		drawLetterX(xMaxView/xScale,0.0,0.0,letter_height,letter_width);

		//y-axis and labels
		glColor3f(0.0f, 1.0f, 0.0f); //green
		glVertex3f(0.0, yMinView/yScale, 0.0);
		glVertex3f(0.0, yMaxView/yScale, 0.0);
		drawLetterMinus(0.0,yMinView/yScale,0.0,letter_width);
		drawLetterY(0.0,yMinView/yScale,0.0,letter_height,letter_width);
		drawLetterPlus(0.0,yMaxView/yScale,0.0,letter_height,letter_width);
		drawLetterY(0.0,yMaxView/yScale,0.0,letter_height,letter_width);

		//z-axis and labels
		glColor3f(0.0f, 0.0f, 1.0f); //blue
		glVertex3f(0.0, 0.0, zMinView/xScale);
		glVertex3f(0.0, 0.0, zMaxView/xScale);
		drawLetterMinus(0.0,0.0,zMinView/xScale,letter_width);
		drawLetterZ(0.0,0.0,zMinView/xScale,letter_height,letter_width);
		drawLetterPlus(0.0,0.0,zMaxView/xScale,letter_height,letter_width);
		drawLetterZ(0.0,0.0,zMaxView/xScale,letter_height,letter_width);

	glEnd();
//	glLineWidth(1.0f);
/*
	glColor3f(1.0f, 0.0f, 0.0f);
	drawText(xMax, 0.0f, 0.0f, "+x");
	glColor3f(0.0f, 1.0f, 0.0f);
	drawText(0.0f, yMax, 0.0f, "+y");
	glColor3f(0.0f, 0.0f, 1.0f);
	drawText(0.0f, 0.0f, zMax, "+z");

	glColor3f(1.0f, 0.0f, 0.0f);
	drawText(xMin, 0.0f, 0.0f, "-x");
	glColor3f(0.0f, 1.0f, 0.0f);
	drawText(0.0f, yMin, 0.0f, "-y");
	glColor3f(0.0f, 0.0f, 1.0f);
	drawText(0.0f, 0.0f, zMin, "-z");
*/
}
#if 0
void GLBasicGraph::drawCage()
{
	const GLfloat x_view = (xMinView-xMaxView)*0.5f;
	const GLfloat y_view = (yMinView-yMaxView)*0.5f;
	const GLfloat z_view = (zMinView-zMaxView)*0.5f;

	glBegin(GL_LINES);
		glVertex3f(xMinView+x_view,yMaxView-y_view,zMaxView-z_view);
		glVertex3f(xMaxView-x_view,yMaxView-x_view,zMaxView-z_view);

		glVertex3f(xMinView,yMaxView,zMaxView);
		glVertex3f(xMinView,yMinView,zMaxView);
		
		glVertex3f(xMinView,yMaxView,zMaxView);
		glVertex3f(xMinView,yMaxView,zMinView);
		
		glVertex3f(xMinView,yMaxView,zMaxView);
		glVertex3f(xMaxView,yMaxView,zMaxView);
		
		glVertex3f(xMaxView,yMaxView,zMaxView);
		glVertex3f(xMaxView,yMaxView,zMinView);
		
		glVertex3f(xMinView,yMaxView,zMinView);
		glVertex3f(xMinView,yMinView,zMinView);
		
		glVertex3f(xMinView,yMinView,zMinView);
		glVertex3f(xMinView,yMinView,zMaxView);
		
		glVertex3f(xMinView,yMinView,zMinView);
		glVertex3f(xMaxView,yMinView,zMinView);
		
		glVertex3f(xMaxView,yMinView,zMinView);
		glVertex3f(xMaxView,yMinView,zMaxView);
		
		glVertex3f(xMaxView,yMinView,zMinView);
		glVertex3f(xMaxView,yMaxView,zMinView);
		
		glVertex3f(xMaxView,yMaxView,zMinView);
		glVertex3f(xMaxView,yMaxView,zMaxView);
		
		glVertex3f(xMaxView,yMinView,zMaxView);
		glVertex3f(xMinView,yMinView,zMaxView);
	
		glVertex3f(xMaxView,yMaxView,zMaxView);
		glVertex3f(xMaxView,yMinView,zMaxView);
	glEnd();
}
#endif
double GLBasicGraph::toGraphXCoord(double x)
{
	//if ( _width < _height )
		return ((x - origin->x()) / xScale);
	//else
	//	return toGraphYCoord(x);
}

double GLBasicGraph::toGraphYCoord(double y)
{
	//if ( _height < _width )
		return ((y - origin->y()) / yScale);
	//else
	//	return toGraphXCoord(y);
}

void GLBasicGraph::mousePressEvent(QMouseEvent *e)
{
	if (!mouseDown)
	{
		lastClick->setX(e->x());
		lastClick->setY(e->y());
		mouseDown = true;
	}

	updateGL();
}

void GLBasicGraph::mouseMoveEvent(QMouseEvent *e)
{
	if (mouseDown)
	{
		xRot += (e->y() - lastClick->y());
		zRot += (e->x() - lastClick->x());

		lastClick->setX(e->x());
		lastClick->setY(e->y());

		//keep between 0 and 360
		if ( xRot >= 360 )
			xRot -= 360;
		if ( xRot < 0 )
			xRot += 360;

		if ( zRot >= 360 )
			zRot -= 360;
		if ( zRot < 0 )
			zRot += 360;

		if ( rotate_timer->isActive() )
		{
			rotating_x = rotating_y = rotating_z = false;
			rotate_timer->stop();
			rotateCanceled();
		}

		emitGraphRotated();

		updateGL();
	}
}

void GLBasicGraph::mouseReleaseEvent(QMouseEvent *)
{
	mouseDown = false;
}

int GLBasicGraph::setXMin(double d)
{
	if (d < xMax)
	{
		xMin = static_cast<GLfloat>(d);
		updateOrtho();
		updateGL();
		return 0;
	}
	else
		return 1;
}

int GLBasicGraph::setXMax(double d)
{
	if (d > xMin)
	{
		xMax = static_cast<GLfloat>(d);
		updateOrtho();
		updateGL();
		return 0;
	}
	else
		return 1;
}

int GLBasicGraph::setYMin(double d)
{
	if (d < yMax)
	{
		yMin = static_cast<GLfloat>(d);
		updateOrtho();
		updateGL();
		return 0;
	}
	else
		return 1;
}

int GLBasicGraph::setYMax(double d)
{
	if (d > yMin)
	{
		yMax = static_cast<GLfloat>(d);
		updateOrtho();
		updateGL();
		return 0;
	}
	else
		return 1;
}

int GLBasicGraph::setZMin(double d)
{
	if (d < zMax)
	{
		zMin = static_cast<GLfloat>(d);
		updateOrtho();
		updateGL();
		return 0;
	}
	else
		return 1;
}

int GLBasicGraph::setZMax(double d)
{
	if (d > zMin)
	{
		zMax = static_cast<GLfloat>(d);
		updateOrtho();
		updateGL();
		return 0;
	}
	else
		return 1;
}

int GLBasicGraph::setXMinView(double d)
{
	if (d < xMaxView)
	{
		xMinView = static_cast<GLfloat>(d);
		updateView();
		updateGL();
		return 0;
	}
	else
		return 1;
}

int GLBasicGraph::setXMaxView(double d)
{
	if (d > xMinView)
	{
		xMaxView = static_cast<GLfloat>(d);
		updateView();
		updateGL();
		return 0;
	}
	else
		return 1;
}

int GLBasicGraph::setYMinView(double d)
{
	if (d < yMaxView)
	{
		yMinView = static_cast<GLfloat>(d);
		updateView();
		updateGL();
		return 0;
	}
	else
		return 1;
}

int GLBasicGraph::setYMaxView(double d)
{
	if (d > yMinView)
	{
		yMaxView = static_cast<GLfloat>(d);
		updateView();
		updateGL();
		return 0;
	}
	else
		return 1;
}

int GLBasicGraph::setZMinView(double d)
{
	if (d < zMaxView)
	{
		zMinView = static_cast<GLfloat>(d);
		updateView();
		updateGL();
		return 0;
	}
	else
		return 1;
}

int GLBasicGraph::setZMaxView(double d)
{
	if (d > zMinView)
	{
		zMaxView = static_cast<GLfloat>(d);
		updateView();
		updateGL();
		return 0;
	}
	else
		return 1;
}

void GLBasicGraph::setPolarEye(double d)
{
	xRot = static_cast<GLfloat>(static_cast<int>(d) % 360);
	updateGL();
}

void GLBasicGraph::setAzimuthalEye(double d)
{
	zRot = static_cast<GLfloat>(static_cast<int>(d) % 360);
	updateGL();
}

void GLBasicGraph::zoomStd()
{
	xRot = 300.0f;
	yRot = 0.0f;
	zRot = 330.0f;

	emitGraphRotated();

	xMinView = -480.0f;
	xMaxView = 480.0f;
	yMinView = -480.0f;
	yMaxView = 480.0f;
	zMinView = -480.0f;
	zMaxView = 480.0f;
	
	updateView();
	updateGL();
}

void GLBasicGraph::zoomIn()
{
	xMinView /= xZoom;//xMinView+(xMaxView-xMinView)/(2*xZoom);
	xMaxView /= xZoom;//xMaxView-(xMaxView-xMinView)/(2*xZoom);
	yMinView /= yZoom;//yMinView+(yMaxView-yMinView)/(2*yZoom);
	yMaxView /= yZoom;//yMaxView-(yMaxView-yMinView)/(2*yZoom);
	zMinView /= xZoom;//zMinView+(zMaxView-zMinView)/(2*zZoom);
	zMaxView /= xZoom;//zMaxView-(zMaxView-zMinView)/(2*zZoom);
	
	updateView();
	updateGL();
	//emitDimensionsChanged();
}

void GLBasicGraph::zoomOut()
{
	xMinView *= xZoom;//xMinView-(xMaxView-xMinView)/(2*xZoom);
	xMaxView *= xZoom;//xMaxView+(xMaxView-xMinView)/(2*xZoom);
	yMinView *= yZoom;//yMinView-(yMaxView-yMinView)/(2*yZoom);
	yMaxView *= yZoom;//yMaxView+(yMaxView-yMinView)/(2*yZoom);
	zMinView *= zZoom;//zMinView-(zMaxView-zMinView)/(2*zZoom);
	zMaxView *= zZoom;//zMaxView+(zMaxView-zMinView)/(2*zZoom);

	updateView();
	updateGL();
	//emitDimensionsChanged();
}

int GLBasicGraph::setDimension(GLfloat _xMin, GLfloat _xMax, GLfloat _yMin, GLfloat _yMax, GLfloat _zMin, GLfloat _zMax)
{
	if (_xMin < _xMax && _yMin < _yMax && _zMin < _zMax)
	{
		xMin = _xMin;
		xMax = _xMax;
		yMin = _yMin;
		yMax = _yMax;
		zMin = _zMin;
		zMax = _zMax;

		updateOrtho();
		updateGL();
		return 0;
	}
	else
		return 1;
}

void GLBasicGraph::emitDimensionsChanged()
{
	GLGraphEvent event(xMin,xMax,yMin,yMax,zMin,zMax);
	emit dimensionsChanged(event);
}

void GLBasicGraph::drawText(GLfloat x, GLfloat y, GLfloat z, const char *s)
{
	#if HAVE_GLUT
	glRasterPos3f(x, y, z);
	for (const char *c = s; *c != '\0'; c++)
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, *c);
	glRasterPos3f(-x, -y, -z);
	#endif //HAVE_GLUT
}

void GLBasicGraph::emitGraphRotated()
{
	emit graphRotated(GLGraphEvent(xRot,zRot));
}

void GLBasicGraph::exportAsImage(const QString &file, const QString &fileType)
{
	QImage image = grabFrameBuffer();
	image.save(file,fileType);
}

void GLBasicGraph::print(
	#ifdef KDE_APP
	KPrinter *printer
	#else
	QPrinter *printer
	#endif //KDE_APP
	)
{
	if ( printer->setup(this) ) // printer dialog
	{               
		qDebug( "Printing...");
		QPainter p;
		if ( !p.begin( printer ) )
			return;
		p.drawImage ( 0, 0, grabFrameBuffer());
		p.end();
		qDebug( "Printing completed");
	}
	else
		qDebug( "Printing aborted" );
}

void GLBasicGraph::updateOrtho()
{
	origin->setX( static_cast<int>(-xMin * _width / (xMax - xMin)) );
	origin->setY( static_cast<int>(-yMin * _height / (yMax - yMin)) );
	xScale = _width  / (xMax - xMin);
	yScale = _height / (yMax - yMin);

	orthoChanged();
}

void GLBasicGraph::updateView()
{
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();

	glOrtho(xMinView,xMaxView,yMinView,yMaxView,zMinView,zMaxView);
}

void GLBasicGraph::autoRotateX(bool on)
{
	rotating_x = on;

	if (on)
		rotate_timer->start( rotate_speed, FALSE );
	else if ( !rotating_y && !rotating_z )
		rotate_timer->stop();
}

void GLBasicGraph::autoRotateY(bool on)
{
	rotating_y = on;

	if (on)
		rotate_timer->start( rotate_speed, FALSE );
	else if ( !rotating_x && !rotating_z )
		rotate_timer->stop();
	
}

void GLBasicGraph::autoRotateZ(bool on)
{
	rotating_z = on;

	if (on)
		rotate_timer->start( rotate_speed, FALSE );
	else if ( !rotating_x && !rotating_y )
		rotate_timer->stop();
}

void GLBasicGraph::autoRotateStep()
{
	if (rotating_x)
		xRot += 1;
	if (rotating_y)
		yRot += 1;
	if (rotating_z)
		zRot += 1;

	emitGraphRotated();
	updateGL();
}

void GLBasicGraph::drawLetterPlus(double pos_x, double pos_y, double pos_z,
                                        double plus_height, double plus_width)
{
	glVertex3d( pos_x-(0.5*plus_width), pos_y , pos_z+(-0.5*plus_height));
	glVertex3d( pos_x-(0.5*plus_width), pos_y , pos_z+(0.5*plus_height));
	glVertex3d( pos_x                 , pos_y , pos_z);
	glVertex3d( pos_x-plus_width      , pos_y , pos_z);
}

void GLBasicGraph::drawLetterMinus(double pos_x, double pos_y, double pos_z,
                                         double minus_width)
{
	glVertex3d( pos_x             , pos_y , pos_z);
	glVertex3d( pos_x-minus_width , pos_y , pos_z);
}

void GLBasicGraph::drawLetterX(double pos_x, double pos_y, double pos_z,
                                 double x_height, double x_width)
{
	glVertex3d( pos_x + x_width, pos_y , pos_z+(-1*x_height));
	glVertex3d( pos_x          , pos_y , pos_z+x_height);
	glVertex3d( pos_x          , pos_y , pos_z+(-1*x_height));
	glVertex3d( pos_x + x_width, pos_y , pos_z+x_height);
}

void GLBasicGraph::drawLetterY(double pos_x, double pos_y, double pos_z,
                                 double y_height, double y_width)
{
	glVertex3d( pos_x+(0.5*y_width), pos_y, pos_z-(0.4*y_height));
	glVertex3d( pos_x+(0.5*y_width), pos_y, pos_z);
	glVertex3d( pos_x              , pos_y, pos_z+(0.6*y_height));
	glVertex3d( pos_x+(0.5*y_width), pos_y, pos_z);
	glVertex3d( pos_x+y_width      , pos_y, pos_z+(0.6*y_height));
	glVertex3d( pos_x+(0.5*y_width), pos_y, pos_z);
}

void GLBasicGraph::drawLetterZ(double pos_x, double pos_y, double pos_z,
                                 double z_height, double z_width)
{
	glVertex3d( pos_x + z_width, pos_y , pos_z+(-1*z_height));
	glVertex3d( pos_x          , pos_y , pos_z+(-1*z_height));
	glVertex3d( pos_x          , pos_y , pos_z+(-1*z_height));
	glVertex3d( pos_x + z_width, pos_y , pos_z+z_height);
	glVertex3d( pos_x + z_width, pos_y , pos_z+z_height);
	glVertex3d( pos_x          , pos_y , pos_z+z_height);
}

void GLBasicGraph::setMouseX(double x, bool snap)
{
	if (snapToGrid && snap)
	{
		double interval = snapIntervalX();
		double remainder = fmod(x - startSnap(), interval);
		if (remainder < 0)
		{
			if ( fabs(remainder) < (interval/2) )
				mouseX = x - remainder; //round up
			else
				mouseX = x - (interval + remainder); //round down
		}
		else
		{
			if ( fabs(remainder) < (interval/2) )
				mouseX = x - remainder; //round down
			else
				mouseX = x + (interval - remainder); //round up
		}
	}
	else
		mouseX = x;
}

void GLBasicGraph::setMouseY(double y, bool snap)
{
	if (snapToGrid && snap)
	{
		double interval = snapIntervalY();
		double remainder = fmod(y, interval);
		if (remainder < 0)
		{
			if ( fabs(remainder) < (interval/2) )
				mouseY = y - remainder;
			else
				mouseY = y - (interval + remainder);
		}
		else
		{
			if ( fabs(remainder) < (interval/2) )
				mouseY = y - remainder;
			else
				mouseY = y + (interval - remainder);
		}
	}
	else
		mouseY = y;
}




