/***************************************************************************
                          glcylindricalgraph.cpp  -  description
                             -------------------
    begin                : Mon Dec 2 2002
    copyright            : (C) 2002 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "glcylindricalgraph.h"

#include "fungparser.h"
#include "cartesian3dcoord.h"
#include "cylindricalcoord.h"
#include "fungmath.h"

GLCylindricalGraph::GLCylindricalGraph(QWidget *parent, const char *name ) : GLExpressionGraph(parent,name)
{
	radiusMin = 0.0;
	radiusMax = 10.0;
	thetaMin = 0.0;
	thetaMax = 6.0;
}
GLCylindricalGraph::~GLCylindricalGraph()
{
}

int GLCylindricalGraph::setThetaMin(double d)
{
	if (d < thetaMax)
	{
		thetaMin = d;
		recompileLists();
		updateGL();
		return 0;
	}
	else
		return 1;
}

int GLCylindricalGraph::setThetaMax(double d)
{
	if (d > thetaMin)
	{
		thetaMax = d;
		recompileLists();
		updateGL();
		return 0;
	}
	else
		return 1;
}

int GLCylindricalGraph::setRadiusMin(double d)
{
	if (d < radiusMax)
	{
		radiusMin = d;
		recompileLists();
		updateGL();
		return 0;
	}
	else
		return 1;
}

int GLCylindricalGraph::setRadiusMax(double d)
{
	if (d > radiusMin)
	{
		radiusMax = d;
		recompileLists();
		updateGL();
		return 0;
	}
	else
		return 1;
}

void GLCylindricalGraph::trace(GLfloat _r, GLfloat _t, FungParser &parser, GLfloat *x, GLfloat *y, GLfloat *z)
{
	double d[] = {_r,_t,animatorValue()};
	GLfloat _z = parser.Eval(d);
	
	CylindricalCoord s(_r,_t,_z);
	Cartesian3DCoord c = s.toCartesianCoord();

	emit activeCoordinateChanged(QString("radius = %1").arg(_r),QString("theta = %1").arg(_t),QString("z = %1").arg(_z));

	*x = c.x();
	*y = c.y();
	*z = c.z();
}

std::vector< std::vector<Cartesian3DCoord> > GLCylindricalGraph::getValues( FungParser &fp, GLfloat *min, GLfloat *max )
{
	qDebug("GLCylindricalGraph::getValues()");
	Cartesian3DCoord2DVector values;

	GLfloat radius, theta, z, current_min = 0.0, current_max = 0.0;
	int x_index, y_index;
	for (radius = radiusMin, x_index=0; radius < radiusMax-xScale; radius+=xScale, x_index++)
	{
		values.push_back( std::vector<Cartesian3DCoord>() );
		for (theta = thetaMin, y_index=0; theta<thetaMax-yScale; theta+=yScale, y_index++)
		{
			double d[] = {radius,theta,animatorValue()};
			z = fp.Eval(d);

			CylindricalCoord v1(radius,theta,z);

			current_min = MIN(current_min,z);
			current_max = MAX(current_max,z);

			values[x_index].push_back( Cartesian3DCoord(v1.toCartesianCoord()) );
		}
	}

	if ( min != 0 )
		*min = current_min;
	if ( max != 0 )
		*max = current_max;

	return values;
}

