/***************************************************************************
                          glexpressiongraph.cpp  -  description
                             -------------------
    begin                : Mon Dec 2 2002
    copyright            : (C) 2002-03 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 
#include "glexpressiongraph.h"

#include <cmath>

#include <qapp.h>
#include <qimage.h>

#include "fungparser.h"
#include "cartesian3dcoord.h"

GLExpressionGraph::GLExpressionGraph(QWidget *parent, const char *name) : GLBasicGraph(parent,name), Animator()
{
	expressionIndex = 0;

	angle = 0;

	drawConnected = true;
	showGrid = true;
	_gridColor = QColor(0,0,0);
	doTrace = false;

	x_percent_res = y_percent_res = 10;

	initializeAnimator( this );
}

GLExpressionGraph::~GLExpressionGraph()
{
	for( std::vector<Expression*>::const_iterator it = functions.begin(); it != functions.end(); it++ )
		delete (*it);
}

void GLExpressionGraph::resizeGL( int w, int h )
{
	GLBasicGraph::resizeGL( w, h );

	qDebug("resizing GLExpressionGraph( w = %d, h = %d",w,h); 

	setResolution(x_percent_res); //resolution depends upon the physical height and width of graph so update
}

void GLExpressionGraph::initializeGL()
{
	GLBasicGraph::initializeGL();
}

void GLExpressionGraph::paintGL()
{
	GLBasicGraph::paintGL(); //this will call glPushMatrix() without a coresponding glPopMatrix()

	glEnable( GL_POLYGON_OFFSET_FILL );
	glPolygonOffset( 1.0, 1.5 );

	for (unsigned int i=0; i<functions.size(); i++)
	{
		if (functions[i]->show)
		{
			//qglColor(functions[i]->color);
			glCallList(functions[i]->name);
			//glPushMatrix();
				//glLoadIdentity();
				//qglColor(functions[i]->color);
//				drawText(getXMin()+0.5f, getYMin()+i+0.5f, 0.0f, QString("z%1(x,y)=").arg(i+1)+functions[i]->expression);
			//glPopMatrix();

		}
	}

	if (isTracing())
	{
		glColor3f(0.0f,1.0f,0.0f);
		drawCube(lastX,lastY,lastZ,5.0f,5.0f);
	}
	
	glPopMatrix();
}

void GLExpressionGraph::drawCube(GLfloat x, GLfloat y, GLfloat z, GLfloat percentx, GLfloat percenty )
{
	GLfloat width  = fabs( getXMaxView() / ( _width /(getXMax() - getXMin()) ) ) * percentx / 100.0;
	GLfloat height = fabs( getYMaxView() / ( _height/(getYMax() - getYMin()) ) ) * percenty / 100.0;
	GLfloat depth = width;

	glBegin(GL_LINES);
		glVertex3f(x-width/2, y-height/2, z-depth/2); glVertex3f(x+width/2, y-height/2, z-depth/2);
		glVertex3f(x-width/2, y-height/2, z-depth/2); glVertex3f(x-width/2, y+height/2, z-depth/2);
		glVertex3f(x-width/2, y-height/2, z-depth/2); glVertex3f(x-width/2, y-height/2, z+depth/2);
		glVertex3f(x-width/2, y-height/2, z+depth/2); glVertex3f(x-width/2, y+height/2, z+depth/2);
		glVertex3f(x-width/2, y-height/2, z+depth/2); glVertex3f(x+width/2, y-height/2, z+depth/2);
		glVertex3f(x+width/2, y-height/2, z-depth/2); glVertex3f(x+width/2, y-height/2, z+depth/2);
		glVertex3f(x+width/2, y-height/2, z-depth/2); glVertex3f(x+width/2, y+height/2, z-depth/2);
		glVertex3f(x+width/2, y+height/2, z+depth/2); glVertex3f(x+width/2, y+height/2, z-depth/2);
		glVertex3f(x+width/2, y+height/2, z+depth/2); glVertex3f(x+width/2, y-height/2, z+depth/2);
		glVertex3f(x+width/2, y+height/2, z+depth/2); glVertex3f(x-width/2, y+height/2, z+depth/2);
		glVertex3f(x-width/2, y+height/2, z+depth/2); glVertex3f(x-width/2, y+height/2, z-depth/2);
		glVertex3f(x-width/2, y+height/2, z-depth/2); glVertex3f(x+width/2, y+height/2, z-depth/2);
	glEnd();
}

void GLExpressionGraph::recompileLists()
{
	for (unsigned int i=0; i<functions.size(); i++)
	{
		FungParser fp;
		std::string s = functions[i]->getExpression(dependent_var());
		fp.Parse(s,std::string(std::string(independent_vars())+",n").c_str(),dependent_var(),functions,!angle);
		compileList(functions[i]->name,fp,s,functions[i]->color);
	}
}

void GLExpressionGraph::compileList(GLuint name, FungParser &fp, std::string &, const QColor &c)
{
	GLfloat min, max;
	Cartesian3DCoord2DVector values = getValues(fp,&min,&max);

	if (isDrawConnected())
	{
		glNewList(name, GL_COMPILE);
			GLfloat z1, z2;
			for( unsigned int x = 0; x < values.size() - 1; x++ )
			{
				glBegin(GL_QUAD_STRIP);
					for ( unsigned int y = 0; y < values[x].size(); y++ )
					{
						z1 = values[x][y].z();
						doColorBlend(z1,c,min,max);
						glVertex3f(values[x][y].x(),values[x][y].y(),z1);
					
						z2 = values[x+1][y].z();
						doColorBlend(z2,c,min,max);
						glVertex3f(values[x+1][y].x(),values[x+1][y].y(),z2);
					}
				glEnd();
			}

			if (getGridState())
			{
				qglColor(gridColor());
				unsigned int x, y;
				for( x = 0; x < values.size() - 0; x++ )
				{
					glBegin(GL_LINES);
						for ( y = 0; y < values[x].size() - 0; y++ )
						{
							if ( x+1 < values.size())
							{
								glVertex3f(values[x][y].x(),values[x][y].y(),values[x][y].z());
								glVertex3f(values[x+1][y].x(),values[x+1][y].y(),values[x+1][y].z());
							}

							if ( y+1 < values[x].size() )
							{
								glVertex3f(values[x][y].x(),values[x][y].y(),values[x][y].z());
								glVertex3f(values[x][y+1].x(),values[x][y+1].y(),values[x][y+1].z());
							}
						}
					glEnd();
				}
			}
			/*glPushMatrix();
				glLoadIdentity();
				drawText(getXMin(), getYMax() - 0.5f, 0.0f, expression.c_str());
			glPopMatrix();*/
		glEndList();
	}
	else      //draw not connected
	{
		glNewList(name, GL_COMPILE);
			glBegin(GL_POINTS);
				qglColor(c);
				for( unsigned int x = 0; x < values.size(); x++ )
				{
					for ( unsigned int y = 0; y < values[x].size(); y++ )
					{
						glVertex3f(values[x][y].x(),values[x][y].y(),values[x][y].z());
					}
				}
			glEnd();
		glEndList();
	}
}

void GLExpressionGraph::addExpression(const Expression & expression)
{
	FungParser fp;

	std::string s = expression.getExpression(dependent_var());
	int ret = fp.Parse(s,std::string(std::string(independent_vars())+",n").c_str(),dependent_var(),functions,!angle);

	if (ret == -1)
	{
		emit infoMessageReady("");
                        
		Expression *node = new Expression(s,expression.show,expression.color,glGenLists(1));

		if (functions.size() < expressionIndex + 1)
			functions.resize(functions.size()+1);
		else
		{
			//glDeleteLists(functions[expressionIndex]->name,1);
			delete functions[expressionIndex];
		}

		functions[expressionIndex]=node;

		compileList(functions[expressionIndex]->name,fp,s,functions[expressionIndex]->color);

		updateGL();
	}
	else
		emit infoMessageReady(fp.ErrorMsg());
}

void GLExpressionGraph::removeExpression(unsigned int index)
{
	if (functions.size() >= index + 1)
	{
		delete functions[index]; //delete the actual value in the pointer, functions[index]
		functions.erase(functions.begin()+index); //remove the pointer to functions[index] from the vector
	}

	updateGL();
}

void GLExpressionGraph::prevExpression()
{
	if ( expressionIndex == 0 )
		expressionIndex = functions.size();
	else
		expressionIndex--;

	updateGL();
}

void GLExpressionGraph::nextExpression()
{
	if ( expressionIndex == functions.size() )
		expressionIndex = 0;
	else
		expressionIndex++;

	updateGL();
}

void GLExpressionGraph::setShowCurrentExpression(bool state)
{
	if ( expressionIndex < functions.size() )
	{
		functions[expressionIndex]->show = state;
		updateGL();
	}
}

void GLExpressionGraph::orthoChanged()
{
	xScale = (getXMax()-getXMin())/xRes;
	yScale = (getYMax()-getYMin())/yRes;

	recompileLists();
}

void GLExpressionGraph::mouseMoveEvent(QMouseEvent *m)
{
	if ( isTracing() && ( functions.size() > expressionIndex ) )
	{
		setMouseX(toGraphXCoord(m->x()));
		setMouseY(toGraphYCoord(m->y()));

		FungParser fp;

		std::string s = functions[expressionIndex]->getExpression(dependent_var());
		int ret = fp.Parse(s,std::string(std::string(independent_vars())+",n").c_str(),dependent_var(),functions,!angle);
		if (ret != -1){return;}

		trace(
			traceDepX(getMouseX()),
			traceDepY(getMouseY()),
			fp,
			&lastX,&lastY,&lastZ
		);
		updateGL();
	}
	GLBasicGraph::mouseMoveEvent(m);
}

void GLExpressionGraph::setResolution(int percent)
{
	x_percent_res = y_percent_res = percent;

	if (percent < 0)
		percent = 0;
	if (percent > 100)
		percent = 100;

	setXRes(percent/100.0f*_width);
	setYRes(percent/100.0f*_height);

	recompileLists();
	updateGL();
}

int GLExpressionGraph::getResolution() const
{
	return x_percent_res;
}

void GLExpressionGraph::doColorBlend(GLfloat f, const QColor & c, GLfloat z_min, GLfloat z_max)
{
	glColor3f( (static_cast<double>(c.red())/255.0f) *((f-z_min)/(z_max-z_min)), (static_cast<double>(c.green())/255.0f) *((f-z_min)/(z_max-z_min)), (static_cast<double>(c.blue())/255.0f) *((f-z_min)/(z_max-z_min)) );
}

void GLExpressionGraph::nextFrameReady()
{
	emit animationValueChanged(animatorValue());
	recompileLists();
	updateGL();
	
	/*if (isSaving())
	{
		QImage image = grabFrameBuffer();
		image.save(QString("/home/fungmeista/%1.png").arg(time(0)),"PNG");
	}*/
}

void GLExpressionGraph::exportToDXF( const char * file )
{
	emit infoMessageReady("Exporting to DXF...");

	std::ofstream output;
	output.open(file);
	
	output << "0\n";
	output << "SECTION\n";
	output << "2\n";
	output << "ENTITIES\n";	
	
	for ( unsigned int i = 0; i < functions.size(); i++ )
	{
		if ( functions[i]->show )
		{
			FungParser fp;

			std::string s = functions[i]->getExpression(dependent_var());
			int ret = fp.Parse(s,std::string(std::string(independent_vars())+",n").c_str(),dependent_var(),functions,!angle);

			Cartesian3DCoord2DVector values = getValues(fp);
			exportSingleExprDXF( values,  &output, i );
		}
	}

	output << "0\n";
	output << "ENDSEC\n";
	output << "0\n";
	output << "EOF\n";
	
	output.close();
	
	emit infoMessageReady("Exporting to DXF... done");
}

void GLExpressionGraph::exportSingleExprDXF( const Cartesian3DCoord2DVector &values, std::ofstream *output, int layer )
{
	for( unsigned int x = 0; x < values.size() - 1; x++ )
	{
		for ( unsigned int y = 0; y < values[x].size() - 1; y++ )
		{
			*output << "0\n";
			*output << "3DFACE\n";
			*output << "8\n"; //layer specifier
			*output << "Fung-Calc " << dependent_var() << char('1'+layer) << "\n"; //layer name
			*output << "62\n"; //color specifier
			*output << "7\n"; //color index

			/* first vertex */
			*output << "10\n";
			*output << values[x][y].x() << "\n"; //x
			*output << "20\n";    
			*output << values[x][y].y() << "\n"; //y
			*output << "30\n";
			*output << values[x][y].z() << "\n"; //z

			/* second vertex */
			*output << "11\n";
			*output << values[x+1][y].x() << "\n"; 
			*output << "21\n";
			*output << values[x+1][y].y() << "\n";
			*output << "31\n";
			*output << values[x+1][y].z() << "\n";

			/* third vertex */
			*output << "12\n";
			*output << values[x+1][y+1].x() << "\n";
			*output << "22\n";
			*output << values[x+1][y+1].y() << "\n";
			*output << "32\n";
			*output << values[x+1][y+1].z() << "\n";

			/* fourth vertex */
			*output << "13\n";
			*output << values[x][y+1].x() << "\n";
			*output << "23\n";
			*output << values[x][y+1].y() << "\n";
			*output << "33\n";
			*output << values[x][y+1].z() << "\n";
		}
		qApp->processEvents();
		emit infoMessageReady("Exporting to DXF...");
	}
}
