/***************************************************************************
                          glpolargraph.cpp  -  description
                             -------------------
    begin                : Sun Dec 1 2002
    copyright            : (C) 2002 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "glpolargraph.h"

#include <cmath>

#include "fungparser.h"
#include "sphericalcoord.h"
#include "cartesian3dcoord.h"
#include "fungmath.h"

GLPolarGraph::GLPolarGraph(QWidget *parent, const char *name) : GLExpressionGraph(parent,name)
{
	tMin = 0.0f;
	tMax = 6.30f;

	aMin = 0.0f;
	aMax = 6.30f;
}

GLPolarGraph::~GLPolarGraph()
{
}

int GLPolarGraph::setPolarMin(GLfloat d)
{
	if (d < tMax)
	{
		tMin = d;
		recompileLists();
		updateGL();
		return 0;
	}
	else
		return 1;
}

int GLPolarGraph::setPolarMax(GLfloat d)
{
	if (d > tMin)
	{                 
		tMax = d;
		recompileLists();
		updateGL();
		return 0;
	}
	else
		return 1;
}

int GLPolarGraph::setAzimuthalMin(GLfloat d)
{
	if (d < aMax)
	{
		aMin = d;
		recompileLists();
		updateGL();
		return 0;
	}
	else
		return 1;
}

int GLPolarGraph::setAzimuthalMax(GLfloat d)
{
	if (d > aMin)
	{
		aMax = d;
		recompileLists();
		updateGL();
		return 0;
	}
	else
		return 1;
}

void GLPolarGraph::trace(GLfloat _a, GLfloat _p, FungParser &parser, GLfloat *x, GLfloat *y, GLfloat *z)
{
	double d[] = {_a,_p,animatorValue()};
	GLfloat _r = parser.Eval(d);

	SphericalCoord s(_a,_p,_r);
	Cartesian3DCoord c = s.toCartesianCoord();

	emit activeCoordinateChanged(QString("azimuthal = %1").arg(_a),QString("polar = %1").arg(_p),QString("radial = %1").arg(_r));
        
	*x = c.x();
	*y = c.y();
	*z = c.z();
}

std::vector< std::vector<Cartesian3DCoord> > GLPolarGraph::getValues( FungParser &fp, GLfloat *min, GLfloat *max )
{
	qDebug("GLPolarGraph::getValues()");
	Cartesian3DCoord2DVector values;

	GLfloat azimuthal, polar, radial, current_min = 0.0, current_max = 0.0;
	int x_index, y_index;
	for (azimuthal = aMin, x_index=0; azimuthal < aMax - xScale; azimuthal+=xScale, x_index++)
	{
		values.push_back( std::vector<Cartesian3DCoord>() );
		for (polar = tMin, y_index=0; polar < tMax - yScale; polar+=yScale, y_index++)
		{
			double d1[] = {azimuthal,polar,animatorValue()};
			radial = fp.Eval(d1);

			SphericalCoord v1(azimuthal,polar,radial);

			float z = v1.toCartesianCoord().z();

			current_min = MIN(current_min,z);
			current_max = MAX(current_max,z);

			values[x_index].push_back( v1.toCartesianCoord() );
		}
	}

	if ( min != 0 )
		*min = current_min;
	if ( max != 0 )
		*max = current_max;

	return values;
}





