/***************************************************************************
                          animator.cpp  -  description
                             -------------------
    begin                : Sat Mar 29 2003
    copyright            : (C) 2003 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "animator.h"

#include <qobject.h>
#include <qtimer.h>

Animator::Animator()
{
	min = 0;
	max = 10;
	step = 0.1;
	speed = 40;

	_displayNValue = true;

	current = min;
}

Animator::~Animator()
{
}

void Animator::initializeAnimator( QObject *parent )
{
	_timer = new QTimer( parent ); //auto-deleted

	parent->connect( _timer, SIGNAL(timeout()), parent, SLOT(slotNextFrame()) );
}

void Animator::play()
{
	_save = true;
	_timer->start( speed, FALSE );
}

void Animator::stop()
{
	_timer->stop();

	current = min;
	nextFrameReady();
}

void Animator::pause()
{
	if ( _timer->isActive() )
		_timer->stop();
	else
		play();
}

int Animator::setAnimationSpeed(int ms)
{
	if ( ms > 0 )
	{
		bool is_active = _timer->isActive(); /* calling changeInterval() will play the graph... record this to set it back how it was */
	
		speed = ms;
		_timer->changeInterval(speed);
		if (!is_active) _timer->stop();
		
		return 0;
	}
	else
		return 1;
	
}

int Animator::setAnimationMin(double d)
{
	if ( d < max )
	{
		min = d;
		return 0;
	}
	else
		return 1;
}

int Animator::setAnimationMax(double d)
{
	if ( d > min )
	{
		max = d;
		return 0;
	}
	else
		return 1;
}

int Animator::setAnimationStep(double d)
{
	if ( d > 0 )
	{
		step = d;
		return 0;
	}
	else
		return 1;
}

void Animator::nextFrame()
{
	if ( current >= max )
	{
		current = min; // loop back around
		_save = false; //stop saving if currently are
	}
	else
	{
		current += step;
		if ( current > max )
			current = max;
	}
	
	nextFrameReady();
}

void Animator::saveAsMovie()
{
	_save = true;
}


