/***************************************************************************
                          fungparser.cpp  -  wrapper for fparser to add a few extra functions
                             -------------------
    begin                : Wed Oct 9 2002
    copyright            : (C) 2002-03 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef NO_QT
#include <qregexp.h>
#endif //NO_QT

#include "fungparser.h"

FungParser::FungParser() : FunctionParser()
{
}
FungParser::~FungParser()
{
}

int FungParser::Parse( std::string & _string, const char * vars, const char * name, std::vector<Expression*> &expressions, bool isDegrees)
{
	AddConstant("pi",3.14159265358979323);
	AddConstant("e",2.718281828);

	QString varss = QString(vars).mid(0,strlen(vars)-2); //ignore animator value

	QStringList variables;

	int t = -1;
	variables<<varss.mid(0,varss.find(','));
	while ((t = varss.find(',',t+1)) != -1) 		//extract all variables from "vars" which are seperated by commas and
		variables<<varss.mid(t+1,varss.find(',',t+1));//store them in the string list
	
	QString result(_string.c_str());

	bool recurse = false;

	for (unsigned int i=0; i<expressions.size(); i++)
	{
		int begin = result.find(QRegExp(QString(name)+QString("%1\\(").arg(i+1)));
		if (begin == -1){continue;}
		else {recurse = true;}
		begin += 1 + QString::number(i+1).length();

		int left_p = 0, right_p = 0, n = 0;

		if (result.contains("(") != result.contains(")")){ParseErrorType=1; return 1;} //can't parse this, bad syntax
		while (true)
		{
			if (QString::compare(result.mid(begin+n,1),"(") == 0)
				left_p++;
			if (QString::compare(result.mid(begin+n,1),")") == 0)
				right_p++;
			if (left_p == right_p)
				break;
			n++;
		}

		QString args = result.mid(begin+1,n-1);

		QStringList arguments;
		t = -1;
		arguments<<args.mid(0,args.find(','));
		while ((t = args.find(',',t+1)) != -1)
			arguments<<args.mid(t+1,args.find(',',t+1));

		if (QString(QString(expressions[i]->getExpression(name).c_str())+"(").contains(QString(name)+QString("%1(").arg(i+1))) //prevent infinite recursion
			return FunctionParser::Parse(result.latin1(),vars,isDegrees);
		if (arguments.size() != variables.size() ){ParseErrorType=8; return 8;}  //wrong number of parameters

		QString subfunction = "("+QString(expressions[i]->getExpression(name).c_str())+")";

		QStringList::Iterator var_it = variables.begin();
		for ( QStringList::Iterator arg_it = arguments.begin(); arg_it != arguments.end(); ++arg_it)
		{
			subfunction = subfunction.replace(QRegExp(QString("%1(?![a-z])").arg(*var_it)),*arg_it); //don't match functions that contain possible variables (tan(), exp(), etc.)
			++var_it;
		}

		result = result.replace(begin-1-QString::number(i+1).length(),n+2+QString::number(i+1).length(),subfunction);
	}
	if (!recurse)  //function has been parsed to be recognized by base parser, now give it to the base parser
		return FunctionParser::Parse( result.latin1(), vars, isDegrees );
	else
	{
		qDebug("calling FunctionParser::parse() recursively");
		std::string s(result.latin1()); 
		return Parse( s, vars, name, expressions, isDegrees );  //still has FungParser specifics - can't send to base parser - continue parsing
	}

	#if 0
	AddConstant("pi",3.14159265358979323);
	AddConstant("e",2.718281828);

	for ( unsigned int i = 0; i < expressions.size(); i++ )
	{
		FungParser *fp = expressions[i]->getParsedExpression(name);
		std::string func_name = QString("%1%2").arg(name).arg(i).latin1();
		AddFunction(func_name,*fp);
	}

	return FunctionParser::Parse(_string,vars,isDegrees);
	#endif


}
