/***************************************************************************
                          fungvector.cpp  -  description
                             -------------------
    begin                : Thu Nov 7 2002
    copyright            : (C) 2002 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <cmath>
 
#include "fungvector.h"
#include "sphericalcoord.h"
#include "cylindricalcoord.h"
#include "cartesian3dcoord.h"


FungVector::FungVector(float x, float y, float z)
{
	this->x = x;
	this->y = y;
	this->z = z;
}

FungVector::FungVector(SphericalCoord &s)
{
	Cartesian3DCoord c = s.toCartesianCoord();

	this->x = c.x();
	this->y = c.y();
	this->z = c.z();
}

FungVector::FungVector(CylindricalCoord &s)
{
	Cartesian3DCoord c = s.toCartesianCoord();

	this->x = c.x();
	this->y = c.y();
	this->z = c.z();
}


FungVector::~FungVector()
{

}

void FungVector::toArray(float vec[3])
{
	vec[0] = x;
	vec[1] = y;
	vec[2] = z;
}

FungVector FungVector::normal(FungVector p1, FungVector p2, FungVector p3)
{
	FungVector ret(0,0,0);
	cross_product(p1,p2,p3,&ret);
	normalize(&ret);
	return ret;
}

void FungVector::cross_product(FungVector &u, FungVector &v, FungVector &w, FungVector *result)
{
	float v1[3];
	float v2[3];

	v1[0] = u.x - v.x;
	v1[1] = u.y - v.y;
	v1[2] = u.z - v.z;

	v2[0] = v.x - w.x;
	v2[1] = v.y - w.y;
	v2[2] = v.z - w.z;

	result->x = v1[1]*v2[2] - v1[2]*v2[2];
	result->y = v1[2]*v2[0] - v1[0]*v2[2];
	result->z = v1[0]*v2[1] - v1[1]*v2[0];
}

float FungVector::magnitude(FungVector &v)
{
	return sqrt(v.x*v.x+v.y*v.y+v.z*v.z);
}

void FungVector::normalize(FungVector *v)
{                                                                                               
	float length = magnitude(*v);
	v->x /= length;
	v->y /= length;
	v->z /= length;
}

