/***************************************************************************
                          arclengthfunction.cpp  -  description
                             -------------------
    begin                : Mon Nov 11 2002
    copyright            : (C) 2002 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "arclengthfunction.h"

#include <cmath>

#include <qpointarray.h>
#include <qpainter.h>
#include <qobject.h>

#include "fungmath.h"
#include "fungparser.h"

ArcLengthFunction::ArcLengthFunction(BasicGraph *graph, int expressions) : MathFunction(graph,expressions)
{
	pointsStorage = new QPointArray();
}
ArcLengthFunction::~ArcLengthFunction()
{
	delete pointsStorage;
}

void ArcLengthFunction::calculate_and_draw(QPainter *painter)
{
	if ( expression2 ) /* parametric graph */
	{
		if ( arg1 > arg2 ) //bounds check
		{
			QString s( QObject::tr("Invalid bounds") );
			setResult( s );
			return;
		}
	
		double step = 0.1;
		qDebug("arg1: %f, arg2: %f",arg1,arg2);
		pointsStorage->resize( static_cast<int>(ROUND((arg2-arg1)/step) + 2) );
		double distance = 0;
		int i = 0;
		for (double t=arg1; t<arg2; t+=step)
		{
			double alpha[] = {t,animatorValue()};
			double alpha2[] = {t+step,animatorValue()};

			double valuex = expression->Eval(alpha);
			double valuex2 = expression->Eval(alpha2);

			double valuey = expression2->Eval(alpha);
			double valuey2 = expression2->Eval(alpha2);

			(*pointsStorage)[i]=QPoint(toPixelXCoord(valuex),toPixelYCoord(valuey));
			(*pointsStorage)[i+1]=QPoint(toPixelXCoord(valuex2),toPixelYCoord(valuey2));

			distance += sqrt(
				((valuex-valuex2)*(valuex-valuex2))+
				((valuey-valuey2)*(valuey-valuey2))
				);

			painter->drawLine(toPixelXCoord(valuex),toPixelYCoord(valuey),toPixelXCoord(valuex2),toPixelYCoord(valuey2));
			
			i++;
		}
		QString s = QString("Arc Length = %1").arg(distance);
		setResult(s);
	}
	else
	{
		if ( lowerBounds.x() > upperBounds.x() ) //bounds check
		{
			QString s( QObject::tr("Invalid bounds") );
			setResult( s );
			return;
		}
	
		pointsStorage->resize(upperBounds.x()-lowerBounds.x() + 1);
		double distance = 0;
		for (int i=lowerBounds.x(); i<upperBounds.x(); i++)
		{
			double alpha[] = {toGraphXCoord(i),animatorValue()};
			double alpha2[] = {toGraphXCoord(i+1),animatorValue()};

			double value = expression->Eval(alpha);
			double value2 = expression->Eval(alpha2);

			(*pointsStorage)[i-lowerBounds.x()]=QPoint(i,toPixelYCoord(value));
			(*pointsStorage)[i+1-lowerBounds.x()]=QPoint(i+1,toPixelYCoord(value2));

			distance += sqrt(
				((alpha[0]-alpha2[0])*(alpha[0]-alpha2[0]))+
				((value-value2)*(value-value2))
				);

			painter->drawLine(i,toPixelYCoord(value),i+1,toPixelYCoord(value2));
		}
		QString s = QString("Arc Length = %1").arg(distance);
		setResult(s);
	}
}

void ArcLengthFunction::draw_stored(QPainter *painter)
{
	painter->drawPolyline(*pointsStorage);
}


