/***************************************************************************
                          definiteintegralfunction.cpp  -  description
                             -------------------
    begin                : Mon Nov 11 2002
    copyright            : (C) 2002 by Fungmeista
    email                : mizunoami44@sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "definiteintegralfunction.h"

#include <cmath>

#include <qpainter.h>
#include <qpointarray.h>
#include <qobject.h>

#include "fungparser.h"
#include "fungmath.h"

DefiniteIntegralFunction::DefiniteIntegralFunction(BasicGraph *graph, int expressions, double _precision) : MathFunction(graph,expressions), precision(_precision)
{
	pointsStorage = new QPointArray;
}

DefiniteIntegralFunction::~DefiniteIntegralFunction()
{
	delete pointsStorage;
}

void DefiniteIntegralFunction::calculate_and_draw(QPainter *painter)
{
	if ( lowerBounds.x() > upperBounds.x() ) //bounds check
	{
		QString s( QObject::tr("Invalid bounds") );
		setResult( s );
		return;
	}

	pointsStorage->resize(upperBounds.x()-lowerBounds.x()+4); //add 1 so that both lower and upper bounds are included and another 2 so that the base of the figure is included

	double area = 0;
	(*pointsStorage)[upperBounds.x()-lowerBounds.x()+3] = QPoint(upperBounds.x(),toPixelYCoord(0));
	for (int i=lowerBounds.x(); i<=upperBounds.x(); i++)
	{
		//use traprule to calculate definite integral
		double height = precision; //smaller height the more precise the answer
		double alpha[] = {toGraphXCoord(i),animatorValue()};
		double alpha2[] = {toGraphXCoord(i+height),animatorValue()};

		double baseone = expression->Eval(alpha);
		double basetwo = expression->Eval(alpha2);
		area += 0.5*(baseone+basetwo)*(toGraphXCoord(height)-toGraphXCoord(0));

		(*pointsStorage)[i+1-lowerBounds.x()]=QPoint(i,toPixelYCoord(baseone));
		(*pointsStorage)[i+2-lowerBounds.x()]=QPoint((int)(ROUND(i+height)),toPixelYCoord(basetwo));
	}
	(*pointsStorage)[0] = QPoint(lowerBounds.x(),toPixelYCoord(0));
	draw_stored(painter);
	
	QString s = QString("Definite Integral = %1").arg(area);
	setResult(s);
}

void DefiniteIntegralFunction::draw_stored(QPainter *painter)
{
	painter->save();
	painter->setBrush(QBrush(QColor(255,0,0),Qt::Dense4Pattern));
	painter->drawPolygon(*pointsStorage);
	painter->restore();
}


