/***************************************************************************
                          mathfunction.cpp  -  description
                             -------------------
    begin                : Mon Nov 11 2002-03
    copyright            : (C) 2002 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "mathfunction.h"

#include <qapplication.h> //used to continue processing events while waiting for the points
#include <qpoint.h>
#include <qpainter.h>

#include "expressiongraph.h"
#include "fungparser.h"

MathFunction::MathFunction(BasicGraph *graph, int expressions) : expression_count(expressions), expression(0), expression2(0)
{
	mouseClicks = 0;
	resultString = QString();
	_active = false;

	basicgraph = graph;
	
	upperBounds = QPoint();
	lowerBounds = QPoint();
}

MathFunction::~MathFunction()
{
}

void MathFunction::setResult( QString & s )
{
	resultString = s;
}

int MathFunction::toPixelXCoord(double x_coordinate)
{
	return basicgraph->toPixelXCoord(x_coordinate);
}

int MathFunction::toPixelYCoord(double y_coordinate)
{
	return basicgraph->toPixelYCoord(y_coordinate);
}

double MathFunction::toGraphXCoord(double x_coordinate)
{
	return basicgraph->toGraphXCoord(x_coordinate);
}

double MathFunction::toGraphYCoord(double y_coordinate)
{
	return basicgraph->toGraphYCoord(y_coordinate);
}

double MathFunction::animatorValue()
{
	return static_cast<ExpressionGraph*>(basicgraph)->animatorValue();
}


void MathFunction::getTwoPoints()
{
	qApp->processOneEvent();
	if (mouseClicks == CANCEL){return;}
	if (mouseClicks == GETLOWERBOUND){getTwoPoints();return;}
	if (mouseClicks == GOTLOWERBOUND)
	{
		lowerBounds.setX( lastClick.x());
		lowerBounds.setY( lastClick.y());
		arg1 = beta;

		arg1x = beta;
		arg2y = beta2;

		basicgraph->repaint(false);
		mouseClicks++;
	}
	if (mouseClicks == GETUPPERBOUND){getTwoPoints();return;}
	if (mouseClicks == GOTUPPERBOUND)
	{
		upperBounds.setX(lastClick.x());
		upperBounds.setY(lastClick.y());
		arg2 = beta;
		
		arg2x = beta;
		arg2y = beta2;
		
		//if (upperBounds.x() <= lowerBounds.x()){basicgraph->repaint(false); return;}
		if (expression){basicgraph->setTrace(false);}
		basicgraph->repaint(false); mouseClicks++;
	}
	if (mouseClicks == CALCULATE){basicgraph->repaint(false); mouseClicks++;}
	if (mouseClicks == CLICKTOCONTINUE){getTwoPoints(); return;}

	basicgraph->repaint(false);
}

void MathFunction::sendClick(QMouseEvent *, double arg, double arg2)
{
	lastClick = basicgraph->getMousePoint();

	beta = arg;
	beta2 = arg2;
	mouseClicks++;
}

void MathFunction::drawFunction(QPainter *painter)
{
	painter->setPen(QPen(QColor(255,0,0),2));

	if (mouseClicks == 0){painter->drawText(30,basicgraph->height()-20,QObject::tr("Select Lower Bounds"));}
	if (mouseClicks == 1 || mouseClicks == 2){drawLeftArrow(painter); painter->drawText(30,basicgraph->height()-20,QObject::tr("Select Upper Bounds"));}
	if (mouseClicks == 4)
	{
		calculate_and_draw(painter);

		drawLeftArrow(painter);
		drawRightArrow(painter);

		painter->drawText(LEFTOFFSET,basicgraph->height()-24,resultString);
		painter->drawText(LEFTOFFSET,basicgraph->height()-10,QObject::tr("Click to continue..."));
	}
	if (mouseClicks == 5)
	{
		draw_stored(painter);

		drawLeftArrow(painter);
		drawRightArrow(painter);

		painter->drawText(LEFTOFFSET,basicgraph->height()-24,resultString);
		painter->drawText(LEFTOFFSET,basicgraph->height()-10,QObject::tr("Click to continue..."));
	}
}

void MathFunction::exec(FungParser *fp, FungParser *fp2)
{
	bool tmpTrace = false;
	if (fp)
	{
		expression = fp;
		tmpTrace = basicgraph->isTracing();
		basicgraph->setTrace(true);
	}

	if (fp2)
	{
		qDebug("is fp2");
		expression2 = fp2;
	}

	_active = true;
	mouseClicks = GETLOWERBOUND;
	basicgraph->repaint(false);
	getTwoPoints();

	if (fp)
		basicgraph->setTrace(tmpTrace);

	_active = false;
	
	expression = 0;
	expression2= 0;
}

void MathFunction::cancel()
{
	mouseClicks = CANCEL;
}

void MathFunction::drawSelectedPoint(double x, double y, QPainter *painter)
{
	painter->save();
	painter->setPen(QPen(QColor(100,200,0),1,Qt::DotLine));
	painter->drawLine(toPixelXCoord(x),0,toPixelXCoord(x),basicgraph->height());
	painter->drawLine(0,toPixelYCoord(y),basicgraph->width(),toPixelYCoord(y));
	painter->fillRect(toPixelXCoord(x)-3,toPixelYCoord(y)-3,6,6,QBrush(QColor(0,100,255),Qt::SolidPattern));
	painter->restore();
}

void MathFunction::drawLeftArrow(QPainter *painter)
{
	int x = lowerBounds.x();
	int y = lowerBounds.y();

	QPointArray points(3);
	points.setPoint(0,x,y);
	points.setPoint(1,x-BOUNDS_TRIANGLE*2,y-BOUNDS_TRIANGLE);
	points.setPoint(2,x-BOUNDS_TRIANGLE*2,y+BOUNDS_TRIANGLE);

	painter->save();
	painter->setPen(QPen(QColor(0,100,255),2));
	painter->setBrush(QBrush(QColor(0,100,255),Qt::SolidPattern));
	painter->drawPolygon(points);
	painter->restore();
}

void MathFunction::drawRightArrow(QPainter *painter)
{
	int x = upperBounds.x();
	int y = upperBounds.y();

	QPointArray points(3);
	points.setPoint(0,x,y);
	points.setPoint(1,x+BOUNDS_TRIANGLE*2,y-BOUNDS_TRIANGLE);
	points.setPoint(2,x+BOUNDS_TRIANGLE*2,y+BOUNDS_TRIANGLE);

	painter->save();
	painter->setPen(QPen(QColor(0,100,255),2));
	painter->setBrush(QBrush(QColor(0,100,255),Qt::SolidPattern));
	painter->drawPolygon(points);
	painter->restore();
}
