/***************************************************************************
                          mathfunctionimplementor.h  -  description
                             -------------------
    begin                : Thu Feb 27 2003
    copyright            : (C) 2003 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _MATHFUNCTIONIMPLEMENTOR_H_
#define _MATHFUNCTIONIMPLEMENTOR_H_

#include <map>
#include <string>

class MathFunction;
class QMouseEvent;
class QPainter;

/**
 *  This class should be subclassed to allow a class to easily use MathFunction classes.  The subclass must implement installMathFunctions() and add any MathFunction's to the
 *  math_functions vector.  Also, call drawMathFunctions(QPainter*) when painting the widget to get the visual for the installed math functions.
 */
class MathFunctionImplementor
{
	typedef std::map<std::string,MathFunction*> MathFunctionMap;
	typedef std::pair<std::string,MathFunction*> MathFunctionPair;

	public:
		MathFunctionImplementor();
		virtual ~MathFunctionImplementor();

		/** Implement this function to install any MathFunction's to a specific class.  A math function should be added by calling
		 * addMathFunction( MathFunction*, const char * id ) Example: addMathFunction(new DistanceMathFunction(this), "Distance"); 
		 */
		virtual void installMathFunctions(){}

	protected:
		/** Call this function when drawing the widget to get the visual for the installed math functionss */
		virtual void drawMathFunctions(QPainter *);
		
		/** If a math function has been exec()'ed and is awaiting points to be selected, this will cancel it and return the widget to how it was. */
		virtual void cancelMathFunctions();
		
		/** Subclasses give a pointer to the MathFunction to add/install.  This class will delete it */
		void addMathFunction( MathFunction *, const char * id );
		
		/** Returns the installed MathFunction with the given id */
		MathFunction* getMathFunction( const char * id );
		
		/** Implement this if a certain math function needs more arguments other than mouse coodinates */
		virtual void sendCustomClick(QMouseEvent *, MathFunction *, double x, double y);
		
		virtual int mathFunctionCount(){return math_functions.size();}
		
		/** Call this function when the mouse is clicked and pass it the generated
		  * QMouseEvent pointer and the actual x and y coordinates of where the mouse
		  * is.
		  */
		void mathFunctionClick(QMouseEvent *e, double x, double y);

	private:
		MathFunctionMap math_functions;
};

#endif
