/***************************************************************************
                          meanvaluefunction.cpp  -  description
                             -------------------
    begin                : Sun Dec 29 2002
    copyright            : (C) 2002 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "meanvaluefunction.h"

#include <qpainter.h>
#include <qpoint.h>
#include <qobject.h>

#include "basicgraph.h"
#include "fungparser.h"

MeanValueFunction::MeanValueFunction(BasicGraph *graph,int expressions) : MathFunction(graph,expressions)
{
}
MeanValueFunction::~MeanValueFunction()
{
}

void MeanValueFunction::calculate_and_draw(QPainter* painter)
{
	if ( lowerBounds.x() > upperBounds.x() ) //bounds check
	{
		QString s( QObject::tr("Invalid bounds") );
		setResult( s );
		return;
	}

	found = false;
	double slope = (toGraphYCoord(upperBounds.y()) - toGraphYCoord(lowerBounds.y())) / (toGraphXCoord(upperBounds.x()) - toGraphXCoord(lowerBounds.x()));
	for(int i=lowerBounds.x(); i<=upperBounds.x(); i++)
	{
		double x = toGraphXCoord(i);
		double x_next = toGraphXCoord(i+1);

		double d[] = {x,animatorValue()};
		double d_next[] = {x_next,animatorValue()};

		double y = expression->Eval(d);
		double y_next = expression->Eval(d_next);

		double slope_search = (y - y_next) / (x - x_next);
		if ( (slope_search <= slope + 0.1) && (slope_search >= slope - 0.1) )
		{
			point_x = x;
			point_y = y;
			found = true;
			break;
		}
	}

	double b = -(slope * point_x - point_y);

	double _x1 = toGraphXCoord(0);
	double _y1 = _x1*slope + b;

	double _x2 = toGraphXCoord(basicgraph->width());
	double _y2 = _x2*slope + b;

	x1 = toPixelXCoord(_x1);
	y1 = toPixelYCoord(_y1);
	x2 = toPixelXCoord(_x2);
	y2 = toPixelYCoord(_y2);

	if (found)
	{
		QString s = QString(QObject::tr("Mean Value = %1")).arg(point_x);
		setResult(s);
	}
	else
	{
		QString s = QObject::tr("None found");
		setResult(s);
	}

	draw_stored(painter);
}

void MeanValueFunction::draw_stored(QPainter* painter)
{
	if (found)
	{
		painter->drawLine(x1,y1,x2,y2); //draw a line tangent to the point found
		drawSelectedPoint(point_x,point_y,painter);
	}
}
