/***************************************************************************
                          basicgraphmain.cpp  -  description
                             -------------------
    begin                : Mon Feb 8 2003
    copyright            : (C) 2003 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "basicgraphmain.h"
 
#include <qlineedit.h>
#include <qpalette.h>
#include <fstream>

#include "basicgraph.h"
#include "graphevent.h"

BasicGraphMain::BasicGraphMain( QWidget* parent, const char* name ) : CommonGraphMain( parent, name )
{
	if ( !name )
	setName( "BasicGraphMain" );
}

BasicGraphMain::~BasicGraphMain()
{

}

void BasicGraphMain::setGraph( BasicGraph *b )
{
	graph = b;

	connect(graph,SIGNAL(dimensionsChanged(const GraphEvent&)),this,SLOT(dimensionsChanged(const GraphEvent&)));
	connect(graph,SIGNAL(infoMessageReady(const QString&)),this,SLOT(setMainStatusMsg(const QString&)));
	connect(graph,SIGNAL(activeCoordinateChanged(const QString&,const QString&,const QString&)),this,SLOT(updateStatusBar(const QString&,const QString&,const QString&)));
}

void BasicGraphMain::dimensionsChanged( const GraphEvent& e )
{
	((QLineEdit*)parent()->child("XMinField"))->setText(QString::number(e.xMin()));
	((QLineEdit*)parent()->child("YMinField"))->setText(QString::number(e.yMin()));
	((QLineEdit*)parent()->child("XMaxField"))->setText(QString::number(e.xMax()));
	((QLineEdit*)parent()->child("YMaxField"))->setText(QString::number(e.yMax()));
	//((QTextEdit*)parent()->child("XScaleField"))->setText(QString::number(e.xScale())); this never needed to be here .?.?.?
	//((QTextEdit*)parent()->child("YScaleField"))->setText(QString::number(e.yScale())); this never needed to be here .?.?.?

	//these are guaranteed to be valid entries so set the color to black in case any were red
	setTextColor("XMinField", Qt::black);
	setTextColor("XMaxField", Qt::black);
	setTextColor("YMinField", Qt::black);
	setTextColor("YMaxField", Qt::black);
}

void BasicGraphMain::setXMin(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = graph->setXMin(d);
	else
		ret = 2;
	
	if (ret != 0)
		setTextColor( "XMinField", Qt::red );
	else
		setTextColor( "XMinField", Qt::black );
}

void BasicGraphMain::setXMax(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = graph->setXMax(d);
	else
		ret = 2;
		
	if (ret != 0)
		setTextColor( "XMaxField", Qt::red );
	else
		setTextColor( "XMaxField", Qt::black );
}

void BasicGraphMain::setXScale(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = graph->setXScale(d);
	else
		ret = 2;
		
	if (ret != 0)
		setTextColor( "XScaleField", Qt::red );
	else
		setTextColor( "XScaleField", Qt::black );
}

void BasicGraphMain::setYMin(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = graph->setYMin(d);
	else
		ret = 2;
		
	if (ret != 0)
		setTextColor( "YMinField", Qt::red );
	else
		setTextColor( "YMinField", Qt::black );
}

void BasicGraphMain::setYMax(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = graph->setYMax(d);
	else
		ret = 2;
		
	if (ret != 0)
		setTextColor( "YMaxField", Qt::red );
	else
		setTextColor( "YMaxField", Qt::black );
}

void BasicGraphMain::setYScale(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = graph->setYScale(d);
	else
		ret = 2;
		
	if (ret != 0)
		setTextColor( "YScaleField", Qt::red );
	else
		setTextColor( "YScaleField", Qt::black );
}

void BasicGraphMain::save ( std::ofstream *fileOutput )
{
	*fileOutput<<"xMin= "<<graph->getXMin()<<std::endl;
	*fileOutput<<"xMax= "<<graph->getXMax()<<std::endl;
	*fileOutput<<"yMin= "<<graph->getYMin()<<std::endl;
	*fileOutput<<"yMax= "<<graph->getYMax()<<std::endl;
	*fileOutput<<"xScale= "<<graph->getXScale()<<std::endl;
	*fileOutput<<"yScale= "<<graph->getYScale()<<std::endl;
	*fileOutput<<"angle= "<<graph->getAngle()<<std::endl;
	*fileOutput<<"BackgroundColor= "<<graph->backgroundColor().name()<<std::endl;
	*fileOutput<<"AxisColor= "<<graph->axisColor().name()<<std::endl;
	*fileOutput<<"GridColor= "<<graph->gridColor().name()<<std::endl;
	*fileOutput<<"ScaleColor= "<<graph->scaleColor().name()<<std::endl;
	*fileOutput<<"showGrid= "<<graph->getGridState()<<std::endl;
	*fileOutput<<"showAxis= "<<graph->getAxisState()<<std::endl;
	*fileOutput<<"showScale= "<<graph->getScaleState()<<std::endl;
	*fileOutput<<"drawConnected= "<<graph->isDrawConnected()<<std::endl;
	*fileOutput<<"trace= "<<graph->isTracing()<<std::endl;
	*fileOutput<<"snapToGrid= "<<graph->isSnapToGrid()<<std::endl;
}

void BasicGraphMain::load( const char *key, const char *value )
{
	if (strcmp(key,"angle=") == 0)
	{
		graph->setAngle(QString(value).toInt());
	}
	else if (strcmp(key,"BackgroundColor=") == 0)
	{
		QColor c(value);
		graph->setBackgroundColor(c);
	}
	else if (strcmp(key,"AxisColor=") == 0)
	{
		QColor c(value);
		graph->setAxisColor(c);
	}
	else if (strcmp(key,"GridColor=") == 0)
	{
		QColor c(value);
		graph->setGridColor(c);
	}
	else if (strcmp(key,"ScaleColor=") == 0)
	{
		QColor c(value);
		graph->setScaleColor(c);
	}
	else if (strcmp(key,"showGrid=") == 0)
	{
		bool show;
		if ( strcmp( value,"0" ) == 0 )
			show = false;
		else
			show = true;
		graph->setGrid(show);
	}
	else if (strcmp(key,"showAxis=") == 0)
	{
		bool show;
		if ( strcmp( value,"0" ) == 0 )
			show = false;
		else
			show = true;
		graph->setAxis(show);
	}
	else if (strcmp(key,"showScale=") == 0)
	{
		bool show;
		if ( strcmp( value,"0" ) == 0 )
			show = false;
		else
			show = true;
		graph->setScale(show);
	}
	else if (strcmp(key,"snapToGrid=") == 0)
	{
		bool snap;
		if ( strcmp( value,"0" ) == 0 )
			snap = false;
		else
			snap = true;
		graph->setSnapToGrid(snap);
	}
}
