/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename slots use Qt Designer which will
** update this file, preserving your code. Create an init() slot in place of
** a constructor, and a destroy() slot in place of a destructor.
*****************************************************************************/
#include <fstream>
#include <ctime>

#include "expression.h"

void FunctionGraphMain::init()
{
	QLineEdit *edit = ExpressionComboBox->lineEdit();
	edit->disconnect( ExpressionComboBox ); //we'll handle QLineEdit events ourselves
	connect( edit, SIGNAL(returnPressed()), this, SLOT(addExpression()) );
	
	setGraph(FGraph);
}

void FunctionGraphMain::addExpression()
{
	ExpressionComboBox->changeItem(ExpressionComboBox->lineEdit()->text(),FGraph->getCurrentExpressionIndex());
	sendExpression();
}

void FunctionGraphMain::sendExpression()
{
	qDebug("sendExpression(): %s",ExpressionComboBox->currentText().latin1());
	FGraph->addExpression(Expression(ExpressionComboBox->currentText(),ActiveFunctionCheckBox->isChecked(),ColorChooser->color()));
	if (static_cast<int>(FGraph->functionCount()) == ExpressionComboBox->count() )
		ExpressionComboBox->insertItem("[New Expression]");
}

void FunctionGraphMain::updateExpression()
{
	int index = FGraph->getCurrentExpressionIndex();
	const Expression ex = FGraph->getExpression(index);
	
	YExpressionLabel->setText(QString("y<sub>%1</sub>(x)=").arg(index+1,0,10));
	ActiveFunctionCheckBox->setChecked(ex.show);
	ExpressionComboBox->setCurrentText(ex.getExpression("y").c_str());

	//set color to color current function... if no function entered set it to a random color
	QColor c = ex.color;
	if (c.isValid())
		ColorChooser->setColor(c);
	else
	{
		srand((unsigned)time(0));
		ColorChooser->setColor(QColor(rand() % 256, rand() % 256, rand() % 256));
	}
}

void FunctionGraphMain::removeExpression()
{
	int index = FGraph->getCurrentExpressionIndex();

	qDebug("removeExpression(): %d",index);

	FGraph->removeExpression(index);
	if (ExpressionComboBox->count() > 1 && ( ExpressionComboBox->count() != index+1) ) //don't remove the "[New Expression]" item
		ExpressionComboBox->removeItem(index);

	updateExpression();
}

void FunctionGraphMain::setCurrentExpression( int index )
{
	qDebug("setCurrentExpression(int)");
	FGraph->setCurrentExpressionIndex(index);
	updateExpression();
}

void FunctionGraphMain::save(std::ofstream * fileOutput)
{
	*fileOutput<<"[FunctionGraph]"<<std::endl;

	ExpressionGraphMain::save(fileOutput);     
    
	for (unsigned int i=0; i<FGraph->functionCount(); i++)
		*fileOutput<<"Function= "<<FGraph->getExpression(i).getExpression("y")<<":"<<FGraph->getExpression(i).color.name()<<" "<<FGraph->getExpression(i).show<<std::endl;
	*fileOutput<<"!"<<std::endl;
}

void FunctionGraphMain::load(std::ifstream & fileInput)
{
	char key[256];
	char value[256];

	while (strcmp(key,"!") != 0 && strcmp(value,"!") != 0)
	{
		fileInput>>key;
		if (strcmp(key,"Function=") == 0)
		{
			fileInput.get(value,256,':');
			QString s(value);
			fileInput.ignore(sizeof(char));
			fileInput>>value;
			QColor color(value);
			fileInput>>value;
			bool show;
			if ( strcmp( value,"0" ) == 0 )
				show = false;
			else
				show = true;
			ExpressionComboBox->lineEdit()->setText(s.stripWhiteSpace());
			ActiveFunctionCheckBox->setChecked(show);
			ColorChooser->setColor(color);
			addExpression();

			FGraph->nextExpression();
			continue;
		}
		
		if ( strcmp(key,"!") == 0 )
			break;

		fileInput>>value;
		ExpressionGraphMain::load(key,value);

		if (strcmp(key,"xMin=") == 0)
		{
			XMinField->setText(QString(value));
		}
		else if (strcmp(key,"xMax=") == 0)
		{
			XMaxField->setText(QString(value));
		}
		else if (strcmp(key,"yMin=") == 0)
		{
			YMinField->setText(QString(value));
		}
		else if (strcmp(key,"yMax=") == 0)
		{
			YMaxField->setText(QString(value));
		}
		else if (strcmp(key,"xScale=") == 0)
		{
			XScaleField->setText(QString(value));
		}
		else if (strcmp(key,"yScale=") == 0)
		{
			YScaleField->setText(QString(value));
		}
	}

	FGraph->nextExpression();
	updateExpression();
}





