/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename slots use Qt Designer which will
** update this file, preserving your code. Create an init() slot in place of
** a constructor, and a destroy() slot in place of a destructor.
*****************************************************************************/
#include <fstream>
#include <ctime>

#include "expression.h"

void ParametricGraphMain::init()
{
	QLineEdit *xedit = XExpressionComboBox->lineEdit();
	xedit->disconnect( XExpressionComboBox ); //we'll handle QLineEdit events ourselves
	connect( xedit, SIGNAL(returnPressed()), this, SLOT(addExpression()) );

	QLineEdit *yedit = YExpressionComboBox->lineEdit();
	yedit->disconnect( YExpressionComboBox ); //we'll handle QLineEdit events ourselves
	connect( yedit, SIGNAL(returnPressed()), this, SLOT(addExpression()) );
	
	setGraph(PGraph);
}

void ParametricGraphMain::addExpression()
{
	XExpressionComboBox->changeItem(XExpressionComboBox->lineEdit()->text(),PGraph->getCurrentExpressionIndex());
	YExpressionComboBox->changeItem(YExpressionComboBox->lineEdit()->text(),PGraph->getCurrentExpressionIndex());

	sendExpression();
}

void ParametricGraphMain::sendExpression()
{
	PGraph->addExpression(Expression(XExpressionComboBox->currentText(),YExpressionComboBox->currentText(),ActiveFunctionCheckBox->isChecked(),ColorChooser->color()));
	if (static_cast<int>(PGraph->functionCount()) == XExpressionComboBox->count() )
	{
		XExpressionComboBox->insertItem("[New Expression]");
		YExpressionComboBox->insertItem("[New Expression]");
	}
}

void ParametricGraphMain::updateExpression()
{
	int index = PGraph->getCurrentExpressionIndex();
	const Expression ex = PGraph->getExpression(index);
	
	XExpressionLabel->setText(QString("x<sub>%1</sub>(t)=").arg(index+1,0,10));
	YExpressionLabel->setText(QString("y<sub>%1</sub>(t)=").arg(index+1,0,10));

	ActiveFunctionCheckBox->setChecked(ex.show);

	XExpressionComboBox->setCurrentText(ex.getExpression("x").c_str());
	YExpressionComboBox->setCurrentText(ex.getExpression("y").c_str());

	//set color to color current function... if no function entered set it to a random color
	QColor c = ex.color;
	if (c.isValid())
		ColorChooser->setColor(c);
	else
	{
		srand((unsigned)time(0));
		ColorChooser->setColor(QColor(rand() % 256, rand() % 256, rand() % 256));
	}
}

void ParametricGraphMain::removeExpression()
{
	int index = PGraph->getCurrentExpressionIndex();

	qDebug("removeExpression(): %d",index);

	PGraph->removeExpression(index);
	if (XExpressionComboBox->count() > 1 && ( XExpressionComboBox->count() != index+1) ) //don't remove the "[New Expression]" item
	{
		XExpressionComboBox->removeItem(index);
		YExpressionComboBox->removeItem(index);
	}

	updateExpression();
}

void ParametricGraphMain::setCurrentExpression( int index )
{
	PGraph->setCurrentExpressionIndex(index);
	updateExpression();
}

void ParametricGraphMain::setTMin(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = PGraph->setTMin(d);
	else
		ret = 2;

	if (ret != 0)
		setTextColor( "TMinField", Qt::red );
	else
		setTextColor( "TMinField", Qt::black );
}

void ParametricGraphMain::setTMax(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = PGraph->setTMax(d);
	else
		ret = 2;

	if (ret != 0)
		setTextColor( "TMaxField", Qt::red );
	else
		setTextColor( "TMaxField", Qt::black );
}

void ParametricGraphMain::setTStep(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = PGraph->setTStep(d);
	else
		ret = 2;

	if (ret != 0)
		setTextColor( "TStepField", Qt::red );
	else
		setTextColor( "TStepField", Qt::black );
}

/** \todo Migrate save and load code out of ui library */
void ParametricGraphMain::save(std::ofstream * fileOutput)
{
	*fileOutput<<"[ParametricGraph]"<<std::endl;

	ExpressionGraphMain::save(fileOutput);        
        
        *fileOutput<<"tMin= "<<PGraph->getTMin()<<std::endl;
	*fileOutput<<"tMax= "<<PGraph->getTMax()<<std::endl;
	*fileOutput<<"tStep= "<<PGraph->getTStep()<<std::endl;

	for (unsigned int i=0; i<PGraph->functionCount(); i++)
		*fileOutput<<"Function= "<<PGraph->getExpression(i).getExpression("x")<<":"<<PGraph->getExpression(i).getExpression("y")<<":"<<PGraph->getExpression(i).color.name()<<" "<<PGraph->getExpression(i).show<<std::endl;
	*fileOutput<<"!"<<std::endl;
}

void ParametricGraphMain::load(std::ifstream & fileInput)
{
	char key[256];
	char value[256];

	while (strcmp(key,"!") != 0 && strcmp(value,"!") != 0)
	{
		fileInput>>key;
		if (strcmp(key,"Function=") == 0)
		{
			fileInput.get(value,sizeof(value),':');
			QString sx(value);
			fileInput.ignore(sizeof(char));

			fileInput.get(value,sizeof(value),':');
			QString sy(value);
			fileInput.ignore(sizeof(char));
			fileInput>>value;

			QColor color(value);
			fileInput>>value;
			bool show;
			if ( strcmp( value,"0" ) == 0 )
				show = false;
			else
				show = true;
				
			XExpressionComboBox->lineEdit()->setText(sx.stripWhiteSpace());
			YExpressionComboBox->lineEdit()->setText(sy.stripWhiteSpace());
			ActiveFunctionCheckBox->setChecked(show);
			ColorChooser->setColor(color);
			addExpression();

			PGraph->nextExpression();
			continue;
		}
		
		if ( strcmp(key,"!") == 0 )
			break;

		fileInput>>value;
		ExpressionGraphMain::load(key,value);

		if (strcmp(key,"xMin=") == 0)
		{
			XMinField->setText(QString(value));
		}
		else if (strcmp(key,"xMax=") == 0)
		{
			XMaxField->setText(QString(value));
		}
		else if (strcmp(key,"yMin=") == 0)
		{
			YMinField->setText(QString(value));
		}
		else if (strcmp(key,"yMax=") == 0)
		{
			YMaxField->setText(QString(value));
		}
		else if (strcmp(key,"xScale=") == 0)
		{
			XScaleField->setText(QString(value));
		}
		else if (strcmp(key,"yScale=") == 0)
		{
			YScaleField->setText(QString(value));
		}
		else if (strcmp(key,"tMin=") == 0)
		{
			TMinField->setText(QString(value));
		}
		else if (strcmp(key,"tMax=") == 0)
		{
			TMaxField->setText(QString(value));
		}
		else if (strcmp(key,"tStep=") == 0)
		{
			TStepField->setText(QString(value));
		}
	}

	PGraph->nextExpression();
	updateExpression();
}



