/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename slots use Qt Designer which will
** update this file, preserving your code. Create an init() slot in place of
** a constructor, and a destroy() slot in place of a destructor.
*****************************************************************************/
#include <fstream>
#include <ctime>

#include "expression.h"

void PolarGraphMain::init()
{
	QLineEdit *edit = ExpressionComboBox->lineEdit();
	edit->disconnect( ExpressionComboBox ); //we'll handle QLineEdit events ourselves
	connect( edit, SIGNAL(returnPressed()), this, SLOT(addExpression()) );
	
	setGraph(PoGraph);
}

void PolarGraphMain::addExpression()
{
	ExpressionComboBox->changeItem(ExpressionComboBox->lineEdit()->text(),PoGraph->getCurrentExpressionIndex());
	sendExpression();
}

void PolarGraphMain::sendExpression()
{
	qDebug("sendExpression(): %s",ExpressionComboBox->currentText().latin1());
	PoGraph->addExpression(Expression(ExpressionComboBox->currentText(),ActiveFunctionCheckBox->isChecked(),ColorChooser->color()));
	if (static_cast<int>(PoGraph->functionCount()) == ExpressionComboBox->count() )
		ExpressionComboBox->insertItem("[New Expression]");
}

void PolarGraphMain::updateExpression()
{
	int index = PoGraph->getCurrentExpressionIndex();
	const Expression ex = PoGraph->getExpression(index);
	
	RExpressionLabel->setText(QString("r<sub>%1</sub>(t)=").arg(index+1,0,10));
	ActiveFunctionCheckBox->setChecked(ex.show);
	ExpressionComboBox->setCurrentText(ex.getExpression("r").c_str());

	//set color to color current function... if no function entered set it to a random color
	QColor c = ex.color;
	if (c.isValid())
		ColorChooser->setColor(c);
	else
	{
		srand((unsigned)time(0));
		ColorChooser->setColor(QColor(rand() % 256, rand() % 256, rand() % 256));
	}
}

void PolarGraphMain::removeExpression()
{
	int index = PoGraph->getCurrentExpressionIndex();

	qDebug("removeExpression(): %d",index);

	PoGraph->removeExpression(index);
	if (ExpressionComboBox->count() > 1 && ( ExpressionComboBox->count() != index+1) ) //don't remove the "[New Expression]" item
		ExpressionComboBox->removeItem(index);

	updateExpression();
}

void PolarGraphMain::setCurrentExpression( int index )
{
	qDebug("setCurrentExpression(int)");
	PoGraph->setCurrentExpressionIndex(index);
	updateExpression();
}

void PolarGraphMain::setTMin(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = PoGraph->setTMin(d);
	else
		ret = 2;

	if (ret != 0)
		setTextColor( "TMinField", Qt::red );
	else
		setTextColor( "TMinField", Qt::black );
}

void PolarGraphMain::setTMax(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = PoGraph->setTMax(d);
	else
		ret = 2;

	if (ret != 0)
		setTextColor( "TMaxField", Qt::red );
	else
		setTextColor( "TMaxField", Qt::black );
}

void PolarGraphMain::setTStep(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = PoGraph->setTStep(d);
	else
		ret = 2;
		
	if (ret != 0)
		setTextColor( "TStepField", Qt::red );
	else
		setTextColor( "TStepField", Qt::black );
}

void PolarGraphMain::save(std::ofstream * fileOutput)
{
	*fileOutput<<"[PolarGraph]"<<std::endl;
	
	ExpressionGraphMain::save(fileOutput);
        
        *fileOutput<<"rScale= "<<PoGraph->getRScale()<<std::endl;
	*fileOutput<<"aScale= "<<PoGraph->getAScale()<<std::endl;
	*fileOutput<<"tMin= "<<PoGraph->getTMin()<<std::endl;
	*fileOutput<<"tMax= "<<PoGraph->getTMax()<<std::endl;
	*fileOutput<<"tStep= "<<PoGraph->getTStep()<<std::endl;
	for (unsigned int i=0; i<PoGraph->functionCount(); i++)
		*fileOutput<<"Function= "<<PoGraph->getExpression(i).getExpression("r")<<":"<<PoGraph->getExpression(i).color.name()<<" "<<PoGraph->getExpression(i).show<<std::endl;
	*fileOutput<<"!"<<std::endl;
}

void PolarGraphMain::load(std::ifstream & fileInput)
{
	char key[256];
	char value[256];

	while (strcmp(key,"!") != 0 && strcmp(value,"!") != 0)
	{
		fileInput>>key;
		if (strcmp(key,"Function=") == 0)
		{
			fileInput.get(key,256,':');
			QString s(key);
			fileInput.ignore(sizeof(char));
			fileInput>>key;
			QColor color(key);
			fileInput>>key;
			bool show;
			if ( strcmp( key,"0" ) == 0 )
				show = false;
			else
				show = true;
			ExpressionComboBox->lineEdit()->setText(s.stripWhiteSpace());
			ActiveFunctionCheckBox->setChecked(show);
			ColorChooser->setColor(color);
			addExpression();

			PoGraph->nextExpression();
			continue;
		}
		
		if ( strcmp(key,"!") == 0 )
			break;

		fileInput>>value;
		ExpressionGraphMain::load(key,value);

		if (strcmp(key,"xMin=") == 0)
		{
			XMinField->setText(QString(value));
		}
		else if (strcmp(key,"xMax=") == 0)
		{
			XMaxField->setText(QString(value));
		}
		else if (strcmp(key,"yMin=") == 0)
		{
			YMinField->setText(QString(value));
		}
		else if (strcmp(key,"yMax=") == 0)
		{
			YMaxField->setText(QString(value));
		}
		else if (strcmp(key,"rScale=") == 0)
		{
			XScaleField->setText(QString(value));
		}
		else if (strcmp(key,"aScale=") == 0)
		{
			YScaleField->setText(QString(value));
		}
		else if (strcmp(key,"tMin=") == 0)
		{
			TMinField->setText(QString(value));
		}
		else if (strcmp(key,"tMax=") == 0)
		{
			TMaxField->setText(QString(value));
		}
		else if (strcmp(key,"tStep=") == 0)
		{
			TStepField->setText(QString(value));
		}
	}
	PoGraph->nextExpression();
	updateExpression();
}






