/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename slots use Qt Designer which will
** update this file, preserving your code. Create an init() slot in place of
** a constructor, and a destroy() slot in place of a destructor.
*****************************************************************************/

#include <fstream>
#include <ctime>

#include <qbuttongroup.h>
#include <qmessagebox.h>

#include "frequencyplot.h"
#include "scatterplot.h"
#include "whiskerplot.h"

void StatPlotGraphMain::init()
{
	plotTypeRadioGroup = new QButtonGroup("Type");

	plotTypeRadioGroup->insert(ScatterPlotRadioButton,0);
	plotTypeRadioGroup->insert(FrequencyRadioButton,1);
	plotTypeRadioGroup->insert(BoxWhiskerRadioButton,2);

	XListComboBox->setEnabled(true);
	YListComboBox->setEnabled(false);
                        
	connect( plotTypeRadioGroup, SIGNAL( clicked(int) ), this, SLOT( doType(int) ) );
	
	setGraph(SGraph);
}

void StatPlotGraphMain::destroy()
{
}

void StatPlotGraphMain::addPlot()
{
	if ( PlotComboBox->count() > (int)info.size() ){return;}
	int index = PlotComboBox->count();
	PlotComboBox->insertItem(QString(tr("Plot %1")).arg(index+1));
	PlotComboBox->setCurrentItem(index);
	loadPlot(index);
}

void StatPlotGraphMain::removePlot()
{
	int current = PlotComboBox->currentItem();

	SGraph->removePlot(current);
	if (PlotComboBox->count() != 1)
		PlotComboBox->removeItem(current);
	if (info.size() != 0)
		info.erase(info.begin()+current);
	if (PlotComboBox->currentItem() == -1)
		PlotComboBox->setCurrentItem(0);

	int totalPlots = PlotComboBox->count();
	for (int i=0; i<totalPlots; i++)
		PlotComboBox->changeItem(QString(tr("Plot %1")).arg(totalPlots-i),totalPlots-i-1);
	loadPlot(PlotComboBox->currentItem());
}

void StatPlotGraphMain::addListMember()
{
	bool ok;
	double alpha = ListMemberField->text().toDouble(&ok);
	if (ok)
	{
		lists.resize(lists.size()+1);
		lists[ListComboBox->currentItem()].push_back(alpha);
		ListMemberListBox->insertItem(ListMemberField->text(),0);
	}
}

void StatPlotGraphMain::removeListMember()
{
	int remove = ListMemberListBox->currentItem();
	if (remove == -1){return;}
	ListMemberListBox->removeItem(remove);
	lists[ListComboBox->currentItem()].erase(lists[ListComboBox->currentItem()].end()-remove-1);
}

void StatPlotGraphMain::newList()
{
	QString text;
	for (int j=0; j<ListComboBox->count()+1; j++)
	{
		int n = 0;
		text = QString(tr("List %1")).arg(j+1);
		for (int i=0; i<ListComboBox->count(); i++)
		{
			if (QString::compare(ListComboBox->text(i),text) == 0)
				n++;
		}
		if (n==0){break;}
	}
	ListComboBox->insertItem(text);
	XListComboBox->insertItem(text);
	YListComboBox->insertItem(text);

	lists.resize(lists.size()+1);
}

void StatPlotGraphMain::deleteList()
{
	int selectedItem = ListComboBox->currentItem();
	if (selectedItem == 0 && ListComboBox->count() == 1){return;}
	ListMemberListBox->clear();
	lists[selectedItem].clear();
}

void StatPlotGraphMain::doType(int id)
{
	switch(id)
	{
		case 0: XListComboBox->setEnabled(true); YListComboBox->setEnabled(true); break; 
		case 1: case 2: XListComboBox->setEnabled(true); YListComboBox->setEnabled(false); break;
		default: qWarning("Error: bad plot mode"); break;
	}
}

void StatPlotGraphMain::loadList(int list)
{
	ListMemberListBox->clear();
	for (int i=lists[list].size()-1; i>=0; i--)
		ListMemberListBox->insertItem(QString::number(lists[list][i]));
}

void StatPlotGraphMain::nextPlot()
{
	if (PlotComboBox->currentItem() == PlotComboBox->count() -1)
		PlotComboBox->setCurrentItem(0);
	else
		PlotComboBox->setCurrentItem(PlotComboBox->currentItem()+1);
	loadPlot(PlotComboBox->currentItem());
}

void StatPlotGraphMain::prevPlot()
{
	if (PlotComboBox->currentItem() == 0)
		PlotComboBox->setCurrentItem(PlotComboBox->count() - 1);
	else
		PlotComboBox->setCurrentItem(PlotComboBox->currentItem()-1);
	loadPlot(PlotComboBox->currentItem());
}

void StatPlotGraphMain::sendPlot()
{
	unsigned int xSize = lists[XListComboBox->currentItem()].size();
	std::vector<double> xList(xSize);
	for (unsigned int i=0; i<lists[XListComboBox->currentItem()].size(); i++)
		xList[i]=lists[XListComboBox->currentItem()][i];

	unsigned int ySize = lists[YListComboBox->currentItem()].size();
	std::vector<double> yList(ySize);
	for (unsigned int i=0; i<lists[YListComboBox->currentItem()].size(); i++)
		yList[i]=lists[YListComboBox->currentItem()][i];

	if (xSize > 0)
	{
		PlotModule *p;
		switch ( plotTypeRadioGroup->id(plotTypeRadioGroup->selected()) )
		{
			case 1: p = new FrequencyPlot(xList,ClassesField->text().toInt(),ActivePlotCheckBox->isChecked(),ColorChooser->color());
				break;
			case 0: p = new ScatterPlot(xList,yList,ActivePlotCheckBox->isChecked(),ColorChooser->color());
				break;
			case 2: p = new WhiskerPlot(xList,ActivePlotCheckBox->isChecked(),ColorChooser->color());
				break;
			default: p = 0; break;
		}
	
		SGraph->addPlot(PlotComboBox->currentItem(),p);
	}
	else
		QMessageBox::warning(this,"Error","Can't graph: x-List empty!");

	PlotInfo pi;
	pi.type = plotTypeRadioGroup->id(plotTypeRadioGroup->selected());
	pi.show = ActivePlotCheckBox->isChecked();
	pi.xList = XListComboBox->currentItem();
	pi.yList = YListComboBox->currentItem();
	pi.classes = ClassesField->text().toInt();

	if ( (int)info.size() <= PlotComboBox->currentItem()+1 )
		info.resize(info.size()+1);

	info[PlotComboBox->currentItem()]=pi;
}

void StatPlotGraphMain::loadPlot(int plot)
{
	SGraph->setActivePlot(plot);

	PlotInfo p;
	if ( (int)info.size() <= plot)
	{
		p.type = 0;
		p.show = true;
		p.xList = 0;
		p.yList = 1;
		p.classes = 5;
	}
	else
		p = info[plot];

	plotTypeRadioGroup->setButton(p.type);
	XListComboBox->setCurrentItem(p.xList);
	YListComboBox->setCurrentItem(p.yList);
	ActivePlotCheckBox->setChecked(p.show);
	ClassesField->setText(QString("%1").arg(p.classes));
	doType(p.type);

	//set color to color current function... if no function entered set it to a random color
	QColor c = SGraph->getColor(SGraph->getCurrentPlotIndex());

	if (c.isValid())
		ColorChooser->setColor(c);
	else
	{
		srand((unsigned)time(0));
		ColorChooser->setColor(QColor(rand() % 256, rand() % 256, rand() % 256));
	}
}

void StatPlotGraphMain::setShowCurrentExpression(bool b)
{
	if (info.size() >= SGraph->getCurrentPlotIndex()+1)
		info[SGraph->getCurrentPlotIndex()].show = b;
	SGraph->setShowCurrentPlot(b);
}

void StatPlotGraphMain::save(std::ofstream * fileOutput)
{
	*fileOutput<<"[StatPlotGraph]"<<std::endl;

	BasicGraphMain::save(fileOutput);        
        
	for (unsigned int i=0; i<lists.size(); i++)
	{
		*fileOutput<<"List= ";
		for (unsigned int n=0; n<lists[i].size(); n++)
			*fileOutput<<lists[i][n]<<" ";
		*fileOutput<<";"<<std::endl;
	}
	for (unsigned int n=0; n<info.size(); n++)
	{
		*fileOutput<<"Plot= ";
		*fileOutput<<info[n].xList + 1<<" "
				<<info[n].yList + 1<<" "
				<<info[n].type<<" "
				<<info[n].classes<<" "
				<<SGraph->getColor(n).name()<<" "
				<<info[n].show<<" "
				<<std::endl;
	}
    *fileOutput<<"!"<<std::endl;
}

void StatPlotGraphMain::load( std::ifstream & fileInput)
{
	char key[256];
	char value[256];

	int tList = 0;
	int tPlot = 0;

	while (strcmp(key,"!") != 0 && strcmp(value,"!") != 0)
	{
		fileInput>>key;
		if (strcmp(key,"List=") == 0)
		{
			fileInput>>value;
			if (tList > 1){newList();}
			while(strcmp(value,";") != 0)
			{
				bool ok;
				double alpha = QString(value).toDouble(&ok);
				if (ok)
				{
					lists.resize(lists.size()+1);
					lists[tList].push_back(alpha);
				}
				fileInput>>value;
			}
			loadList(0);
			tList++;
			continue;
		}
		else if (strcmp(key,"Plot=") == 0)
		{
			if (tPlot > 0){addPlot();}

			fileInput>>value;
			int xlist = QString(value).toInt() - 1;
			XListComboBox->setCurrentItem(xlist);

			fileInput>>value;
			int ylist = QString(value).toInt() - 1;
			YListComboBox->setCurrentItem(ylist);

			fileInput>>value;
			int type = QString(value).toInt();
			plotTypeRadioGroup->setButton(type);
			doType(type);

			fileInput>>value;
			int classes = QString(value).toInt();
			ClassesField->setText(QString(value));

			fileInput>>value;
			QColor color(value);

			fileInput>>value;
			bool show = strcmp( value,"0" );

			if (tPlot == 0){ActivePlotCheckBox->setChecked(show);}

			std::vector<double> xList(lists[xlist].size());
			for (unsigned int i=0; i<lists[xlist].size(); i++)
				xList[i]=lists[xlist][i];

			std::vector<double> yList(lists[ylist].size());
			for (unsigned int i=0; i<lists[ylist].size(); i++)
				yList[i]=lists[ylist][i];

			PlotModule *p;
			switch ( plotTypeRadioGroup->id(plotTypeRadioGroup->selected()) )
			{
				case 0: p = new ScatterPlot(xList,yList,show,color);
					break;
				case 1: p = new FrequencyPlot(xList,classes,show,color);
					break;
				case 2: p = new WhiskerPlot(xList,show,color);
					break;
				default: qDebug("Error: StatPlotGraphMain::load(): unknown type: %d",plotTypeRadioGroup->id(plotTypeRadioGroup->selected())); p = 0; break;
			}

			if (p)
			{
				SGraph->addPlot(tPlot,p);

				PlotInfo pi;
				pi.type = type;
				pi.show = show;
				pi.xList = xlist;
				pi.yList = ylist;
				pi.classes = classes;
				info.push_back(pi);

				tPlot++;
			}
			continue;
		}
		
		if ( strcmp(key,"!") == 0 )
			break;

		fileInput>>value;
		BasicGraphMain::load(key,value);

		if (strcmp(key,"xMin=") == 0)
		{
			XMinField->setText(QString(value));
		}
		else if (strcmp(key,"xMax=") == 0)
		{
			XMaxField->setText(QString(value));
		}
		else if (strcmp(key,"yMin=") == 0)
		{
			YMinField->setText(QString(value));
		}
		else if (strcmp(key,"yMax=") == 0)
		{
			YMaxField->setText(QString(value));
		}
		else if (strcmp(key,"xScale=") == 0)
		{
			XScaleField->setText(QString(value));
		}
		else if (strcmp(key,"yScale=") == 0)
		{
			YScaleField->setText(QString(value));
		}
	}
        
        PlotComboBox->setCurrentItem(0);
	loadPlot(0);
}






