/***************************************************************************
                          glbasicgraphmain.cpp  -  description
                             -------------------
    begin                : Mon Feb 17 2003
    copyright            : (C) 2003 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "glbasicgraphmain.h"

#include <fstream>

#include <qlineedit.h>
 
#include "glexpressiongraph.h"
#include "glbasicgraph.h"
#include "glgraphevent.h"

GLBasicGraphMain::GLBasicGraphMain( QWidget* parent, const char* name ) : CommonGraphMain( parent, name )
{
	if ( !name )
	setName( "GLBasicGraphMain" );
}

GLBasicGraphMain::~GLBasicGraphMain()
{

}

void GLBasicGraphMain::dimensionsChanged( const GLGraphEvent& e )
{
	static_cast<QLineEdit*>(parent()->child("XMinField"))->setText(QString::number(e.xMin()));
	static_cast<QLineEdit*>(parent()->child("XMaxField"))->setText(QString::number(e.xMax()));
	static_cast<QLineEdit*>(parent()->child("YMinField"))->setText(QString::number(e.yMin()));
	static_cast<QLineEdit*>(parent()->child("YMaxField"))->setText(QString::number(e.yMax()));
	static_cast<QLineEdit*>(parent()->child("ZMinField"))->setText(QString::number(e.zMin()));
	static_cast<QLineEdit*>(parent()->child("ZMaxField"))->setText(QString::number(e.zMax()));

	//these are guaranteed to be valid entries so set the color to black in case any were red
	setTextColor("XMinField", Qt::black);
	setTextColor("XMaxField", Qt::black);
	setTextColor("YMinField", Qt::black);
	setTextColor("YMaxField", Qt::black);
	setTextColor("ZMinField", Qt::black);
	setTextColor("ZMaxField", Qt::black);
}

void GLBasicGraphMain::setXMin(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = _graph->setXMin(d);
	else
		ret = 2;
		
	if (ret != 0)
		setTextColor( "XMinField", Qt::red );
	else
		setTextColor( "XMinField", Qt::black );
}

void GLBasicGraphMain::setXMax(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = _graph->setXMax(d);
	else
		ret = 2;
		
	if (ret != 0)
		setTextColor( "XMaxField", Qt::red );
	else
		setTextColor( "XMaxField", Qt::black );
}

void GLBasicGraphMain::setYMin(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = _graph->setYMin(d);
	else
		ret = 2;
		
	if (ret != 0)
		setTextColor( "YMinField", Qt::red );
	else
		setTextColor( "YMinField", Qt::black );
}

void GLBasicGraphMain::setYMax(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = _graph->setYMax(d);
	else
		ret = 2;
		
	if (ret != 0)
		setTextColor( "YMaxField", Qt::red );
	else
		setTextColor( "YMaxField", Qt::black );
}

void GLBasicGraphMain::setZMin(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = _graph->setZMin(d);
	else
		ret = 2;
		
	if (ret != 0)
		setTextColor( "ZMinField", Qt::red );
	else
		setTextColor( "ZMinField", Qt::black );
}

void GLBasicGraphMain::setZMax(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = _graph->setZMax(d);
	else
		ret = 2;
		
	if (ret != 0)
		setTextColor( "ZMaxField", Qt::red );
	else
		setTextColor( "ZMaxField", Qt::black );
}

void GLBasicGraphMain::setPolarEye( const QString & s )
{
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
	{
		_graph->setPolarEye(d);
		setTextColor( "PolarEyeField", Qt::black );
	}
	else
		setTextColor( "PolarEyeField", Qt::red );
}

void GLBasicGraphMain::setAzimuthalEye( const QString & s )
{
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
	{
		_graph->setAzimuthalEye(d);
		setTextColor( "AzimuthalEyeField", Qt::black );
	}
	else
		setTextColor( "AzimuthalEyeField", Qt::red );
}

void GLBasicGraphMain::removeExpression()
{
	static_cast<GLExpressionGraph*>(_graph)->removeExpression(static_cast<GLExpressionGraph*>(_graph)->getCurrentExpressionIndex());
}

void GLBasicGraphMain::graphRotated( const GLGraphEvent &event )
{
    static_cast<QLineEdit*>(parent()->child("PolarEyeField"))->setText(QString::number(event.polarEye()));
    static_cast<QLineEdit*>(parent()->child("AzimuthalEyeField"))->setText(QString::number(event.azimuthalEye()));
}

void GLBasicGraphMain::save ( std::ofstream *fileOutput )
{
	GLExpressionGraph *graph = static_cast<GLExpressionGraph*>(_graph);

	*fileOutput<<"xMin= "<<graph->getXMin()<<std::endl;
	*fileOutput<<"xMax= "<<graph->getXMax()<<std::endl;
	*fileOutput<<"yMin= "<<graph->getYMin()<<std::endl;
	*fileOutput<<"yMax= "<<graph->getYMax()<<std::endl;
	*fileOutput<<"zMin= "<<graph->getZMin()<<std::endl;
	*fileOutput<<"zMax= "<<graph->getZMax()<<std::endl;
	*fileOutput<<"xMinView= "<<graph->getXMinView()<<std::endl;
	*fileOutput<<"xMaxView= "<<graph->getXMaxView()<<std::endl;
	*fileOutput<<"yMinView= "<<graph->getYMinView()<<std::endl;
	*fileOutput<<"yMaxView= "<<graph->getYMaxView()<<std::endl;
	*fileOutput<<"zMinView= "<<graph->getZMinView()<<std::endl;
	*fileOutput<<"zMaxView= "<<graph->getZMaxView()<<std::endl;
	*fileOutput<<"zEye= "<<graph->getZEye()<<std::endl;
	*fileOutput<<"xEye= "<<graph->getXEye()<<std::endl;
	*fileOutput<<"angle= "<<graph->getAngle()<<std::endl;
	*fileOutput<<"BackgroundColor= "<<graph->backgroundColor().name()<<std::endl;
	*fileOutput<<"AxisColor= "<<graph->axisColor().name()<<std::endl;
	*fileOutput<<"GridColor= "<<graph->gridColor().name()<<std::endl;
//    *fileOutput<<"ScaleColor= "<<graph->scaleColor().name()<<std::endl;
	*fileOutput<<"showGrid= "<<graph->getGridState()<<std::endl;
	*fileOutput<<"showAxis= "<<graph->getAxisState()<<std::endl;
//    *fileOutput<<"showScale= "<<graph->getScaleState()<<std::endl;
	*fileOutput<<"drawConnected= "<<graph->isDrawConnected()<<std::endl;
	*fileOutput<<"trace= "<<graph->isTracing()<<std::endl;
	*fileOutput<<"snapToGrid= "<<graph->isSnapToGrid()<<std::endl;
	*fileOutput<<"resolution= "<<graph->getResolution()<<std::endl;
	*fileOutput<<"animationMin= "<<graph->getAnimationMin()<<std::endl;
	*fileOutput<<"animationMax= "<<graph->getAnimationMax()<<std::endl;
	*fileOutput<<"animationStep= "<<graph->getAnimationStep()<<std::endl;
	*fileOutput<<"animationSpeed= "<<graph->getAnimationSpeed()<<std::endl;
	*fileOutput<<"animatorValue= "<<graph->animatorValue()<<std::endl;
}

void GLBasicGraphMain::load ( char *key, char *value, GLExpressionGraph *graph )
{
	if (strcmp(key,"angle=") == 0)
	{
		graph->setAngle(QString(value).toInt());
	}
	else if (strcmp(key,"BackgroundColor=") == 0)
	{
		QColor c(value);
		graph->setBackgroundColor(c);
	}
	else if (strcmp(key,"AxisColor=") == 0)
	{
		QColor c(value);
		graph->setAxisColor(c);
	}
	else if (strcmp(key,"GridColor=") == 0)
	{
		QColor c(value);
		graph->setGridColor(c);
	}
	else if (strcmp(key,"showGrid=") == 0)
	{
		bool show;
		if ( strcmp( value,"0" ) == 0 )
			show = false;
		else
			show = true;
		graph->setGrid(show);
	}
	else if (strcmp(key,"showAxis=") == 0)
	{
		bool show;
		if ( strcmp( value,"0" ) == 0 )
			show = false;
		else
			show = true;
		graph->setAxis(show);
	}
	else if (strcmp(key,"drawConnected=") == 0)
	{
		bool show;
		if ( strcmp( value,"0" ) == 0 )
			show = false;
		else
			show = true;
		graph->setDrawConnected(show);
	}
	else if (strcmp(key,"trace=") == 0)
	{
		bool show;
		if ( strcmp( value,"0" ) == 0 )
			show = false;
		else
			show = true;
		graph->setTrace(show);
	}
	else if (strcmp(key,"xMinView=") == 0)
	{
		graph->setXMinView(QString(value).toDouble());
	}
	else if (strcmp(key,"xMaxView=") == 0)
	{
		graph->setXMaxView(QString(value).toDouble());
	}
	else if (strcmp(key,"yMinView=") == 0)
	{
		graph->setYMinView(QString(value).toDouble());
	}
	else if (strcmp(key,"yMaxView=") == 0)
	{
		graph->setYMaxView(QString(value).toDouble());
	}
	else if (strcmp(key,"zMinView=") == 0)
	{
		graph->setZMinView(QString(value).toDouble());
	}
	else if (strcmp(key,"zMaxView=") == 0)
	{
		graph->setZMaxView(QString(value).toDouble());
	}
	else if (strcmp(key,"animationMin=") == 0)
	{
		graph->setAnimationMin(QString(value).toDouble());
	}
	else if (strcmp(key,"animationMax=") == 0)
	{
		graph->setAnimationMax(QString(value).toDouble());
	}
	else if (strcmp(key,"animationStep=") == 0)
	{
		graph->setAnimationStep(QString(value).toDouble());
	}
	else if (strcmp(key,"animationSpeed=") == 0)
	{
		graph->setAnimationSpeed(QString(value).toInt());
	}
	else if (strcmp(key,"animatorValue=") == 0)
	{
		graph->setAnimationFrame(QString(value).toDouble());
	}
}

