/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename slots use Qt Designer which will
** update this file, preserving your code. Create an init() slot in place of
** a constructor, and a destroy() slot in place of a destructor.
*****************************************************************************/

#include <fstream>
#include <iostream>
#include <ctime>

#include "expression.h"

void GLCylindricalGraphMain::init()
{
	//can I do this in GLCylindricalGraphMain.ui ?
	//PolarEyeLabel->setText(QString(tr("eye"))+QChar(0x03B8)); //theta
	//AzimuthalEyeLabel->setText(QString(tr("eye"))+QChar(0x03A6)); //ph    
    
	QLineEdit *edit = ExpressionComboBox->lineEdit();
	edit->disconnect( ExpressionComboBox ); //we'll handle QLineEdit events ourselves
	connect( edit, SIGNAL(returnPressed()), this, SLOT(addExpression()) );
	
	setGraph(GLCGraph);
}

void GLCylindricalGraphMain::addExpression()
{
	ExpressionComboBox->changeItem(ExpressionComboBox->lineEdit()->text(),GLCGraph->getCurrentExpressionIndex());
	sendExpression();
}

void GLCylindricalGraphMain::sendExpression()
{
	qDebug("sendExpression(): %s",ExpressionComboBox->currentText().latin1());
	GLCGraph->addExpression(Expression(ExpressionComboBox->currentText(),ActiveFunctionCheckBox->isChecked(),ColorChooser->color()));
	if (static_cast<int>(GLCGraph->functionCount()) == ExpressionComboBox->count() )
		ExpressionComboBox->insertItem("[New Expression]");
}

void GLCylindricalGraphMain::updateExpression()
{
	int index = GLCGraph->getCurrentExpressionIndex();
	const Expression ex = GLCGraph->getExpression(index);
	
	ExpressionLabel->setText(QString("z<sub>%1</sub>(r,t)=").arg(index+1,0,10));
	ActiveFunctionCheckBox->setChecked(ex.show);
	ExpressionComboBox->setCurrentText(ex.getExpression("z").c_str());

	//set color to color current function... if no function entered set it to a random color
	QColor c = ex.color;
	if (c.isValid())
		ColorChooser->setColor(c);
	else
	{
		srand((unsigned)time(0));
		ColorChooser->setColor(QColor(rand() % 256, rand() % 256, rand() % 256));
	}
}

void GLCylindricalGraphMain::removeExpression()
{
	int index = GLCGraph->getCurrentExpressionIndex();

	qDebug("removeExpression(): %d",index);

	GLCGraph->removeExpression(index);
	if (ExpressionComboBox->count() > 1 && ( ExpressionComboBox->count() != index+1) ) //don't remove the "[New Expression]" item
		ExpressionComboBox->removeItem(index);

	updateExpression();
}

void GLCylindricalGraphMain::setCurrentExpression( int index )
{
	qDebug("setCurrentExpression(int)");
	GLCGraph->setCurrentExpressionIndex(index);
	updateExpression();
}

void GLCylindricalGraphMain::save( std::ofstream * fileOutput )
{
	*fileOutput<<"[3DCylindricalGraph]"<<std::endl;

	GLBasicGraphMain::save( fileOutput );

	*fileOutput<<"tMin= "<<GLCGraph->getThetaMin()<<std::endl;
	*fileOutput<<"tMax= "<<GLCGraph->getThetaMax()<<std::endl;
	*fileOutput<<"rMin= "<<GLCGraph->getRadiusMin()<<std::endl;
	*fileOutput<<"rMax= "<<GLCGraph->getRadiusMax()<<std::endl;

	for (int i=0; i<GLCGraph->functionCount(); i++)
		*fileOutput<<"Function= "<<GLCGraph->getExpression(i).getExpression("z")<<":"<<GLCGraph->getExpression(i).color.name()<<" "<<GLCGraph->getExpression(i).show<<std::endl;
	*fileOutput<<"!"<<std::endl;
}

void GLCylindricalGraphMain::load( std::ifstream & fileInput )
{
	//FIXME: fix loading angle
	char key[256];
	char value[256];

	while (strcmp(key,"!") != 0 && strcmp(value,"!") != 0)
	{
		fileInput>>key;
		if (strcmp(key,"Function=") == 0)
		{
			fileInput.get(key,256,':');
			QString s(key);
			fileInput.ignore(sizeof(char));
			fileInput>>key;
			QColor color(key);
			fileInput>>key;
			bool show;
			if ( strcmp( key,"0" ) == 0 )
				show = false;
			else
				show = true;
			ExpressionComboBox->lineEdit()->setText(s.stripWhiteSpace());
			ActiveFunctionCheckBox->setChecked(show);
			ColorChooser->setColor(color);
			addExpression();

			GLCGraph->nextExpression();
			continue;
		}

		if ( strcmp(key,"!") == 0 )
			break;

		fileInput>>value;
		GLBasicGraphMain::load(key,value,GLCGraph);
		if (strcmp(key,"xMin=") == 0)
		{
			XMinField->setText(QString(value));
		}
		else if (strcmp(key,"xMax=") == 0)
		{
			XMaxField->setText(QString(value));
		}
		else if (strcmp(key,"yMin=") == 0)
		{
			YMinField->setText(QString(value));
		}
		else if (strcmp(key,"yMax=") == 0)
		{
			YMaxField->setText(QString(value));
		}
		else if (strcmp(key,"zMin=") == 0)
		{
			ZMinField->setText(QString(value));
		}
		else if (strcmp(key,"zMax=") == 0)
		{
			ZMaxField->setText(QString(value));
		}
		else if (strcmp(key,"tMin=") == 0)
		{
			TMinField->setText(QString(value));
		}
		else if (strcmp(key,"tMax=") == 0)
		{
			TMaxField->setText(QString(value));
		}
		else if (strcmp(key,"rMin=") == 0)
		{
			RMinField->setText(QString(value));
		}
		else if (strcmp(key,"rMax=") == 0)
		{
			RMaxField->setText(QString(value));
		}
		else if (strcmp(key,"zEye=") == 0)
		{
			AzimuthalEyeField->setText(QString(value));
		}
		else if (strcmp(key,"xEye=") == 0)
		{
			PolarEyeField->setText(QString(value));
		}
		else if (strcmp(key,"resolution=") == 0)
		{
			ResolutionSlider->setValue(QString(value).toInt());
		}
	}
	GLCGraph->nextExpression();
	updateExpression();
}

void GLCylindricalGraphMain::setRadiusMin(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = GLCGraph->setRadiusMin(d);
	else
		ret = 2;

	if (ret != 0)
		setTextColor( "RMinField", Qt::red );
	else
		setTextColor( "RMinField", Qt::black );
}

void GLCylindricalGraphMain::setRadiusMax(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = GLCGraph->setRadiusMax(d);
	else
		ret = 2;

	if (ret != 0)
		setTextColor( "RMaxField", Qt::red );
	else
		setTextColor( "RMaxField", Qt::black );
}

void GLCylindricalGraphMain::setThetaMin(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = GLCGraph->setThetaMin(d);
	else
		ret = 2;

	if (ret != 0)
		setTextColor( "TMinField", Qt::red );
	else
		setTextColor( "TMinField", Qt::black );
}

void GLCylindricalGraphMain::setThetaMax(const QString &s)
{
	int ret;
	bool ok;
	double d = s.toDouble(&ok);
	if (ok)
		ret = GLCGraph->setThetaMax(d);
	else
		ret = 2;

	if (ret != 0)
		setTextColor( "TMaxField", Qt::red );
	else
		setTextColor( "TMaxField", Qt::black );
}

