/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename slots use Qt Designer which will
** update this file, preserving your code. Create an init() slot in place of
** a constructor, and a destroy() slot in place of a destructor.
*****************************************************************************/

#include <fstream>
#include <iostream>
#include <ctime>

#include "expression.h"

void GLFunctionGraphMain::init()
{
	//can I do this in GLFunctionGraphMain.ui ?
	//PolarEyeLabel->setText(QString(tr("eye"))+QChar(0x03B8)); //theta
	//AzimuthalEyeLabel->setText(QString(tr("eye"))+QChar(0x03A6)); //phi    
    
	QLineEdit *edit = ExpressionComboBox->lineEdit();
	edit->disconnect( ExpressionComboBox ); //we'll handle QLineEdit events ourselves
	connect( edit, SIGNAL(returnPressed()), this, SLOT(addExpression()) );
	
	setGraph(GLFGraph);
}

void GLFunctionGraphMain::addExpression()
{
	ExpressionComboBox->changeItem(ExpressionComboBox->lineEdit()->text(),GLFGraph->getCurrentExpressionIndex());
	sendExpression();
}

void GLFunctionGraphMain::sendExpression()
{
	qDebug("sendExpression(): %s",ExpressionComboBox->currentText().latin1());
	GLFGraph->addExpression(Expression(ExpressionComboBox->currentText(),ActiveFunctionCheckBox->isChecked(),ColorChooser->color()));
	if (static_cast<int>(GLFGraph->functionCount()) == ExpressionComboBox->count() )
		ExpressionComboBox->insertItem("[New Expression]");
}

void GLFunctionGraphMain::updateExpression()
{
	int index = GLFGraph->getCurrentExpressionIndex();
	const Expression ex = GLFGraph->getExpression(index);
	
	ExpressionLabel->setText(QString("z<sub>%1</sub>(x,y)=").arg(index+1,0,10));
	ActiveFunctionCheckBox->setChecked(ex.show);
	ExpressionComboBox->setCurrentText(ex.getExpression("z").c_str());

	//set color to color current function... if no function entered set it to a random color
	QColor c = ex.color;
	if (c.isValid())
		ColorChooser->setColor(c);
	else
	{
		srand((unsigned)time(0));
		ColorChooser->setColor(QColor(rand() % 256, rand() % 256, rand() % 256));
	}
}

void GLFunctionGraphMain::removeExpression()
{
	int index = GLFGraph->getCurrentExpressionIndex();

	qDebug("removeExpression(): %d",index);

	GLFGraph->removeExpression(index);
	if (ExpressionComboBox->count() > 1 && ( ExpressionComboBox->count() != index+1) ) //don't remove the "[New Expression]" item
		ExpressionComboBox->removeItem(index);

	updateExpression();
}

void GLFunctionGraphMain::setCurrentExpression( int index )
{
	qDebug("setCurrentExpression(int)");
	GLFGraph->setCurrentExpressionIndex(index);
	updateExpression();
}

void GLFunctionGraphMain::save( std::ofstream * fileOutput )
{
	*fileOutput<<"[3DFunctionGraph]"<<std::endl;

	GLBasicGraphMain::save( fileOutput );    
    
	for (int i=0; i<GLFGraph->functionCount(); i++)
		*fileOutput<<"Function= "<<GLFGraph->getExpression(i).getExpression("z")<<":"<<GLFGraph->getExpression(i).color.name()<<" "<<GLFGraph->getExpression(i).show<<std::endl;
	*fileOutput<<"!"<<std::endl;
}

void GLFunctionGraphMain::load( std::ifstream & fileInput )
{
	//FIXME: fix loading angle?
	char key[256];
	char value[256];

	while (strcmp(key,"!") != 0 && strcmp(value,"!") != 0)
	{
		fileInput>>key;
		if (strcmp(key,"Function=") == 0)
		{
			fileInput.get(key,256,':');
			QString s(key);
			fileInput.ignore(sizeof(char));
			fileInput>>key;
			QColor color(key);
			fileInput>>key;
			bool show;
			if ( strcmp( key,"0" ) == 0 )
				show = false;
			else
				show = true;
			ExpressionComboBox->lineEdit()->setText(s.stripWhiteSpace());
			ActiveFunctionCheckBox->setChecked(show);
			ColorChooser->setColor(color);
			addExpression();

			GLFGraph->nextExpression();
			continue;
		}
		
		if ( strcmp(key,"!") == 0 )
			break;

		fileInput>>value;
		GLBasicGraphMain::load(key,value,GLFGraph);

		if (strcmp(key,"xMin=") == 0)
		{
			XMinField->setText(QString(value));
		}
		else if (strcmp(key,"xMax=") == 0)
		{
			XMaxField->setText(QString(value));
		}
		else if (strcmp(key,"yMin=") == 0)
		{
			YMinField->setText(QString(value));
		}
		else if (strcmp(key,"yMax=") == 0)
		{
			YMaxField->setText(QString(value));
		}
		else if (strcmp(key,"zMin=") == 0)
		{
			ZMinField->setText(QString(value));
		}
		else if (strcmp(key,"zMax=") == 0)
		{
			ZMaxField->setText(QString(value));
		}
		else if (strcmp(key,"zEye=") == 0)
		{
			AzimuthalEyeField->setText(QString(value));
		}
		else if (strcmp(key,"xEye=") == 0)
		{
			PolarEyeField->setText(QString(value));
		}
		else if (strcmp(key,"resolution=") == 0)
		{
			ResolutionSlider->setValue(QString(value).toInt());
		}
	}
	GLFGraph->nextExpression();
	updateExpression();
}
