/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename slots use Qt Designer which will
** update this file, preserving your code. Create an init() slot in place of
** a constructor, and a destroy() slot in place of a destructor.
*****************************************************************************/

#include <fstream>
#include <iostream>
#include <ctime>

#include "expression.h"

void GLPolarGraphMain::init()
{
	//can I do this in GLPolarGraphMain.ui ?
	//PolarEyeLabel->setText(QString(tr("eye"))+QChar(0x03B8)); //theta
	//AzimuthalEyeLabel->setText(QString(tr("eye"))+QChar(0x03A6)); //phi
   
	QLineEdit *edit = ExpressionComboBox->lineEdit();
	edit->disconnect( ExpressionComboBox ); //we'll handle QLineEdit events ourselves
	connect( edit, SIGNAL(returnPressed()), this, SLOT(addExpression()) );
	
	setGraph(GLPoGraph);
}

void GLPolarGraphMain::addExpression()
{
	ExpressionComboBox->changeItem(ExpressionComboBox->lineEdit()->text(),GLPoGraph->getCurrentExpressionIndex());
	sendExpression();
}

void GLPolarGraphMain::sendExpression()
{
	qDebug("sendExpression(): %s",ExpressionComboBox->currentText().latin1());
	GLPoGraph->addExpression(Expression(ExpressionComboBox->currentText(),ActiveFunctionCheckBox->isChecked(),ColorChooser->color()));
	if (static_cast<int>(GLPoGraph->functionCount()) == ExpressionComboBox->count() )
		ExpressionComboBox->insertItem("[New Expression]");
}

void GLPolarGraphMain::updateExpression()
{
	int index = GLPoGraph->getCurrentExpressionIndex();
	const Expression ex = GLPoGraph->getExpression(index);
	
	ExpressionLabel->setText(QString("r<sub>%1</sub>(a,p)=").arg(index+1,0,10));
	ActiveFunctionCheckBox->setChecked(ex.show);
	ExpressionComboBox->setCurrentText(ex.getExpression("r").c_str());

	//set color to color current function... if no function entered set it to a random color
	QColor c = ex.color;
	if (c.isValid())
		ColorChooser->setColor(c);
	else
	{
		srand((unsigned)time(0));
		ColorChooser->setColor(QColor(rand() % 256, rand() % 256, rand() % 256));
	}
}

void GLPolarGraphMain::removeExpression()
{
	int index = GLPoGraph->getCurrentExpressionIndex();

	qDebug("removeExpression(): %d",index);

	GLPoGraph->removeExpression(index);
	if (ExpressionComboBox->count() > 1 && ( ExpressionComboBox->count() != index+1) ) //don't remove the "[New Expression]" item
		ExpressionComboBox->removeItem(index);

	updateExpression();
}

void GLPolarGraphMain::setCurrentExpression( int index )
{
	qDebug("setCurrentExpression(int)");
	GLPoGraph->setCurrentExpressionIndex(index);
	updateExpression();
}

void GLPolarGraphMain::save( std::ofstream * fileOutput )
{
	*fileOutput<<"[3DPolarGraph]"<<std::endl;

	GLBasicGraphMain::save( fileOutput );

	*fileOutput<<"aMin= "<<GLPoGraph->getAzimuthalMin()<<std::endl;
	*fileOutput<<"aMax= "<<GLPoGraph->getAzimuthalMax()<<std::endl;
	*fileOutput<<"tMin= "<<GLPoGraph->getPolarMin()<<std::endl;
	*fileOutput<<"tMax= "<<GLPoGraph->getPolarMax()<<std::endl;

	for (int i=0; i<GLPoGraph->functionCount(); i++)
		*fileOutput<<"Function= "<<GLPoGraph->getExpression(i).getExpression("z")<<":"<<GLPoGraph->getExpression(i).color.name()<<" "<<GLPoGraph->getExpression(i).show<<std::endl;
	*fileOutput<<"!"<<std::endl;
}

void GLPolarGraphMain::load( std::ifstream & fileInput )
{
	//FIXME: fix loading angle
	char key[256];
	char value[256];

	while (strcmp(key,"!") != 0 && strcmp(value,"!") != 0)
	{
		fileInput>>key;
		if (strcmp(key,"Function=") == 0)
		{
			fileInput.get(key,256,':');
			QString s(key);
			fileInput.ignore(sizeof(char));
			fileInput>>key;
			QColor color(key);
			fileInput>>key;
			bool show;
			if ( strcmp( key,"0" ) == 0 )
				show = false;
			else
				show = true;
			ExpressionComboBox->lineEdit()->setText(s.stripWhiteSpace());
			ActiveFunctionCheckBox->setChecked(show);
			ColorChooser->setColor(color);
			addExpression();

			GLPoGraph->nextExpression();
			continue;
		}

		if ( strcmp(key,"!") == 0 )
			break;

		fileInput>>value;
		GLBasicGraphMain::load(key,value,GLPoGraph);

		if (strcmp(key,"xMin=") == 0)
		{
			XMinField->setText(QString(value));
		}
		else if (strcmp(key,"xMax=") == 0)
		{
			XMaxField->setText(QString(value));
		}
		else if (strcmp(key,"yMin=") == 0)
		{
			YMinField->setText(QString(value));
		}
		else if (strcmp(key,"yMax=") == 0)
		{
			YMaxField->setText(QString(value));
		}
		else if (strcmp(key,"zMin=") == 0)
		{
			ZMinField->setText(QString(value));
		}
		else if (strcmp(key,"zMax=") == 0)
		{
			ZMaxField->setText(QString(value));
		}
		else if (strcmp(key,"aMin=") == 0)
		{
			AMinField->setText(QString(value));
		}
		else if (strcmp(key,"aMax=") == 0)
		{
			AMaxField->setText(QString(value));
		}
		else if (strcmp(key,"tMin=") == 0)
		{
			TMinField->setText(QString(value));
		}
		else if (strcmp(key,"tMax=") == 0)
		{
			TMaxField->setText(QString(value));
		}
		else if (strcmp(key,"zEye=") == 0)
		{
			AzimuthalEyeField->setText(QString(value));
		}
		else if (strcmp(key,"xEye=") == 0)
		{
			PolarEyeField->setText(QString(value));
		}
		else if (strcmp(key,"resolution=") == 0)
		{
			ResolutionSlider->setValue(QString(value).toInt());
		}
	}
	GLPoGraph->nextExpression();
	updateExpression();
}

void GLPolarGraphMain::setPolarMin(const QString& s)
{
	int ret;
	bool ok;
	double r = s.toDouble(&ok);
	if (ok)
		ret = GLPoGraph->setPolarMin(r);
	else
		ret = 2;
		
	if (ret != 0)
		setTextColor( "TMinField", Qt::red );
	else
		setTextColor( "TMinField", Qt::black );
}

void GLPolarGraphMain::setPolarMax(const QString& s)
{
	int ret;
	bool ok;
	double r = s.toDouble(&ok);
	if (ok)
		ret = GLPoGraph->setPolarMax(r);
	else
		ret = 2;
		
	if (ret != 0)
		setTextColor( "TMaxField", Qt::red );
	else
		setTextColor( "TMaxField", Qt::black );
}

void GLPolarGraphMain::setAzimuthalMin(const QString& s)
{
	int ret;
	bool ok;
	double r = s.toDouble(&ok);
	if (ok)
		ret = GLPoGraph->setAzimuthalMin(r);
	else
		ret = 2;
		
	if (ret != 0)
		setTextColor( "AMinField", Qt::red );
	else
		setTextColor( "AMinField", Qt::black );
}

void GLPolarGraphMain::setAzimuthalMax(const QString& s)
{
	int ret;
	bool ok;
	double r = s.toDouble(&ok);
	if (ok)
		ret = GLPoGraph->setAzimuthalMax(r);
	else
		ret = 2;
		
	if (ret != 0)
		setTextColor( "AMaxField", Qt::red );
	else
		setTextColor( "AMaxField", Qt::black );
}

