/***************************************************************************
                          interface.ui.h  -  The brains behind communication
                                             between the graphs and UI                 
                             -------------------
    begin                : Sat Mar 29 2002
    copyright            : (C) 2002-2003 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <fstream>
#include <cmath>

#include <qmessagebox.h>
#include <qcolordialog.h>
#include <qapplication.h>

#ifdef KDE_APP
#include <kfiledialog.h>
#include <kfile.h>
#include <kiconloader.h>
#include <kaboutdata.h>
#include <kaboutapplication.h>
#include <kstatusbar.h>
#include <qmenubar.h>
#include <kprinter.h>
#else
#include <qfiledialog.h>
#include <qstatusbar.h>
#include <kprinter.h>
#endif //KDE_APP

#include <qvbox.h>
#include <qlabel.h>
#include <qobjectlist.h>

#include "newgraphdialog.h"
#include "graph.h"
#include "zoomfactordialog.h"
#include "usage.h"
#include "animationparamsdialog.h"

#ifdef GLGRAPH
#include "glexpressiongraph.h"
#endif //GLGRAPH

#include "commongraph.h"
#include "basicgraph.h"
#include "expressiongraph.h"
#include "statplotgraph.h"
#include "fungmath.h"

#include "config.h"

void Fung_main::init()
{
	//set up workspace
	QVBox* vb = new QVBox( this , "QVBox");
	vb->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
	ws = new QWorkspace( vb , "Workspace");
	ws->setScrollBarsEnabled( true );
	setCentralWidget( vb );

	#ifdef KDE_APP
	printer = new KPrinter;
	#else
	//set up Qt printer object    
	printer = new QPrinter;
	printer->setColorMode(QPrinter::Color);
	printer->setPrintProgram("lpr");
	#endif //KDE_APP

	printer->setCreator(QString("Fung-Calc v%1").arg(VERSION));

	#if ( (QT_VERSION & 0xFF0000) >= 3 && (QT_VERSION & 0x00FF00) >= 1 ) //this doesn't seem to work right in Qt < 3.1
	//set up status bar
	statusBar()->addWidget(new QLabel(this,"standard"),6,false);
        statusBar()->addWidget(new QLabel(this,"label1"),3,false);
        statusBar()->addWidget(new QLabel(this,"label2"),3,false);
	statusBar()->addWidget(new QLabel(this,"label3"),3,false);
	#endif

	menubarItemsCount = menubar->count();

	//set id to index for easy reference
	for (unsigned int i=0; i<menubarItemsCount; i++)
		menubar->setId(i,i);

	//create usage window
	usageWindow = new UsageDialog(this, "UsageDialog", false, Qt::WStyle_Customize | WStyle_Dialog );
	//connect( usageWindow, SIGNAL(close()), UsageAction, SLOT( setOn(false) ) );

	//connections
	connect( ws, SIGNAL( windowActivated(QWidget*) ), this, SLOT( updateToolBar(QWidget*) ) );
	connect( ws, SIGNAL( windowActivated(QWidget*) ), this, SLOT( windowOpened() ) );
	showRelaventActions(false);

	#ifdef KDE_APP
	fileOpenAction->setIconSet( BarIconSet("fileopen") );
	fileSaveAction->setIconSet( BarIconSet("filesave") );
	filePrintAction->setIconSet( BarIconSet("fileprint") );
	fileExitAction->setIconSet( BarIconSet("exit") );
	fileNewAction->setIconSet( BarIconSet("filenew") );

	PlayAction->setIconSet( BarIconSet("player_play") );
	PauseAction->setIconSet( BarIconSet("player_pause") );
	StopAction->setIconSet( BarIconSet("player_stop") );
	
	windowCloseAction->setIconSet( BarIconSet("fileclose") );
	#endif //KDE_APP
}

void Fung_main::destroy()
{
	delete printer;    
}

void Fung_main::windowOpened()
{
	bool graph_opened = ( ws->activeWindow() ) ? true : false;
	showRelaventActions( graph_opened );

	CommonGraph * graph = getGraph(false);
	if (graph)
	{
		#ifdef GLGRAPH
		GLBasicGraph * gl_bs = dynamic_cast<GLBasicGraph*>(graph);
		if (gl_bs)
			connect( gl_bs, SIGNAL(rotateCanceled()), this, SLOT( cancelRotate() ) );
		#endif //GLGRAPH

		Animator *animator = dynamic_cast<Animator*>(graph);
		if (animator)
		{
			#ifdef GLGRAPH
			GLExpressionGraph *gl_exp_gr = dynamic_cast<GLExpressionGraph*>(animator);
			if (gl_exp_gr)
				connect( gl_exp_gr, SIGNAL(animationValueChanged(double)), this, SLOT(animationValueChanged(double)) );
			else
			#endif //GLGRAPH
			{
				ExpressionGraph *exp_gr = dynamic_cast<ExpressionGraph*>(animator);
				if (exp_gr)
					connect( exp_gr, SIGNAL(animationValueChanged(double)), this, SLOT(animationValueChanged(double)) );
			}
		}

		CommonGraphMain *window = dynamic_cast<CommonGraphMain*>(ws->activeWindow()); //we assume that only CommonGraphMain widgets are added to the workspace
		//assert(window);
		window->updateStatusBar( "", "", "" ); //blank the status bar
	}
}

void Fung_main::windowClosed()
{
	//since this is called right before the graph window is closed, a graph window
	//is opened if there are at least two windows opened
	bool graph_opened = ( ws->windowList().count() >= 2 ) ? true : false;

	showRelaventActions( graph_opened );
}

//show or hide actions that depend on a certain type of graph
void Fung_main::showRelaventActions( bool graph_opened )
{
	for (unsigned int i=1; i<menubarItemsCount-1; i++) //hides/shows all menu items between the first, "File", and last, "Help"
	{
		#if ( (QT_VERSION & 0xFF0000) >= 3 && (QT_VERSION & 0x00FF00) >= 1 ) //not available unless have Qt >= 3.1
			menubar->setItemVisible(i,graph_opened);
		#else
			menubar->setItemEnabled(i,graph_opened);
		#endif
	}

	bool is_gl = false, is_animator = false, is_stat;

	CommonGraph *graph = getGraph(false);

	#ifdef GLGRAPH
	is_gl = ( dynamic_cast<GLBasicGraph*>( graph ) ) ? true : false;
	#endif //GLGRAPH

	#if ( (QT_VERSION & 0xFF0000) >= 3 && (QT_VERSION & 0x00FF00) >= 1 ) //not available unless have QT >= 3.1
	menubar->setItemVisible(6,is_gl);
	#else
	menubar->setItemEnabled(6,is_gl);
	#endif

	is_animator = ( dynamic_cast<Animator*>( graph ) ) ? true : false;
	#if ( (QT_VERSION & 0xFF0000) >= 3 && (QT_VERSION & 0x00FF00) >= 1 ) //not available unless have QT >= 3.1
	menubar->setItemVisible(3,is_animator);
	#else
	menubar->setItemEnabled(3,is_animator);
	#endif

	is_stat = ( dynamic_cast<StatPlotGraph*>( graph ) ) ? true : false;

	//available for only 3D graphs
	ExportToDXFAction->setEnabled(is_gl);

	//available for all but 3D graphs
	axisColorAction->setEnabled(!is_gl);
	scaleToggleAction->setEnabled(!is_gl);
	DisplayNValueAction->setEnabled(!is_gl);
	scaleColorAction->setEnabled(!is_gl);

	//available for all but stat plots
	drawStyleAction->setEnabled(!is_stat);

	//only available if a graph window is opened
	fileSaveAction->setEnabled( graph_opened );
	fileSaveAllAction->setEnabled( graph_opened );
	ExportToImageAction->setEnabled( graph_opened );
	filePrintAction->setEnabled( graph_opened );

	//show relevant toolbars
	(graph_opened) ? GraphOptionsToolbar->show() : GraphOptionsToolbar->hide();
	(is_animator) ? AnimationToolbar->show() : AnimationToolbar->hide();
}

//load graph state into menus and such
void Fung_main::updateToolBar(QWidget*)
{
	CommonGraph* graph;
	if ( (graph = getGraph(false) ) != 0)
	{
		Animator *animator;
		if ( (animator = dynamic_cast<Animator*>(graph)) != 0 )
		{
			AnimationMenu->setEnabled(true);
			AnimationSlider->setValue(static_cast<int>(ROUND(animator->animatorValue())));
			AnimationSlider->setMinValue( static_cast<int>( ROUND(animator->getAnimationMin()) ) );
			AnimationSlider->setMaxValue( static_cast<int>( ROUND(animator->getAnimationMax()) ) );
			DisplayNValueAction->setOn(animator->displayAnimatorValue());
		}
		else
			AnimationMenu->setEnabled(false);
		
		#ifdef GLGRAPH
		GLBasicGraph * gl_bs = dynamic_cast<GLBasicGraph*>(graph);
		if (gl_bs)
		{
			RotateXAction->setOn(gl_bs->isAutoRotatingX());
			RotateYAction->setOn(gl_bs->isAutoRotatingY());
			RotateZAction->setOn(gl_bs->isAutoRotatingZ());
		}
		#endif //GLGRAPH
		
		gridToggleAction->setOn(graph->getGridState());
		axisToggleAction->setOn(graph->getAxisState());
		drawStyleAction->setOn(graph->isDrawConnected());
		traceAction->setOn(graph->isTracing());
		scaleToggleAction->setOn(graph->getScaleState());
		SnapAction->setOn(graph->isSnapToGrid());

		degreesAction->setOn(!graph->getAngle());
		radiansAction->setOn(graph->getAngle());

		//whether or not the particular feature is available for the type of graph... scroll down
		//a bit to see which slot represents which feature
		bool enabled[][11] = {
			{true,  true,  false, true, true,  true,  true,  false, false, true , true},//Function graph
			{false, false, false, true, false, false, false, false, false, false, true }, //Polar graph
			{false, true, false, true, false, false, false, false, false, false, true }, //Parametric graph
			{false, false, true,  true, false, false, false, false, true , false, true }, //Stat plot graph
			{false, false, false,  true, false, false, false, false, false , false, true }, //Density plot
			#ifdef GLGRAPH
			{false, false, false, false, false, false, false, false, false, false, false }, //3D Function graph
			{false, false, false, false, false, false, false, false, false, false, false }, //3D Polar graph
			{false, false, false, false, false, false, false, false, false, false, false }, //3D Cylindrical graph
			#endif
			};

		int type = graph->getGraphType();

		DefiniteIntegralAction->setEnabled(enabled[type][0]);
		ArcLengthAction->setEnabled(enabled[type][1]);
		ZoomStatAction->setEnabled(enabled[type][2]);
		distanceAction->setEnabled(enabled[type][3]);
		MaximumAction->setEnabled(enabled[type][4]);
		MinimumAction->setEnabled(enabled[type][5]);
		ZeroAction->setEnabled(enabled[type][6]);
		IntersectionAction->setEnabled(enabled[type][7]);
		ZoomStatAction->setEnabled(enabled[type][8]);
		MeanValueAction->setEnabled(enabled[type][9]);
		ZoomBoxAction->setEnabled(enabled[type][10]);
	}
}

//#include <qxml.h>
//#include <qdom.h>
//#include <qfile.h>
//#include "fungxmlhandler.h"

int Fung_main::loadXML(QString &)
{
/*	FungXMLHandler handler( this, ws );

	QFile xmlFile( file );
	QXmlInputSource source( xmlFile );
	QXmlSimpleReader reader;
	reader.setContentHandler( &handler );
	return !reader.parse( source );   */
	return 0;    
}

int Fung_main::load(QString &file)
{
	std::ifstream fileInput(file);
	if (fileInput.bad())
		return 1;

	
	char str[256];
	while ((fileInput>>str) != 0)
	{
		QWidget *new_graph = 0;
		if (strcmp(str,"[FunctionGraph]") == 0)
		{
			qDebug("Loading Function Graph");
			new_graph = Graph::createGraph(ws,Graph::FUNCTIONGRAPH);
		}
		if (strcmp(str,"[PolarGraph]") == 0)
		{
			qDebug("Loading Polar Graph");
			new_graph = Graph::createGraph(ws,Graph::POLARGRAPH);
		}
		if (strcmp(str,"[ParametricGraph]") == 0)
		{
			qDebug("Loading Parametric Graph");
			new_graph = Graph::createGraph(ws,Graph::PARAMETRICGRAPH);
		}
		if (strcmp(str,"[StatPlotGraph]") == 0)
		{
			qDebug("Loading Stat Plot Graph");
			new_graph = Graph::createGraph(ws,Graph::STATGRAPH);
		}
		if (strcmp(str,"[DensityPlot]") == 0)
		{
			qDebug("Loading Density Plot");
			new_graph = Graph::createGraph(ws,Graph::DENSITYPLOT);
		}               
		#ifdef GLGRAPH
		if (strcmp(str,"[3DFunctionGraph]") == 0)
		{
			qDebug("Loading 3D Function Graph");
			new_graph = Graph::createGraph(ws,Graph::GLFUNCTIONGRAPH);
		}
		if (strcmp(str,"[3DPolarGraph]") == 0)
		{
			qDebug("Loading 3D Polar Graph");
			new_graph = Graph::createGraph(ws,Graph::GLPOLARGRAPH);
		}
		if (strcmp(str,"[3DCylindricalGraph]") == 0)
		{
			qDebug("Loading 3D Cylindrical Graph");
			new_graph = Graph::createGraph(ws,Graph::GLCYLINDRICALGRAPH);
		}
		#endif //GLGRAPH
                
        	if (new_graph)
		{
			static_cast<CommonGraphMain*>(new_graph)->load(fileInput);                
			connect( new_graph, SIGNAL(destroyed()), this, SLOT(windowClosed()) ); //when graphs are destroyed we need to update the toolbar appropriately
		}
        }
	fileInput.close();

	windowCascade();
	return 0;
}

void Fung_main::fileExit()
{
	close();
}

void Fung_main::fileNew()
{
	NewGraphDialog dlg(this,"newgraphdialog",true,0);
	if (QDialog::Accepted == dlg.exec())
		connect( Graph::createGraph(ws,dlg.getSelected()), SIGNAL(destroyed()), this, SLOT(windowClosed()) );
}

void Fung_main::fileOpen()
{
	#ifdef KDE_APP
	KFileDialog fd( QString::null, "*.fgc|Fung-Calc Files (*.fgc)", this, "file dialog", TRUE );
	#else
	QFileDialog fd( this, "file dialog", TRUE );
	fd.setFilter(tr( "Fung-Calc Files (*.fgc)" ));
	#endif

	fd.setCaption(tr( "Select file..." ));
	if (fd.exec() == QDialog::Accepted)
	{
		QString file = fd.selectedFile();        
		load(file);
		updateToolBar(0);
	}
}

void Fung_main::filePrint()
{
	CommonGraph* graph = getGraph();
	if (graph != 0)
		graph->print( printer );
}

void Fung_main::fileSave()
{
	#ifdef KDE_APP
	KFileDialog fd( QString::null, "*.fgc|Fung-Calc Files (*.fgc)", this, "file dialog", TRUE );
	fd.setMode( KFile::File );
	#else
	QFileDialog fd( this, "file dialog", TRUE );
	fd.setFilter( tr("Fung-Calc Files (*.fgc)") );
	fd.setMode( QFileDialog::AnyFile );
	#endif

	fd.setCaption( tr("Save as...") );

	if (fd.exec() == QDialog::Accepted)
	{
		QString file(fd.selectedFile());
		if ( static_cast<unsigned int>(file.find(".fgc")) != file.length()-4 )
			file.append(".fgc");
		std::ofstream fileOutput(file);
		
		static_cast<CommonGraphMain*>(ws->activeWindow())->save(&fileOutput);
	
		fileOutput.close();
	}
}

void Fung_main::fileSaveAll()
{
	#ifdef KDE_APP
	KFileDialog fd( QString::null, "*.fgc|Fung-Calc Files (*.fgc)", this, "file dialog", TRUE );
	fd.setMode( KFile::File );
	#else
	QFileDialog fd( this, "file dialog", TRUE );
	fd.setFilter( tr("Fung-Calc Files (*.fgc)") );
	fd.setMode( QFileDialog::AnyFile );
	#endif

	fd.setCaption( tr("Save as...") );

	if (fd.exec() == QDialog::Accepted)
	{
		QString file(fd.selectedFile());
		if ( static_cast<unsigned int>(file.find(".fgc")) != file.length()-4 )
			file.append(".fgc");
		std::ofstream fileOutput(file);

		QWidgetList list = ws->windowList(); //((QApplication*)this->parent())->allWidgets();
		QWidgetListIt it( list );
		QWidget * w;
		while ( (w=it.current()) != 0 )
		{
			++it;
                        qDebug(w->name());
			static_cast<CommonGraphMain*>(w)->save(&fileOutput);                        
		}

		fileOutput.close();
	}
}

void Fung_main::exportAsImage()
{
	CommonGraph *graph;
	if ((graph = getGraph()) != 0 )
	{
		#ifdef KDE_APP
		KFileDialog fd( 
			QString::null,
				"*.png|PNG (*.png)\n*.xpm|X PixMap (*.xpm)\n*.jpeg *.jpg|JPEG (*.jpg,*.jpeg)\n*.bmp|Windows Bitmap (*.bmp)"
			, this, "file dialog", TRUE );
		fd.setMode( KFile::File );
		#else
		QFileDialog fd( this, "file dialog", TRUE );
		fd.setMode( QFileDialog::AnyFile );
		fd.addFilter( "XPM (*.xpm)" );
		fd.addFilter( "BMP (*.bmp)" );
		#ifdef HAVE_LIBJPEG
		fd.addFilter( "JPEG (*.jpg,*.jpeg)" );
		#endif //HAVE_LIBJPEG
		fd.addFilter( "PNG (*.png)" );
		#endif

		fd.setCaption( tr("Export As...") );

		if (fd.exec() == QDialog::Accepted)
		{
			#ifdef KDE_APP
			QString suffix(fd.currentFilter());
			suffix = suffix.mid(2,4).upper();
			#else
			QString suffix(fd.selectedFilter());
			suffix.truncate(4);
			suffix = suffix.stripWhiteSpace();
			#endif

			QString file = fd.selectedFile();
			if ( static_cast<unsigned int>(file.find("."+suffix.lower())) != file.length()-(suffix.length()+1) )
				file.append("."+suffix.lower());
			graph->exportAsImage(file,suffix);
		}
	}
}

void Fung_main::windowClose()
{
	if (ws->activeWindow() != 0)
		ws->activeWindow()->close();
}

void Fung_main::windowCloseAll()
{
	QWidgetList windows = ws->windowList();
	for (unsigned int i=0; i<windows.count(); i++)
		windows.at(i)->close();
}

void Fung_main::toggleGrid(bool state)
{
	CommonGraph *graph;
	if ((graph = getGraph()) != 0 )
		graph->setGrid(state);
}

void Fung_main::toggleAxis(bool state)
{
	CommonGraph *graph;
	if ((graph = getGraph()) != 0 )
		graph->setAxis(state);
}

void Fung_main::toggleDrawConnected(bool state)
{
	CommonGraph *graph;
	if ((graph = getGraph()) != 0 )
		graph->setDrawConnected(state);
}

void Fung_main::toggleTrace(bool state)
{
	CommonGraph *graph;
	if ((graph = getGraph()) != 0 )
		graph->setTrace(state);
}

void Fung_main::toggleSnap(bool state)
{
	CommonGraph *graph;
	if ((graph = getGraph()) != 0 )
		graph->setSnapToGrid(state);
}

void Fung_main::toggleScale(bool state)
{
	CommonGraph *graph;
	if ((graph = getGraph()) != 0 )
		graph->setScale(state);
}

void Fung_main::setAxisColor()
{
	CommonGraph* graph = getGraph();
	if (graph != 0)
	{
		QColor cc = QColorDialog::getColor(graph->axisColor());
		if (cc.isValid())
			graph->setAxisColor(cc);
	}
}

void Fung_main::setGridColor()
{
	CommonGraph* graph = getGraph();
	if (graph != 0)
	{
		QColor cc = QColorDialog::getColor(graph->gridColor());
		if (cc.isValid())
			graph->setGridColor(cc);
	}
}

void Fung_main::setBackgroundColor()
{
	CommonGraph* graph = getGraph();
	if (graph != 0)
	{
		QColor cc = QColorDialog::getColor(graph->backgroundColor());
		if (cc.isValid())
			graph->setBackgroundColor(cc);
	}
}

void Fung_main::setScaleColor()
{
	CommonGraph* graph = getGraph();
	if (graph != 0)
	{
		QColor cc = QColorDialog::getColor(graph->scaleColor());
		if (cc.isValid())
			graph->setScaleColor(cc);
	}
}

void Fung_main::setZoomStd()
{
	CommonGraph* graph = getGraph();
	if (graph != 0)
		graph->zoomStd();
}

void Fung_main::setZoomBox()
{
	CommonGraph* graph = getGraph();
	if (graph != 0)
		graph->zoomBox();
}

void Fung_main::setZoomStat()
{
	CommonGraph* graph = getGraph();
	if (graph != 0)
		graph->zoomStat();
}

void Fung_main::setZoomTrig()
{

}

void Fung_main::setZoomIn()
{
	CommonGraph* graph = getGraph();
	if (graph != 0)
		graph->zoomIn();
}

void Fung_main::setZoomOut()
{
	CommonGraph* graph = getGraph();
	if (graph != 0)
		graph->zoomOut();
}

void Fung_main::setZoomFactor()
{
	ZoomFactorDialog zoomFactorDialog(this,"zoomFactorDialog");
	CommonGraph* graph = getGraph();
	if (graph != 0)
	{
		zoomFactorDialog.setXZoomFactor(graph->getXZoomFactor());
		zoomFactorDialog.setYZoomFactor(graph->getYZoomFactor());
		if (zoomFactorDialog.exec() == QDialog::Accepted)
		{
			graph->setXZoomFactor(zoomFactorDialog.getXZoomFactor());
			graph->setYZoomFactor(zoomFactorDialog.getYZoomFactor());
		}
	}
}

void Fung_main::setDegrees()
{
	//radiansAction->setOn(false);
	//degreesAction->setOn(true);
	CommonGraph* graph = getGraph();
	if (graph != 0)
		graph->setAngle(0);
}

void Fung_main::setRadians()
{
	//radiansAction->setOn(true);
	//degreesAction->setOn(false);
	CommonGraph* graph = getGraph();
	if (graph != 0)
		graph->setAngle(1);
}

CommonGraph* Fung_main::getGraph(bool warning)
{
	if (!ws->activeWindow())
	{
		if (warning){QMessageBox::information(this,tr("Error"),tr("No graph opened.  Click \'New\' to create a new graph."));}
		return 0;
	}

	if (!ws->activeWindow()->children()) //not sure about this...
		return 0;

	//There's got to be a better way to do this...

	QObjectList *list = const_cast<QObjectList*>(ws->activeWindow()->children());
	for ( QObject * widget = list->first(); widget; widget = list->next() )
	{
		CommonGraph *c = dynamic_cast<CommonGraph*>(widget);
		if (c)
			return c;
	}

	qDebug("Fung_main::getGraph(bool): You shouldn't be seeing this message!");
	return 0; //error if control reaches here
}

void Fung_main::helpAbout()
{
	#ifdef KDE_APP
	KAboutApplication about(KGlobal::instance()->aboutData());
	about.exec();
	#else
	QMessageBox::about(this,
		tr("About..."),
		QString(tr("Fung-Calc v%1 by Jason Kivlighn (Fungmeista) \n"
							"Function Parser by Juha Nieminen (warp@iki.fi)\n"
							"Brian - Ideas, advice, GUI design\n\n"
							"Please e-mail me at mizunoami44@users.sourceforge.net \n"
							"if you have any questions or comments. \n\n"
							"For more information and/or documentation, visit us \n"
							"at http://fung-calc.sourceforge.net \n\n"
							"(C) 2002-2003, Fungmeista")).arg(VERSION));
	#endif
}

void Fung_main::windowTile()
{
	ws->tile();
}

void Fung_main::windowCascade()
{
	ws->cascade();
	QWidgetList list = ws->windowList();
	for (unsigned int i=0; i<list.count(); i++)
		list.at(i)->resize(525,625);
}

void Fung_main::doDistance()
{
	CommonGraph *graph;
	if ((graph = getGraph()) != 0 )
	{
		if (graph->execMathFunction("Distance") == -1)
			QMessageBox::information(this, tr("Error"), QString(tr("Enter expression!")));
	}
}

void Fung_main::doArcLength()
{
	CommonGraph *graph;
	if ((graph = getGraph()) != 0 )
	{
		if (graph->execMathFunction("Arc Length") == -1)
			QMessageBox::information(this, tr("Error"), QString(tr("Enter expression!")));
	}
}

void Fung_main::doDefiniteIntegral()
{
	CommonGraph *graph;
	if ((graph = getGraph()) != 0 )
	{
		if (graph->execMathFunction("Definite Integral") == -1)
			QMessageBox::information(this, tr("Error"), QString(tr("Enter expression!")));
	}
}

void Fung_main::doMaximum()
{
	CommonGraph *graph;
	if ((graph = getGraph()) != 0 )
	{
		if (graph->execMathFunction("Maximum") == -1)
			QMessageBox::information(this, tr("Error"), QString(tr("Enter expression!")));
	}
}

void Fung_main::doMinimum()
{
	CommonGraph *graph;
	if ((graph = getGraph()) != 0 )
	{
		if (graph->execMathFunction("Minimum") == -1)
			QMessageBox::information(this, tr("Error"), QString(tr("Enter expression!")));
	}
}

void Fung_main::doIntersection()
{
	CommonGraph *graph;
	if ((graph = getGraph()) != 0 )
	{
		if (graph->execMathFunction("Intersection") == -1)
			QMessageBox::information(this, tr("Error"), QString(tr("Enter expression!")));
	}
}

void Fung_main::doZero()
{
	CommonGraph *graph;
	if ((graph = getGraph()) != 0 )
	{
		if (graph->execMathFunction("Zero") == -1)
			QMessageBox::information(this, tr("Error"), QString(tr("Enter expression!")));
	}
}

void Fung_main::doTangent()
{
	CommonGraph *graph;
	if ((graph = getGraph()) != 0 )
	{
		if (graph->execMathFunction("Tangent") == -1)
			QMessageBox::information(this, tr("Error"), QString(tr("Enter expression!")));
	}
}

void Fung_main::doMeanValue()
{
	CommonGraph *graph;
	if ((graph = getGraph()) != 0 )
	{
		if (graph->execMathFunction("Mean Value") == -1)
			QMessageBox::information(this, tr("Error"), QString(tr("Enter expression!")));
	}
}

void Fung_main::value()
{
/*	CommonGraph *graph = getGraph();
	Expression ex;
	ExpressionGraph *bg = dynamic_cast<ExpressionGraph*>(graph);
	if (bg)
		ex = bg->getExpression(graph->getCurrentExpressionIndex());
	else
	{
		GLExpressionGraph *gl_bs = dynamic_cast<GLExpressionGraph*>(graph);
		if( gl_bs )
			ex = bg->getExpression(graph->getCurrentExpressionIndex());
	}
	
	ValueDialog d(ex);
*/}


void Fung_main::showUsage()
{
	usageWindow->show();
}

/******************** GLBasicGraph only ******************/
void Fung_main::autoRotateX(bool b)
{
	#ifdef GLGRAPH
	static_cast<GLBasicGraph*>(getGraph())->autoRotateX(b);
	#endif //GLGRAPH
}

void Fung_main::autoRotateY(bool b)
{
	#ifdef GLGRAPH
	static_cast<GLBasicGraph*>(getGraph())->autoRotateY(b);
	#endif //GLGRAPH
}

void Fung_main::autoRotateZ(bool b)
{
	#ifdef GLGRAPH
	static_cast<GLBasicGraph*>(getGraph())->autoRotateZ(b);
	#endif //GLGRAPH
}

void Fung_main::autoRotateAll()
{
	#ifdef GLGRAPH
	RotateXAction->setOn(true);
	static_cast<GLBasicGraph*>(getGraph())->autoRotateX(true);
	RotateYAction->setOn(true);
	static_cast<GLBasicGraph*>(getGraph())->autoRotateY(true);
	RotateZAction->setOn(true);
	static_cast<GLBasicGraph*>(getGraph())->autoRotateZ(true);
	#endif //GLGRAPH
}

void Fung_main::cancelRotate()
{
	#ifdef GLGRAPH
	RotateXAction->setOn(false);
	static_cast<GLBasicGraph*>(getGraph())->autoRotateX(false);
	RotateYAction->setOn(false);
	static_cast<GLBasicGraph*>(getGraph())->autoRotateY(false);
	RotateZAction->setOn(false);
	static_cast<GLBasicGraph*>(getGraph())->autoRotateZ(false);
	#endif //GLGRAPH
}

void Fung_main::exportToDXF()
{
	#ifdef GLGRAPH
	CommonGraph *graph;
	if ((graph = getGraph()) != 0 )
	{
		GLExpressionGraph * gl_exp_gr = dynamic_cast<GLExpressionGraph*>(graph);
		if (gl_exp_gr)
		{
			#ifdef KDE_APP
			KFileDialog fd( QString::null, "*.dxf *.DXF|Drawing Exchange Format Files (*.dxf, *.DXF)", this, "file dialog", TRUE );
			fd.setMode( KFile::File );
			#else
			QFileDialog fd( this, "file dialog", TRUE );
			fd.setMode( QFileDialog::AnyFile );
			fd.addFilter( "DXF (*.dxf)" );
			#endif

			fd.setCaption( tr("Export As...") );

			if (fd.exec() == QDialog::Accepted)
			{
				#ifdef KDE_APP
				QString suffix(fd.currentFilter());
				suffix = suffix.mid(2,3);
				#else
				QString suffix(fd.selectedFilter());
				suffix.truncate(4);
				suffix = suffix.stripWhiteSpace();
				#endif

				QString file = fd.selectedFile();
				if ( static_cast<unsigned int>(file.find("."+suffix.lower())) != file.length()-(suffix.length()+1) )
					file.append("."+suffix.lower());
				gl_exp_gr->exportToDXF(file.latin1());
			}
		}
	}
	#endif //GLGRAPH
}
/******************end GLBasicGraph only *****************/

/******************** Animator only ******************/
void Fung_main::play()
{
	CommonGraph *graph = getGraph();
	Animator *animator;
	if ( (animator = dynamic_cast<Animator*>(graph)) != 0 )
		animator->play();
}

void Fung_main::stop()
{
	CommonGraph *graph = getGraph();
	Animator *animator;
	if ( (animator = dynamic_cast<Animator*>(graph)) != 0 )
		animator->stop();   
}

void Fung_main::pause()
{
	CommonGraph *graph = getGraph();
	Animator *animator;
	if ( (animator = dynamic_cast<Animator*>(graph)) != 0 )
		animator->pause();   
}

void Fung_main::setAnimationFrame( int n )
{
	CommonGraph *graph = getGraph();
	Animator *animator;
	if ( (animator = dynamic_cast<Animator*>(graph)) != 0 )
		animator->setAnimationFrame(n);      
}

void Fung_main::showAnimationParams()
{
	CommonGraph *graph = getGraph();
	Animator *animator;
	if ( (animator = dynamic_cast<Animator*>(graph)) != 0 )
	{
		AnimationParamsDialog dialog(this);
		dialog.setGraph(animator);
		if ( dialog.exec() == QDialog::Accepted )
		{
			AnimationSlider->setMinValue( static_cast<int>( ROUND(dialog.min()) ) );
			AnimationSlider->setMaxValue( static_cast<int>( ROUND(dialog.max()) ) );
			//AnimationSlider->setLineStep( static_cast<int>( ROUND(dialog.step()) ) );
		}
	}
}

void Fung_main::animationValueChanged( double d )
{
	AnimationSlider->setValue(static_cast<int>(ROUND(d)));
}

void Fung_main::displayNValue( bool b )
{
	CommonGraph *graph = getGraph();
	Animator *animator;
	if ( (animator = dynamic_cast<Animator*>(graph)) != 0 )
		animator->setDisplayAnimatorValue(b);      
}
/****************** end Animator only *****************/

