/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Wed May  1 16:37:08 PDT 2002
    copyright            : (C) 2002-03 by Fungmeista
    email                : mizunoami44@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifdef KDE_APP
#include <kapplication.h>
#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <klocale.h>
#else
#include <qapplication.h>
#endif //KDE_APP

#include <qfont.h>
#include <qstring.h>
#include <qtextcodec.h>
#include <qtranslator.h>

#include <iostream>

#include "interface.h"
#include "config.h"

#ifdef KDE_APP
static KCmdLineOptions options[] =
{
    { "+file1 file2 ...",          I18N_NOOP("File(s) to open"),                            0 },
    { "verbose",                   I18N_NOOP("Display extra output"),                       0 },
    { "translation-dir <dir>",     I18N_NOOP("Override default path to translation files"), 0 },
    { 0,0,0 }
};
#endif

bool verbose = false;

void message_output( QtMsgType type, const char *msg );

int main(int argc, char *argv[])
{
	#ifdef KDE_APP
	static const char *description = I18N_NOOP("Fung-Calc - An advanced, yet easy to use, graphing calculator");
	KAboutData aboutData("fung-calc", I18N_NOOP("Fung-Calc"),
                       VERSION, description, KAboutData::License_GPL,
                       I18N_NOOP("(c) 2002-2003 Fungmeista"));
	aboutData.addAuthor("Jason Kivlighn (Fungmeista)", I18N_NOOP("Creator and main programmer"), "mizunoami44@users.sourceforge.net");
	
	aboutData.addCredit("Juha Nieminen", I18N_NOOP("Function parser"), "warp@iki.fi");
	aboutData.addCredit("Brian Kivlighn", I18N_NOOP("Ideas, advice, GUI modifications"), "");

	KCmdLineArgs::init(argc, argv, &aboutData);
	KCmdLineArgs::addCmdLineOptions( options );

	KApplication a;
	#else
	QApplication a(argc, argv);
	#endif //KDE_APP
        
	qInstallMsgHandler( message_output );
	
	/** @todo Add a splashscreen at startup? */
	
	Fung_main *fung_calc = new Fung_main();
	
	QString translation_dir;

	#ifndef KDE_APP                
	for (int i=1; i<a.argc(); i++)
	{
		if ( QString( a.argv()[i] ).mid(0,18) == "--translation-dir=" )
			translation_dir = QString( a.argv()[i] ).mid( 18, (18 - strlen(a.argv()[i])) );
                else if ( strcmp ( a.argv()[i], "--verbose" ) == 0 || strcmp ( a.argv()[i], "-v" ) == 0 )
                {
			verbose = true;                
                }
                else if ( strcmp ( a.argv()[i], "--version" ) == 0 )
                {
			std::cout<<"Fung-Calc "<<VERSION<<std::endl;
			std::cout<<std::endl;
			std::cout<<"This is free software; see the source for copying conditions."<<std::endl;

			exit(0);                
                }                 
		else if ( strcmp( a.argv()[i], "--help" ) == 0 )
		{
			std::cout<<"Usage: fung-calc [options] file1 file2 ..."<<std::endl;
			std::cout<<std::endl;
                        
			std::cout<<"Options:"<<std::endl<<std::endl;
                        
			std::cout<<" --translation-dir=DIR     Override default path to translation files"<<std::endl;
			std::cout<<" --verbose, -v             Verbose mode"<<std::endl;
			std::cout<<" --version                 Display version number and exit"<<std::endl;                                             
			std::cout<<std::endl;

			std::cout<<"Arguments:"<<std::endl<<std::endl;
			std::cout<<" file1 file2 ...           Files to open"<<std::endl;

			exit(0);
		}
		else
		{
			QString file = QString(a.argv()[i]);                        
			qDebug("Loading "+file+"...");
			int ret = fung_calc->load(file);
			if (!ret)
				qDebug("Successfully loaded file.");
			else
				qDebug("Error: Unable to load.");
		}
	}
	#else //is KDE_APP
	KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

	if (args->isSet("verbose"))
		verbose = true;
	
	translation_dir = args->getOption("translation-dir");
	
	for( int i = 0; i < args->count(); i++ )
	{
		QString file = args->arg(i);
		int ret = fung_calc->load(file);
		if (!ret)
			qDebug("Successfully loaded file: %s",file.latin1());
		else
			qDebug("Error: Unable to load: %s",file.latin1());
	}

	args->clear();
	#endif

        
        if ( translation_dir.isNull() )
		translation_dir = QString(INSTALL_DIR) + QString("/share/fung-calc/translations");

	QTranslator tor( 0 );

	qDebug( QString("Loading locale: ") + QTextCodec::locale() + QString(" from ") + translation_dir );

	tor.load( QString("fung-calc.")+QTextCodec::locale() + QString(".qm") , translation_dir );

	a.installTranslator( &tor );

	a.setMainWidget(fung_calc);

	fung_calc->show();
	return a.exec();
}

void message_output( QtMsgType type, const char *msg )
{
	if (verbose)
	{
		switch ( type ) 
		{
			case QtDebugMsg:
				fprintf( stderr, "%s\n", msg );
				break;
			case QtWarningMsg:
				fprintf( stderr, "Warning: %s\n", msg );
				break;
			case QtFatalMsg:
				fprintf( stderr, "Fatal: %s\n", msg );
				//abort();
		}
	}
}
